@@ -12,7 +12,7 @@ fi
 export PAGER=cat
 
 # Matt might update this when confident of purity since the indicated date.
-export SINCE=2015-04-10
+export SINCE=2016-01-11
 
 export basecmd="\
 git log \
@@ -26,10 +26,13 @@ git fetch --all
 echo "-----"
 
 $basecmd \
+  --author='Matt Kaufmann <kaufmann@horatio-168.cs.utexas.edu>' \
   --author='Matt Kaufmann <kaufmann@cs.utexas.edu>' \
   --author='MattKaufmann <matthew.j.kaufmann@gmail.com>' \
   --author='kaufmann <kaufmann@unknown58b035fde782.attlocal.net>' \
   --author='Matt Kaufmann <kaufmann@matts-mbp.attlocal.net>' \
+  --author='Matt Kaufmann <kaufmann@Matts-MBP.attlocal.net>' \
+  --author="Matt Kaufmann <kaufmann@Matts-MacBook-Pro.local>" \
   | grep -v '^[a-z0-9]\{40\}' \
   | grep -v '^books/' \
   > /tmp/git-log-matt.txt
