@@ -7,10 +7,6 @@
 
 (in-package "CGEN")
 
-;Useful Macros for concise/convenient code.
-(include-book "std/util/bstar" :dir :system)
-;(include-book "basis")
-
 (include-book "type")
 
 (include-book "simple-graph-array")
@@ -21,14 +17,14 @@
 
 ;; The following 2 functions need to be revisited and rewritten if necessary
 (defun let-binding->dep-graph-alst (vt-lst ans)
-  "Walk down the var-term list vt-lst and add edges. 
+  "Walk down the var-term list vt-lst and add edges.
    Build graph alist in ans"
   (if (endp vt-lst)
       ans
     (b* (((list var term) (car vt-lst))
          (fvars (all-vars term)));only non-buggy for terms
-   
-      (let-binding->dep-graph-alst 
+
+      (let-binding->dep-graph-alst
        (cdr vt-lst)
        (cgen::union-entry-in-adj-list var fvars ans)))))
 
@@ -47,7 +43,7 @@
                   :mode :program))
   (b* ((vars (union-eq (all-vars1-lst (strip-cars vt-lst) nil)
                        (all-vars1-lst (strip-cdrs vt-lst) nil)))
-       (dep-g (let-binding->dep-graph-alst vt-lst 
+       (dep-g (let-binding->dep-graph-alst vt-lst
                                            (cgen::make-empty-adj-list vars)))
        (sorted-vars (cgen::approximate-topological-sort dep-g debug-flag)))
     (get-ordered-alst (reverse sorted-vars) vt-lst nil)))
@@ -59,7 +55,7 @@
                     (W2 (CONS W4 X3))
                     (Z (CONS Y X3))
                     (Y (CONS X X3))
-                    (W (CONS Z Y))) 
+                    (W (CONS Z Y)))
                   nil)
 ||#
 
@@ -112,12 +108,12 @@
        (wrld (w state))
        (pform (acl2::prettyify-clause (list term) nil wrld))
        ((mv phyps pconcl)  (mv (get-hyps pform) (get-concl pform)))
-       
-       ((er hyps) (acl2::translate-term-lst phyps 
-                                            t nil t str wrld state)) 
+
+       ((er hyps) (acl2::translate-term-lst phyps
+                                            t nil t str wrld state))
        ((er concl) (acl2::translate pconcl t nil t str wrld state)))
     (mv hyps concl state)))
-  
+
 
 ;-- 2 utility functions by Matt, to get elided variable replaced term information --
 (defun get-dest-elim-replaced-terms (elim-sequence)
@@ -141,21 +137,21 @@
          )
       (cond ((eq proc 'acl2::generalize-clause)
 ;Generalize
-             (let ((ans1 
+             (let ((ans1
                     (list-up-lists
                      (acl2::tagged-object 'acl2::terms ttree)
                      (acl2::tagged-object 'acl2::variables ttree))))
-               (collect-replaced-terms 
+               (collect-replaced-terms
                 (cdr hist) (append ans1 ans))))
             ((eq proc 'acl2::eliminate-destructors-clause)
 ;Destructor elimination
-             (let* ((elim-sequence 
+             (let* ((elim-sequence
                      (acl2::tagged-object 'acl2::elim-sequence ttree))
                     (ans1 (get-dest-elim-replaced-terms elim-sequence)))
                (collect-replaced-terms
                 (cdr hist) (append ans1 ans))))
 ;Else (simplification and etc etc)
-            (t (let* ((binding-lst 
+            (t (let* ((binding-lst
                        (acl2::tagged-object 'acl2::binding-lst ttree))
 ;TODO: Modified! Show (convert-conspairs-to-listpairs binding-lst) = (listlis (strip-cars binding-lst) (strip-cdrs binding-lst))
                       (ans1 (list-up-lists (strip-cars binding-lst) (strip-cdrs binding-lst))))
@@ -165,12 +161,12 @@
 
 
 
-(defun cgen-search-clause (cl name 
-                              ens hist 
+(defun cgen-search-clause (cl name
+                              ens hist
                               elim-elided-var-map
                               cgen-state
                               vl ctx state)
-   "helper function for test-checkpoint. It basically sets up 
+   "helper function for test-checkpoint. It basically sets up
    everything for the call to csearch."
   (declare (xargs :stobjs (state) :mode :program))
   (b* (((mv erp type-alist &)
@@ -187,24 +183,22 @@
 ;the current subgoal i.e. cl
        (vars (all-vars1-lst cl '()))
        (vt-acl2-alst (if erp ;contradiction
-                         (pairlis$ vars (make-list (len vars)
-                                                   :initial-element 
-                                                   (list 'ACL2::ALL)))
+                         (make-dumb-type-alist vars)
                        (decode-acl2-type-alist type-alist vars)))
        ((mv hyps concl) (clause-mv-hyps-concl cl))
 ;       (wrld (w state))
        (elided-var-map (append (collect-replaced-terms hist nil)
                                elim-elided-var-map))
        (- (cw? (verbose-stats-flag vl)
-               "~|CEgen/Verbose: Search clause with elide-map ~x0. ~|" 
+               "~|CEgen/Verbose: Search clause with elide-map ~x0. ~|"
                elided-var-map))
 
        ;; Ordering is necessary to avoid errors in printing top-level cts
 
        (ord-elide-map (do-let*-ordering elided-var-map (system-debug-flag vl)))
        (tau-interval-alist (tau-interval-alist-clause cl vars ens))
-       ((mv erp cgen-state state)  (cgen-search-fn name hyps concl 
-                                                   vt-acl2-alst tau-interval-alist ord-elide-map 
+       ((mv erp cgen-state state)  (cgen-search-fn name hyps concl
+                                                   vt-acl2-alst tau-interval-alist ord-elide-map
                                                    NIL
                                                    cgen-state
                                                    ctx state)))
@@ -227,7 +221,7 @@
 ;; stobjs in their arguments and return values.
 
 (defun unsupported-fns (fns wrld)
-  "gather functions that 
+  "gather functions that
 1. take stobjs as args
 2. constrained (encapsulate) and no attachment"
   (if (endp fns)
@@ -235,13 +229,13 @@
     (let* ((fn (car fns))
            (constrainedp (acl2-getprop fn 'acl2::constrainedp wrld :default nil))
            (att (acl2-getprop fn 'acl2::attachment wrld :default nil)))
-          
+
       (if (or (or-list (acl2::stobjs-in fn wrld))
               (and constrainedp
                    (null att)))
           (cons fn (unsupported-fns (cdr fns) wrld))
         (unsupported-fns (cdr fns) wrld)))))
-  
+
 
 
 
@@ -258,8 +252,8 @@
        (- (cw? (and unsupportedp (verbose-flag vl))
 "~|CEgen/Note: Skipping testing completely, since some functions in this conjecture either take stobj arguments or are constrained without an attachment.~%")))
     (mv all-execp unsupportedp)))
-       
-     
+
+
 
 (defun not-equivalid-p (processor-hist)
   "proof has wandered into non-equi-valid path"
@@ -277,16 +271,16 @@
        (hyps (if (eq hyp 't) '() (acl2::expand-assumptions-1 hyp)))
        (vars (vars-in-dependency-order hyps concl vl (w state)))
        (d-typ-al (dumb-type-alist-infer
-                   (cons (dumb-negate-lit concl) hyps) vars 
+                   (cons (dumb-negate-lit concl) hyps) vars
                    vl wrld))
        (cgen-state (cput top-vt-alist d-typ-al))
        (- (cw? (system-debug-flag vl)
                "~|CEgen/Sysdebug: update-gcs%-top-level : ~x0 dumb top vt-alist: ~x1 ~|"
                term d-typ-al)))
-;   in 
+;   in
     cgen-state))
 
-        
+
 ;; The following function implements a callback function (computed hint)
 ;; which calls the counterexample generation testing code. Thus the
 ;; the so called "automated" combination of testing and theorem proving
@@ -298,7 +292,7 @@
   "?: This function is a callback called via an override hint +
 backtrack no-op hint combination.  On SUBGOALS that are not
 checkpoints it is a no-op. On checkpoints it calls the main
-cgen/testing procedure. 
+cgen/testing procedure.
 
 Note that this (observer) hint combination means that when this
 callback function is called, that particular processor had a HIT and
@@ -319,22 +313,22 @@ then it returns (value '(:do-not '(acl2::generalize)
 ; INVARIANT - no new prove call is made during the computation of this
 ; function (this is very important, but now I can relax this invariant,
 ; with the introduction of post and pre functions at event level)
-  ;; (acl2::with-timeout1 
+  ;; (acl2::with-timeout1
   ;;  (acl2s-defaults :get subgoal-timeout)
    (b* (
 ;TODObug: test? defaults should be the one to be used
        (vl (acl2s-defaults :get verbosity-level))
-       
+
        ((unless (and (f-boundp-global 'cgen-state state)
-                     (cgen-state-p (@ cgen-state)))) 
+                     (cgen-state-p (@ cgen-state))))
         ;; Ignore contexts where cgen-state is not well-formed. TODO SHouldnt we give a message here at least for regression/testing?
         (prog2$
          (cw? (debug-flag vl)
               "~|CEgen/Debug: test-checkpoint -- cgen-state malformed; skip cgen/testing...~%")
          (value nil)))
-       
 
-       ((mv all-execp unsupportedp) 
+
+       ((mv all-execp unsupportedp)
         (cgen-exceptional-functions cl vl (w state)))
 ;27 June 2012 - Fixed bug, in CCG, some lemmas are non-executable, since they
 ;involve calling the very function being defined. We should avoid testing
@@ -344,26 +338,26 @@ then it returns (value '(:do-not '(acl2::generalize)
        ((when unsupportedp) (value nil))
 
        (- (cw? (system-debug-flag vl)
-               "test-checkpoint : id=~x0 processor=~x1 ctx= ~x2 hist-len=~x3~|" 
-               id processor ctx ;(acl2::prettyify-clause cl nil (w state)) 
+               "test-checkpoint : id=~x0 processor=~x1 ctx= ~x2 hist-len=~x3~|"
+               id processor ctx ;(acl2::prettyify-clause cl nil (w state))
                (len hist)))
- 
+
        ((unless (member-eq processor
                            '(;acl2::preprocess-clause
                              ;;acl2::simplify-clause
-                             acl2::settled-down-clause 
+                             acl2::settled-down-clause
                              acl2::eliminate-destructors-clause
                              acl2::fertilize-clause
                              acl2::generalize-clause
                              acl2::eliminate-irrelevance-clause
-                             acl2::push-clause)))  
+                             acl2::push-clause)))
 ; NOTE: I can also use (f-get-global 'checkpoint-processors state)
         (value nil));no-op
-       
+
        (name (acl2::string-for-tilde-@-clause-id-phrase id))
        (wrld (w state))
        (cgen-state (@ cgen-state))
-       (pspv-user-supplied-term (acl2::access acl2::prove-spec-var 
+       (pspv-user-supplied-term (acl2::access acl2::prove-spec-var
                                      pspv :user-supplied-term))
        (cgen-state (if (eq (cget user-supplied-term) :undefined)
                        (update-cgen-state-givens/callback pspv-user-supplied-term)
@@ -372,27 +366,27 @@ then it returns (value '(:do-not '(acl2::generalize)
        ((unless (equal user-supplied-term pspv-user-supplied-term))
         (prog2$ ;Invariant
          (cw? (verbose-flag vl)
-              "~|CEgen/Note: We encountered a new goal ~x0, in the course of testing ~x1. ~ 
-Nested testing not allowed! Skipping testing of new goal...~%" 
+              "~|CEgen/Note: We encountered a new goal ~x0, in the course of testing ~x1. ~
+Nested testing not allowed! Skipping testing of new goal...~%"
               (acl2::prettyify-clause (list pspv-user-supplied-term) nil wrld)
               (acl2::prettyify-clause (list user-supplied-term) nil wrld))
          (value nil)))
-        
+
        (- (cw? (verbose-stats-flag vl)
                "~%~%~|CEgen/Note: At checkpoint ~x0 ~x1~|" name processor))
        (ens (acl2::access acl2::rewrite-constant
-                          (acl2::access 
+                          (acl2::access
                            acl2::prove-spec-var pspv :rewrite-constant)
                           :current-enabled-structure))
-       
+
   ;     (abo? (not-equi-valid-p (cget processor-hist))) ;all bets off
        ((mv & cgen-state state) (cgen-search-clause cl name
-                                                    ens hist 
+                                                    ens hist
                                                     ;;abo? processor
                                                     '() ;deprecated arg for manual elim elided var map
                                                     cgen-state
                                                     vl ctx state))
-       
+
 ; Assumption Jan 6th 2013 (check with Matt)
 ; We only arrive here with processor P, if it was a hit i.e if P
 ; is fertilize-clause then cross-fertilization was successful and
@@ -406,8 +400,8 @@ Nested testing not allowed! Skipping testing of new goal...~%"
        (state (f-put-global 'cgen-state cgen-state state)) ;put it back in globals
        (gcs% (cget gcs))
        )
-   
-;  in  
+
+;  in
    (if (or (cget stopping-condition-p)
            (and (> (access gcs% cts) 0)
                 (or abo? (eq processor 'acl2::push-clause))))
@@ -427,9 +421,7 @@ Nested testing not allowed! Skipping testing of new goal...~%"
               ((mv H C) (cgen::clause-mv-hyps-concl gen-cl))
               (vars (vars-in-dependency-order H C vl wrld))
 
-              (vt-alist (pairlis$ vars (make-list (len vars)
-                                                  :initial-element 
-                                                  (list 'ACL2::ALL))))
+              (vt-alist (make-dumb-type-alist vars))
               ;; (term (if (null H)
               ;;           C
               ;;           `(implies (and ,@H) ,C)))
@@ -438,11 +430,11 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 ; hopefully it will not affect any computation based on it, certainly will
 ; not affect all-vars. CHECK! 20th March 2013
               (tau-interval-alist (tau-interval-alist-clause gen-cl vars))
-              ((mv erp res state) (cgen-search-local name H C 
-                                                     type-alist tau-interval-alist 
+              ((mv erp res state) (cgen-search-local name H C
+                                                     type-alist tau-interval-alist
                                                      NIL vt-alist
-                                                     (cget params) 
-                                                     *initial-test-outcomes%* 
+                                                     (cget params)
+                                                     *initial-test-outcomes%*
 ; we dont care about witnesses and the start time and do no accumulation.
                                                      *initial-gcs%*
                                                      ctx state))
@@ -450,13 +442,13 @@ Nested testing not allowed! Skipping testing of new goal...~%"
               ((list & test-outcomes% &) res)
               (num-cts-found (access test-outcomes% |#cts|)))
           (value (if (> num-cts-found 0)
-                     (progn$ 
-                      (cw? (normal-output-flag vl) "~| Generalized subgoal: ~x0~|" 
+                     (progn$
+                      (cw? (verbose-stats-flag vl) "~| Generalized subgoal: ~x0~|"
                            (acl2::prettyify-clause gen-cl nil (w state)))
-                      (cw? (normal-output-flag vl)
+                      (cw? (verbose-stats-flag vl)
                            "~| Counterexample found: ~x0 ~|"
                            (car (access test-outcomes% cts)))
-                      (cw? (normal-output-flag vl) "~| Backtracking...~|")
+                      (cw? (verbose-flag vl) "~| Bad generalization! Backtracking...~|")
                       '(:do-not '(acl2::generalize)
                                 :no-thanks t))
                    nil)))
@@ -483,18 +475,18 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 (defmacro acl2s::enable-acl2s-random-testing ()
 ;; this has to be a makevent because enable-acl2s-random-testing is the
 ;; expansion result of the make-event in set-acl2s-random-testing-enabled
-`(make-event  
-  '(progn 
-     (acl2::add-override-hints 
-      '((list* :backtrack 
+`(make-event
+  '(progn
+     (acl2::add-override-hints!
+      '((list* :backtrack
 ;take parent pspv and hist, not the ones returned by clause-processor
 
-               `(acl2::test-checkpoint acl2::id 
+               `(acl2::test-checkpoint acl2::id
                                        acl2::clause
                                        acl2::clause-list
                                        acl2::processor
 ;TODO:ask Matt about sending parent pspv and hist
-                                       ',acl2::pspv 
+                                       ',acl2::pspv
                                        ',acl2::hist
                                        acl2::ctx
                                        state
@@ -502,10 +494,10 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 
              ;; `(mv-let (erp tval state)
              ;;          (trans-eval
-             ;;           `(test-each-checkpoint ',acl2::id 
-             ;;                                  ',acl2::clause 
-             ;;                                  ',acl2::processor 
-             ;;                                  ',',acl2::pspv 
+             ;;           `(test-each-checkpoint ',acl2::id
+             ;;                                  ',acl2::clause
+             ;;                                  ',acl2::processor
+             ;;                                  ',',acl2::pspv
              ;;                                  ',',acl2::hist state
              ;;                                  ts$)
              ;;           'acl2s-testing ; ctx
@@ -520,15 +512,15 @@ Nested testing not allowed! Skipping testing of new goal...~%"
      )))
 
 (defmacro acl2s::disable-acl2s-random-testing ()
-`(make-event  
+`(make-event
      '(progn
-        (acl2::remove-override-hints 
-         '((list* :backtrack 
-                  `(acl2::test-checkpoint acl2::id 
-                                          acl2::clause 
+        (acl2::remove-override-hints
+         '((list* :backtrack
+                  `(acl2::test-checkpoint acl2::id
+                                          acl2::clause
                                           acl2::clause-list
-                                          acl2::processor 
-                                          ',acl2::pspv 
+                                          acl2::processor
+                                          ',acl2::pspv
                                           ',acl2::hist
                                           acl2::ctx
                                           state
@@ -536,10 +528,10 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 ;take parent pspv and hist, not the ones returned by clause-processor
                  ;; `(mv-let (erp tval state)
                  ;;      (trans-eval
-                 ;;       `(test-each-checkpoint ',acl2::id 
-                 ;;                              ',acl2::clause 
-                 ;;                              ',acl2::processor 
-                 ;;                              ',',acl2::pspv 
+                 ;;       `(test-each-checkpoint ',acl2::id
+                 ;;                              ',acl2::clause
+                 ;;                              ',acl2::processor
+                 ;;                              ',',acl2::pspv
                  ;;                              ',',acl2::hist state
                  ;;                              ts$)
                  ;;       'acl2s-testing ; ctx
@@ -555,14 +547,14 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 
 ;Dont print the "Thanks" message:
 (defmacro acl2::dont-print-thanks-message-override-hint ()
-`(make-event  
-  '(acl2::add-override-hints 
+`(make-event
+  '(acl2::add-override-hints
     '((cond ((or (null acl2::keyword-alist)
                  (assoc-keyword :no-thanks acl2::keyword-alist))
              acl2::keyword-alist)
             (t
              (append '(:no-thanks t) acl2::keyword-alist)))))))
-   
+
 
 
 
@@ -603,6 +595,10 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 
 
 
+
+
+
+
 ; [2014-05-03 Sat] event-stack stores in its entries, either a keyword
 ; :ignore-cgen or the ctx of the current event/form. The former
 ; indicates that cgen has ignored this event, the latter that cgen is
@@ -610,17 +606,17 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 ; conjecture under test. This is crucial to match the actions taken in
 ; initialize-event with actions in finalize-event. At the top-level,
 ; this stack better be empty (how to ensure this invariant?).
-(defun initialize-event-user-cgen (ctx-form body state)
+(defun initialize-event-user-cgen/old (ctx-form body state)
   (declare (xargs :stobjs state
                   :mode :program
                   :verify-guards nil))
-  (declare (ignorable body)) 
+  (declare (ignorable body))
 
   (b* (((unless (and (f-boundp-global 'cgen-state state)
                      (f-boundp-global 'event-stack state)))
         state) ;ignore
        (vl (acl2s-defaults :get verbosity-level))
-  
+
 ; Always push the event into the event-stack for matching with
 ; finalize. The default is to push :ignore-cgen, which is overriden
 ; later, if cgen takes this event to give the conjecture under test.
@@ -630,10 +626,10 @@ Nested testing not allowed! Skipping testing of new goal...~%"
        (ctx (compute-event-ctx ctx-form))
        ((unless (allowed-cgen-event-ctx-p ctx))
         (prog2$
-         (cw? (> vl 8) 
+         (cw? (> vl 8)
               "~|CEgen/Warning: initialize-event -- ctx ~x0 ignored...~%" ctx)
          state))
-       
+
        (cgen-state (@ cgen-state))
        ;TODO: Perhaps just flush cgen-state here. Reason: currently allowed ctx
        ; are not usually nested: i.e Why would someone call thm inside a
@@ -643,20 +639,20 @@ Nested testing not allowed! Skipping testing of new goal...~%"
          (cw? (debug-flag vl)
               "~|CEgen/Warning: initialize-event -- nested event ignored...~%")
          state))
-       
+
        ((mv start state) (acl2::read-run-time state))
        (cgen-state (init-cgen-state/event (acl2s::acl2s-defaults-alist) start ctx))
        (- (cw? (debug-flag vl)
               "~|CEgen/Note: CGEN-STATE initialized for ctx ~x0~%" ctx))
        (state (f-put-global 'cgen-state cgen-state state))
-       (event-stack~ (cons ctx event-stack)) ;overwrite/update 
+       (event-stack~ (cons ctx event-stack)) ;overwrite/update
        (state (f-put-global 'event-stack event-stack~ state)))
     state))
 
 
 (defstub print-testing-summary (* * state) => (mv * * state))
-       
-(defun finalize-event-user-cgen (ctx-form body state)
+
+(defun finalize-event-user-cgen/old (ctx-form body state)
   (declare (xargs :mode :program ;print-testing-summary is program-mode
                   :verify-guards nil :stobjs state))
   (declare (ignorable ctx-form body))
@@ -664,13 +660,13 @@ Nested testing not allowed! Skipping testing of new goal...~%"
                      (f-boundp-global 'event-stack state)))
         state) ;ignore
        (vl (acl2s-defaults :get verbosity-level)) ;todo replace with the one from cgen-state
-  
+
 ; Always pop the ctx from the event-stack (matching with initialize).
        (event-stack (@ event-stack))
        (event-stack~ (cdr event-stack))
        (state (f-put-global 'event-stack event-stack~ state))
 
-       
+
        ((when (eq (car event-stack) :ignore-cgen))
         (prog2$
          (cw? (> vl 8)
@@ -686,13 +682,13 @@ Nested testing not allowed! Skipping testing of new goal...~%"
        (gcs% (cget gcs))
 
        (ctx (compute-event-ctx ctx-form))
-       (print-summary-p (and (cget print-cgen-summary) 
+       (print-summary-p (and (cget print-cgen-summary)
                              (> (access gcs% cts) 0)
 ; dont print at the end of defun/defuns events (any help to CCG by cgen is invisible) TODO
                              (member-eq ctx '(ACL2::THM ACL2::DEFTHM ACL2::VERIFY-GUARDS))))
-       
+
        (- (cw? (system-debug-flag vl) "~|CEgen/SysDebug: Exiting event with gcs% : ~x0. ~ ctx: ~x1 print-cgen-summ : ~x2 ~%" gcs% ctx (cget print-cgen-summary)))
-                        
+
        ((mv & & state) ;ignore error in print function
         (if print-summary-p
             (print-testing-summary cgen-state ctx state) ;state is important also because we call trans-eval inside this fun
@@ -701,8 +697,77 @@ Nested testing not allowed! Skipping testing of new goal...~%"
               "~|CEgen/Note: CGEN-STATE finalized for ctx ~x0~%" ctx))
        (state (f-put-global 'cgen-state nil state))) ;clean up cgen state
     state))
-          
-      
+
+
+
+;; [2015-02-04 Wed] Simplify the pre/post event hooks used by Cgen.
+;; Assume that no event of concern to use will be nested. i.e thm, defthm, verify-guards, test? will never be nested.
+(defun initialize-event-user-cgen (ctx-form body state)
+  (declare (xargs :stobjs state :mode :program :verify-guards nil))
+  (declare (ignorable body))
+; As soon as it sees a testable/cgen-able event it resets/inits the two globals:
+; cgen-state using init-cgen-state/event and event-ctx with ctx-form
+  (b* (((unless (and (f-boundp-global 'cgen-state state)
+                     (f-boundp-global 'event-ctx state)))
+        state) ;ignore
+       (vl (acl2s-defaults :get verbosity-level))
+       (ctx (compute-event-ctx ctx-form))
+       ((unless (allowed-cgen-event-ctx-p ctx))
+        (prog2$
+         (cw? (> vl 8) "~|CEgen/Warning: initialize-event -- ctx ~x0 ignored...~%" ctx)
+         state))
+
+       ((mv start state) (acl2::read-run-time state))
+       (cgen-state (init-cgen-state/event (acl2s::acl2s-defaults-alist) start ctx))
+       (- (cw? (debug-flag vl) "~|CEgen/Note: CGEN-STATE initialized for ctx ~x0~%" ctx))
+       (state (f-put-global 'cgen-state cgen-state state))
+       (state (f-put-global 'event-ctx ctx-form state)))
+    state))
+
+(defun finalize-event-user-cgen (ctx-form body state)
+  (declare (xargs :mode :program ;print-testing-summary is program-mode
+                  :verify-guards nil :stobjs state))
+  (declare (ignorable body))
+; If event-ctx global matches ctx-form, we know (sure??) this is the top cgen event and do the following. (if not matched, we ignore/skip).
+; We reset event-ctx to nil.
+; Finalize cgen-state with end-time, print-summary and reset cgen-state to nil.
+  (b* (((unless (and (f-boundp-global 'cgen-state state)
+                     (f-boundp-global 'event-ctx state)))
+        state) ;ignore
+       (vl (acl2s-defaults :get verbosity-level)) ;todo replace with the one from cgen-state
+       (event-ctx (@ event-ctx))
+       ((unless (equal event-ctx ctx-form)) ;check if it is the matching finalize
+        (prog2$
+         (cw? (> vl 8) "~|CEgen/Warning: finalize-event -- ignore non-top event...~%")
+         state))
+
+       (state (f-put-global 'event-ctx nil state)) ;reset/clean
+
+       ;; symmetric to initialize-event-user-cgen, update end time
+       ((mv end state) (acl2::read-run-time state))
+       (cgen-state (@ cgen-state))
+; removed assert here for valid cgen-state, as there are many events that occur that have nothing to do with testing!
+       (cgen-state (cput end-time end))
+       (state (f-put-global 'cgen-state cgen-state state))
+       (gcs% (cget gcs))
+
+       (ctx (compute-event-ctx ctx-form))
+       (print-summary-p (and (cget print-cgen-summary)
+                             ;(> (access gcs% cts) 0) ;commented out to collect vacuous stats
+; dont print at the end of defun/defuns events (any help to CCG by cgen is invisible) TODO
+                             (member-eq ctx '(ACL2::THM ACL2::DEFTHM ACL2::VERIFY-GUARDS))))
+
+       (- (cw? (system-debug-flag vl) "~|CEgen/SysDebug: Exiting event with gcs% : ~x0. ~ ctx: ~x1 print-cgen-summ : ~x2 ~%" gcs% ctx (cget print-cgen-summary)))
+
+       ((mv & & state) ;ignore error in print function
+        (if print-summary-p
+            (print-testing-summary cgen-state ctx state) ;state is important also because we call trans-eval inside this fun
+          (value nil)))
+       (- (cw? (debug-flag vl)
+              "~|CEgen/Note: CGEN-STATE finalized for ctx ~x0~%" ctx))
+       (state (f-put-global 'cgen-state nil state))) ;clean up cgen state
+    state))
+
 
 
 (defun initialize-event-user-cgen-gv (ctx body state)
@@ -724,9 +789,13 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 ;brr which does not end gracefully (no finalize-event is called).
 ; Actually even simple interrupts, leave the event-stack unfinalized, and hence
 ; we might need to manually flush these globals.
+
+; [2015-02-04 Wed] After the above simplification (disallowing nested
+; cgen-relevant events), the above note is mostly irrelevant.
+
 (defmacro flush ()
 ;; ":Doc-Section cgen
-  
+
 ;;   Flush/Reset the transient Cgen state globals to sane values.~/~/
 
 ;;   ~bv[]
@@ -735,7 +804,7 @@ Nested testing not allowed! Skipping testing of new goal...~%"
 ;;    ~ev[]
 ;; "
 ;flush any transient/polluted globals due to interrupts
-  `(er-progn 
+  `(er-progn
     (assign cgen::cgen-state nil)
-    (assign cgen::event-stack nil)
+    (assign cgen::event-ctx nil)
     (value nil)))
