@@ -6,7 +6,7 @@
 
 (in-package "CGEN"
 )
-(include-book "std/util/bstar" :dir :system)
+
 (include-book "basis")
 (include-book "../defdata/defdata-util")
 
@@ -31,30 +31,30 @@
     ((eq ts-decoded '*TS-IMPROPER-CONS*)         '(acl2s::improper-cons) );;; conses that are not proper
     ((eq ts-decoded '*TS-STRING*)                '(acl2s::string) );;; strings
     ((eq ts-decoded '*TS-CHARACTER*)             '(acl2s::character) );;; characters
-    
+
 ;non-primitive types but defined in ground acl2 theory and base.lisp
     ((eq ts-decoded '*TS-UNKNOWN*)               '(acl2s::all) );should give out error?
     ((eq ts-decoded '*TS-NON-NIL* )              '(acl2s::all-but-nil))
     ((eq ts-decoded '*TS-ACL2-NUMBER*)           '(acl2s::acl2-number) )
     ((eq ts-decoded '*TS-RATIONAL-ACL2-NUMBER*)  '(acl2s::acl2-number) )
     ((eq ts-decoded '*TS-RATIONAL* )             '(acl2s::rational) )
-    ((eq ts-decoded '*TS-TRUE-LIST-OR-STRING*)   '(acl2s::true-list acl2s::string)) 
+    ((eq ts-decoded '*TS-TRUE-LIST-OR-STRING*)   '(acl2s::true-list acl2s::string))
     ((eq ts-decoded '*TS-SYMBOL*   )             '(acl2s::symbol) )
     ((eq ts-decoded '*TS-INTEGER*   )            '(acl2s::integer) )
     ((eq ts-decoded '*TS-NON-POSITIVE-RATIONAL*) '(acl2s::negative-rational '0))
     ((eq ts-decoded '*TS-NON-NEGATIVE-RATIONAL*) '(acl2s::positive-rational '0))
     ((eq ts-decoded '*TS-NEGATIVE-RATIONAL* )    '(acl2s::negative-rational) )
     ((eq ts-decoded '*TS-POSITIVE-RATIONAL* )    '(acl2s::positive-rational) )
-    ((eq ts-decoded '*TS-NON-NEGATIVE-INTEGER*)  '(acl2s::nat));(0 pos)) 
-    ((eq ts-decoded '*TS-NON-POSITIVE-INTEGER*)  '(acl2s::neg '0)) 
+    ((eq ts-decoded '*TS-NON-NEGATIVE-INTEGER*)  '(acl2s::nat));(0 pos))
+    ((eq ts-decoded '*TS-NON-POSITIVE-INTEGER*)  '(acl2s::neg '0))
     ((eq ts-decoded '*TS-RATIO*)                 '(acl2s::ratio) )
     ((eq ts-decoded '*TS-CONS*  )                '(acl2s::cons) )
     ((eq ts-decoded '*TS-BOOLEAN*)               '(acl2s::boolean) )
     ((eq ts-decoded '*TS-TRUE-LIST*)             '(acl2s::true-list) )
-    
+
     ((eq ts-decoded '*TS-EMPTY*)                 '(acl2s::empty));is it possible?
     (t  (if (consp ts-decoded)
-            (cond 
+            (cond
              ((equal 'TS-UNION (car ts-decoded))
               (get-types-from-type-set-decoded-lst (cdr ts-decoded) nil))
              ((and (equal 'TS-COMPLEMENT (car ts-decoded))
@@ -62,12 +62,12 @@
               '(acl2s::atom))
              (t '(acl2s::all)))
           '(acl2s::all)))))
- 
+
 (defun get-types-from-type-set-decoded-lst (ts-lst ans)
    (if (endp ts-lst)
      ans
-     (get-types-from-type-set-decoded-lst 
-      (cdr ts-lst) 
+     (get-types-from-type-set-decoded-lst
+      (cdr ts-lst)
       (append (acl2::get-type-from-type-set-decoded (car ts-lst))
               ans))))
  )
@@ -82,7 +82,7 @@
       (list typ))))
 
 (defun get-types-from-type-set-lst (ts-lst)
-  (declare (xargs :mode :program 
+  (declare (xargs :mode :program
                   :guard (integer-listp ts-lst)))
   (if (endp ts-lst)
     nil
@@ -106,10 +106,10 @@
          (ts (if (consp ts-info) (cadr ts-info) nil)))
      (if ts
          (let ((types (get-type-list-from-type-set ts)))
-          (get-var-types-from-type-alist acl2-type-alist 
-                                          (cdr freevars) 
+          (get-var-types-from-type-alist acl2-type-alist
+                                          (cdr freevars)
                                           (acons var types ans)))
-       (get-var-types-from-type-alist acl2-type-alist 
+       (get-var-types-from-type-alist acl2-type-alist
                                        (cdr freevars) ans)))))
 
 (defun decode-acl2-type-alist (acl2-type-alist freevars)
@@ -129,6 +129,15 @@
 (verify-termination acl2::cons-term); ASK MATT to make these logic mode
 (set-verify-guards-eagerness 1)
 
+
+(defun make-dumb-type-alist (vars)
+  "the default dumb type-alist with all variables associated with TOP i.e acl2s::all"
+  (declare (xargs :guard (symbol-listp vars))) ;use proper-symbol-listp
+  (pairlis$ vars (make-list (len vars)
+                            :initial-element
+                            (list 'ACL2S::ALL))))
+
+
 (defun get-acl2-type-alist-fn (cl vars ens state)
   (declare (xargs :mode :program :stobjs (state)))
   (b* (((mv erp type-alist &)
@@ -144,20 +153,22 @@
 ;This context, gives us the type-alist ACL2 inferred from the
 ;the current subgoal i.e. cl
        (vt-acl2-alst (if erp ;contradiction
-                         (pairlis$ vars (make-list (len vars)
-                                                   :initial-element 
-                                                   (list 'ACL2S::ALL)))
+                         (make-dumb-type-alist vars)
                        (decode-acl2-type-alist type-alist vars))))
    vt-acl2-alst))
 
 
 (defmacro get-acl2-type-alist (cl &optional vars ens)
-  `(get-acl2-type-alist-fn ,cl 
+  `(get-acl2-type-alist-fn ,cl
                            ,(or vars '(acl2::all-vars1-lst cl '()))
                            ,(or ens '(acl2::ens state))
                            state))
 
-; utility fn to print if verbose flag is true 
+
+
+
+
+; utility fn to print if verbose flag is true
 (defmacro cw? (verbose-flag &rest rst)
   `(if ,verbose-flag
      (cw ,@rst)
@@ -196,7 +207,7 @@
 ;;       ans
 ;;     (tau-alist-clauses (cdr clauses) sign ens wrld state
 ;;                        (append (tau-alist-clause (car clauses) sign ens wrld state) ans))))
-      
+
 
 
 (defun all-vals (key alist)
@@ -224,9 +235,9 @@
       ans
     (b* ((tau (car taus))
            (interval (acl2::access acl2::tau tau :interval)))
-    (conjoin-tau-interval-lst (cdr taus) 
+    (conjoin-tau-interval-lst (cdr taus)
                               (acl2::conjoin-intervals interval ans)))))
-  
+
 
 (defun tau-interval-alist (var-taus-alist)
 ;[var . taus] -> [var . interval]
@@ -234,7 +245,7 @@
   (if (endp var-taus-alist)
       '()
     (b* (((cons var taus) (car var-taus-alist))
-         (interval (conjoin-tau-interval-lst taus nil)) ;nil represents the universal interval 
+         (interval (conjoin-tau-interval-lst taus nil)) ;nil represents the universal interval
          )
       (if (null interval) ;universal
           (tau-interval-alist (cdr var-taus-alist))
@@ -252,10 +263,10 @@
       tau-interval-alist))
 
 (defmacro tau-interval-alist-clause (cl &optional vars ens)
-  `(tau-interval-alist-clause-fn ,cl 
+  `(tau-interval-alist-clause-fn ,cl
                                  ,(or vars '(all-vars-lst cl))
                                  ,(or ens '(acl2::ens state))
-                                 state))     
+                                 state))
 
 
 
@@ -275,50 +286,17 @@
 
 ;; ;redundant from defdata.lisp
 ;; (defrec types-info%
-;;   (size 
-;;    predicate 
+;;   (size
+;;    predicate
 ;;    enumerator enum-uniform
 ;;    enumerator-test enum-uniform-test
 ;;    recursivep derivedp consistentp
 ;;    type-class defs) NIL)
- 
 
-;; NOTE: In the following the type 'empty' has
-;; special status and treated seperately
-;; (def meet (typ1 typ2 vl wrld)
-;;   (decl :sig ((symbol symbol vl plist-worldp) -> symbol)
-;;         :doc "find smaller type in subtype hierarchy/lattice")
-;;   (declare (xargs :verify-guards nil))
-;;   ;; (decl :sig ((possible-defdata-type-p possible-defdata-type-p
-;; ;;                plist-world) -> possible-defdata-type-p)
-;;   (b* (((when (or (eq 'acl2::empty typ1)
-;;                   (eq 'acl2::empty typ2))) 'acl2::empty)
-;;        ((when (eq typ2 typ1)) typ2)
-;;        ((unless (and (is-a-typeName typ1 wrld)
-;;                      (is-a-typeName typ2 wrld)))
-;;         (prog2$
-;;          (cw? (verbose-stats-flag vl)
-;;               "~|CEgen/Note: ~x0 or ~x1 not a defdata type. ~ Meet returning universal type ALL.~|" typ1 typ2)
-;;          'acl2::all))
-;;        ((when (eq 'acl2::all typ1)) typ2)
-;;        ((when (eq 'acl2::all typ2)) typ1)
-;;        ((when (is-subtype typ1 typ2 wrld))   typ1)
-;;        ((when (is-subtype typ2 typ1 wrld))   typ2)
-;;        ((when (is-disjoint typ2 typ1 wrld))  'acl2::empty) ;Should we instead define the NULL type??? Modified: so Ans is YES instead of Ans: NO, the way its used now, this is right!
-;; ;give preference to custom type
-;;        ((when (is-a-custom-type typ1 wrld)) typ1)
-;;        ((when (is-a-custom-type typ2 wrld)) typ2)
 
-;; ; choose the one that was defined later (earlier in 
-;; ; reverse chronological order)
-;;        (all-types (strip-cars (table-alist 'defdata::types-info-table wrld)))
-;;        )
-;;    (if (< (position-eq typ1 all-types) (position-eq typ2 all-types)) 
-;;        typ1 
-;;      typ2)))
 
 (defmacro   verbose-stats-flag ( vl)
-  `(> ,vl 2)) 
+  `(> ,vl 2))
 
 (defmacro   debug-flag ( vl)
   `(> ,vl 3))
@@ -339,7 +317,7 @@
          (cw? (verbose-stats-flag vl)
               "~|CEgen/Note: ~x0 or ~x1 not a name. ~ Meet returning universal type ALL.~|" typ1 typ2)
          'acl2s::all))
-                     
+
        (typ1-al-entry (assoc-eq typ1 M))
        (typ2-al-entry (assoc-eq typ2 M))
        ((unless (and typ1-al-entry typ2-al-entry))
@@ -360,16 +338,16 @@
 ;;        ((when (defdata::is-a-custom-type typ1 wrld)) typ1)
 ;;        ((when (defdata::is-a-custom-type typ2 wrld)) typ2)
 
-; choose the one that was defined later (earlier in 
+; choose the one that was defined later (earlier in
 ; reverse chronological order)
-       (all-types (strip-cars (table-alist 'defdata::type-metadata-table wrld)))
+       (all-types (strip-cars (table-alist 'DEFDATA::TYPE-METADATA-TABLE wrld)))
        )
-   (if (< (position-eq typ1 all-types) (position-eq typ2 all-types)) 
-       typ1 
+   (if (< (position-eq typ1 all-types) (position-eq typ2 all-types))
+       typ1
      typ2)))
 
 (def dumb-type-alist-infer-from-term (term vl wrld  ans.)
-  (decl :sig ((pseudo-term-listp fixnum plist-worldp  symbol-alistp) 
+  (decl :sig ((pseudo-term-listp fixnum plist-worldp  symbol-alistp)
               -> symbol-alistp)
         :doc "main aux function to infer type-alist from term")
   (declare (xargs :verify-guards nil))
@@ -378,13 +356,13 @@
   (f* ((add-eq-typ... (t1) (if (acl2::equivalence-relationp R wrld)
                                (put-assoc x (list t1) ans.)
                              ans.)))
-    
+
 ; Copied from v-cs%-alist-from-term. Keep in sync!
   (case-match term
-    
+
 ;the following is a rare case (I found it when the conclusion is nil
 ;and its negation is 'T
-    (('quote c) (declare (ignore c))  ans.) ;ignore quoted constant terms 
+    (('quote c) (declare (ignore c))  ans.) ;ignore quoted constant terms
 
 ;TODO possible field variable (i.e f is a getter/selector) Note that
 ; term cannot have a lambda applicaton/let, so the car of the term is
@@ -393,23 +371,23 @@
 
 ;x has to be an atom below, otherwise, we would have caught that case above.
     (('not x)      (put-assoc x (list ''nil) ans.))
-    
+
     ((P x)   (b* ((tname (defdata::is-type-predicate P wrld))
                   ((unless tname) ans.)
                   (curr-typs-entry (assoc-eq x ans.))
-                  ((unless (and curr-typs-entry 
+                  ((unless (and curr-typs-entry
                                 (consp (cdr curr-typs-entry))))
 ; no or invalid entry, though this is not possible, because we call it with
-; default type-alist of ((x . ('ALL)) ...)
+; default type-alist of ((x . ('ACL2S::ALL)) ...)
                    ans.)
                   (curr-typs (cdr curr-typs-entry))
-                  (- (cw? (and (verbose-stats-flag vl) 
+                  (- (cw? (and (verbose-stats-flag vl)
                                (consp (cdr curr-typs)))
                           "~|CEgen/Warning: Ignoring rest of union types ~x0 ~|" (cdr curr-typs)))
-                     
+
                   (curr-typ (car curr-typs))
                   ((when (defdata::possible-constant-value-p curr-typ)) ans.)
-                   
+
                   (final-typ (meet tname curr-typ vl wrld)))
                (put-assoc x (list final-typ) ans.)))
 
@@ -421,12 +399,12 @@
     ((R ('quote c) x)    (add-eq-typ... (kwote c)))
     ;((R x (f . args))    (add-eq-constraint... (acl2::cons-term f args)))
     ;((R (f . args) x)    (add-eq-constraint... (acl2::cons-term f args)))
-    
+
     ;; has to be a (R t1 t2 ...) or atomic term
     (&                   ans.))))
 
 (def dumb-type-alist-infer-from-terms (H vl wrld  ans.)
-  (decl :sig ((pseudo-term-listp fixnum plist-worldp  
+  (decl :sig ((pseudo-term-listp fixnum plist-worldp
                                  symbol-alistp) -> symbol-alistp)
         :doc "aux function for dumb extraction of defdata types from terms in H")
   (declare (xargs :verify-guards nil))
@@ -437,15 +415,11 @@
       (dumb-type-alist-infer-from-terms (cdr H) vl wrld ans.))))
 
 (def dumb-type-alist-infer (H vars vl wrld)
-  (decl :sig ((pseudo-term-listp symbol-listp fixnum plist-worldp) 
+  (decl :sig ((pseudo-term-listp proper-symbol-listp fixnum plist-worldp)
               -> symbol-alistp)
         :doc "dumb infer defdata types from terms in H")
   (declare (xargs :verify-guards nil))
-  (dumb-type-alist-infer-from-terms 
-   H vl wrld
-   (pairlis$ vars (make-list (len vars)
-                             :initial-element 
-                             (list 'ACL2S::ALL)))))
+  (dumb-type-alist-infer-from-terms H vl wrld (make-dumb-type-alist vars)))
 
 (defmacro   debug-flag  (vl)
   `(> ,vl 3))
@@ -460,7 +434,7 @@
 ; variables that the user wants in his final type-alist
 ; A1 and A2 and the return value have type
 ; (listof (cons symbolp (listof possible-defdata-type-p)))
-; TODO: if val has more than 1 type, then we treat it as (list 'ALL)
+; TODO: if val has more than 1 type, then we treat it as (list 'ACL2S::ALL)
 
 ; Usually its called with A1 as the acl2 type alist and A2 as the
 ; top-level type alist. so it might contain
@@ -479,17 +453,21 @@
                                  (M (table-alist 'defdata::type-metadata-table wrld))
                                  (P (defdata::predicate-name dt M))
 
+                                 ((unless (defdata::plausible-predicate-functionp P wrld)) ;abort before calling ev-fncall on non-function
+                                  (prog2$ (cw? (debug-flag vl)
+                                               "~|CEGen/Warning:: ~x0: Bad args to eval-and-get-meet: ~x1 ~x2. ~|" ctx typ1 typ2)
+                                          (mv t st))) ;prefer singleton type
                                  ;; args to ev-fncall-w is a list of evaluated values.
                                  ((mv erp res) (acl2::ev-fncall-w P (list (if (quotep st) ;possible bug caught, what if st is not quoted!
                                                                               (acl2::unquote st)
-                                                                            st)) 
+                                                                            st))
                                                                   wrld nil nil t nil nil))
                                  (- (cw? (and erp (debug-flag vl))
-                                         "~|CEgen/Error in ~x0: while calling ev-fncall-w on ~x1~|" ctx (cons P (list st))))
+                                         "~|CEgen/Error:: ~x0: while calling ev-fncall-w on ~x1~|" ctx (cons P (list st))))
                                  (- (cw? (and (not erp) (not res) (debug-flag vl))
                                          "~|CEgen/Debug:: ~x0 evaluated to nil~|" (cons P (list st))))
                                  ((when erp)
-                                  (mv t 'ACL2S::EMPTY)))
+                                  (mv t st))) ;prefer singleton type
                               (if res (mv nil st) (mv nil 'ACL2S::EMPTY)))))
   (if (endp A1)
       (mv nil '())
@@ -499,7 +477,7 @@
          (types2-entry (assoc-eq var A2))
          (types2 (if types2-entry (cdr types2-entry) '(ACL2S::ALL)))
          (typ2 (get-type... types2))
-         ((unless (and (possible-defdata-type-p typ1) 
+         ((unless (and (possible-defdata-type-p typ1)
                        (possible-defdata-type-p typ2)))
           (mv t '()))
          ((mv erp rest) (meet-type-alist (cdr A1) A2 vl wrld ))
