@@ -22,7 +22,7 @@ data last modified: [2014-08-06]
 
 
 ; Generate Enumerator/acc defun bodies
-  
+
 ;lets forbid some names to be used in defdata body
 (defun forbidden-names-enum () '(_seed _v _i _choice))
 (verify-guards forbidden-names-enum)
@@ -31,7 +31,7 @@ data last modified: [2014-08-06]
 
 ;; (defmacro split (k i)
 ;;   `(let ((i1-k (split-nat ,k ,i)))
-;;      (mv . ,(pair-prefix 'nth (list-up-lists (make-numlist-from 0 k) 
+;;      (mv . ,(pair-prefix 'nth (list-up-lists (make-numlist-from 0 k)
 ;;                                        (make-list k :initial-element 'i1-k))))))
 
 
@@ -86,18 +86,17 @@ data last modified: [2014-08-06]
               (i1--ik (numbered-vars 'i k))
               (enum-arg-exprs (make-enum-Is... (remove-names-lst (cdr s)) i1--ik))
               (binding (bind-names-vals (cdr s) enum-arg-exprs))
-              (s2 (keep-names-lst (cdr s)))
               (?satisfies-exprs (get-all :satisfies kwd-alist))
-              (rst (make-enum-Is... s2 i1--ik)))
+              (rst (make-enum-Is... (keep-names-lst (cdr s)) i1--ik)))
 
-           `(B* (((list . ,i1--ik) (SPLIT-NAT ,k ,i)))
+           `(B* (((list . ,i1--ik) (SPLIT-NAT ,k ,i))) ;TODO: This needlessly splits seeds for constant texps too!!
                     ,(if binding
                          `(B* ,binding (,(car s) . ,rst))
                        `(,(car s) . ,rst)))))
         (t
 ; possible dependent expr?
          `(,(car s) . ,(make-enum-Is... (cdr s) (make-list (len (cdr s)) :initial-element i))))))
-         
+
 (defun make-enum-Is (texps is   kwd-alist M C B wrld)
   (declare (ignorable kwd-alist))
   (if (endp texps)
@@ -111,7 +110,7 @@ data last modified: [2014-08-06]
 (defun mv2-let-seq (bs vals body)
   (if (endp bs)
       body
-    (list 'MV-LET 
+    (list 'MV-LET
           (car bs)
           (car vals)
           (mv2-let-seq (cdr bs) (cdr vals) body))))
@@ -120,32 +119,61 @@ data last modified: [2014-08-06]
 
 (defloop bind-mv2-names-enum/acc-calls (texps vals temp-names acc-exp)
   "b* mv binding for each name decl texp -> corresponding val, albeit when not named, take a temp name"
-  (for ((texp in texps) (val in vals) (nm in temp-names)) 
+  (for ((texp in texps) (val in vals) (nm in temp-names))
        (collect (if (named-defdata-exp-p texp)
                     (list (list 'MV (car texp) acc-exp) val)
                   (list (list 'MV nm acc-exp) val)))))
-                                                          
-
-(defmacro make-enum/acc-I... (s x)
-  `(make-enum/acc-I ,s ,x kwd-alist M C B wrld))
-(defmacro make-enum/acc-Is... (ss xs)
-  `(make-enum/acc-Is ,ss ,xs kwd-alist M C B wrld))
 
 (mutual-recursion
- (defun make-enum/acc-I (s i   kwd-alist M C B wrld)
+ (defun has-recursive-reference-p (texp new-names)
+   (cond ((possible-constant-value-p texp) nil)
+         ((proper-symbolp texp) (member-eq texp new-names))
+
+         ((not (true-listp texp)) (member-equal (cdr texp) new-names))
+         (t (has-recursive-reference-lst-p (cdr texp) new-names))))
+ (defun has-recursive-reference-lst-p (texps new-names)
+   (if (endp texps)
+       nil
+     (or (has-recursive-reference-p (car texps) new-names)
+         (has-recursive-reference-lst-p (cdr texps) new-names))))
+ )
+
+(defun bound-seeds-to-recursive-calls (i i1--ik texps new-names)
+  (if (endp texps)
+      '()
+    (b* ((i_current (car i1--ik))
+         (texp (car texps)))
+      (cons
+       (if (has-recursive-reference-p texp new-names)
+           `(if (or (zp ,i_current) (< ,i_current ,i)) ,i_current (1- ,i_current))
+         i_current)
+       (bound-seeds-to-recursive-calls i (cdr i1--ik) (cdr texps) new-names)))))
+
+
+
+
+(defmacro make-enum/acc-I... (s x)
+  `(make-enum/acc-I ,s ,x kwd-alist new-names M C B wrld))
+(defmacro make-enum/acc-Is... (ss xs)
+  `(make-enum/acc-Is ,ss ,xs kwd-alist new-names M C B wrld))
+
+(mutual-recursion
+ (defun make-enum/acc-I (s i kwd-alist new-names M C B wrld)
+   (declare (ignorable new-names))
    "enumerator/acc interpretation for core defdata exp s.
 i is the name of the current indicial argument (dont confuse with an integer) that is used as recursion measure.
 kwd-alist gives some defaults.
+new-names is self-explanatory (imp to find recursive references)
 M is type metadata table + some basic info for the clique of types being defined.
 C is the constructor table + some basic info for new constructors.
 B is the builtin combinator table."
 
    (cond ((possible-constant-value-p s) `(MV ,(if (quotep s) s (list 'QUOTE s)) _SEED))
-         ((proper-symbolp s) (if (assoc-eq s M) 
+         ((proper-symbolp s) (if (assoc-eq s M)
                                 `(,(get2 s :enum/acc M) ,i _SEED)
                               `(MV ,s _SEED)))
 
-         ((not (true-listp s)) (car s)) ;(make-enum/acc-I (cdr s) i kwd-alist M C B wrld)) ;name decl
+         ((not (true-listp s)) (car s)) ;(make-enum/acc-I (cdr s) i kwd-alist M C B wrld)) ;name decl   ;TODO.check -- seems wrong!!
 
          ((assoc-eq (car s) B) ;builtin combinator
           (b* ((enum/acc-I-fn (get2 (car s) :enum/acc-I B))
@@ -161,7 +189,7 @@ B is the builtin combinator table."
 ;                           (RANDOM-INDEX-SEED (MIN ,k ,i) _SEED) ;i forces termination
                            (CASE _CHOICE
                                  . ,(append (list-up-lists (make-numlist-from 0 k)
-                                                     (make-enum/acc-Is (cdr s) (make-list k :initial-element i) kwd-alist M C B wrld))
+                                                     (make-enum/acc-Is... (cdr s) (make-list k :initial-element i)))
                                             '((OTHERWISE (mv nil _SEED))))))
 
                 nil)))) ;unsupported -- raise catchable error.
@@ -173,14 +201,16 @@ B is the builtin combinator table."
                (_v1--_vk (numbered-vars '_V k))
                (binding (bind-mv2-names-enum/acc-calls (cdr s) enum/acc-exprs _v1--_vk '_SEED))
                (names (replace-calls-with-names _v1--_vk (cdr s))))
-               
-            `(B* (((list . ,i1--ik) (SPLIT-NAT ,k ,i)))
+
+            `(B* (((mv (list . ,i1--ik) _SEED) (RANDOM-INDEX-LIST-SEED ,k ,i _SEED))) ;TODO.check later. random-indexes less that i force termination.
+                  ;((list . ,i1--ik) (list ,@(bound-seeds-to-recursive-calls i i1--ik (cdr s) new-names))))
+
                  (B* ,binding (MV (,(car s) . ,names) _SEED)))))
          (t
 ; possible dependent expr?
           `(,(car s) . ,(make-enum/acc-Is... (cdr s) (make-list (len (cdr s)) :initial-element i))))))
- 
- (defun make-enum/acc-Is (texps is   kwd-alist M C B wrld)
+
+ (defun make-enum/acc-Is (texps is kwd-alist new-names M C B wrld)
    (declare (ignorable kwd-alist))
    (if (endp texps)
        '()
@@ -202,7 +232,7 @@ B is the builtin combinator table."
        (guard-decl-forms (and guard-lambda `((DECLARE (XARGS :MODE :PROGRAM ;hack
                                                              :GUARD ,(expand-lambda (cons guard-lambda actuals))))))))
     (append measure-decl-forms guard-decl-forms)))
-    
+
 (defun make-enum/acc-declare-forms (ivar kwd-alist)
   (b* ((guard-lambda (get1 :enum/acc-guard kwd-alist)) ;its a lambda
        (actuals (list ivar '_SEED)))
@@ -224,7 +254,7 @@ B is the builtin combinator table."
        (enum-body (make-enum-I ndef ivar kwd-alist M C B wrld))
        (enum-decls (make-enum-declare-forms ivar kwd-alist))
        )
-    
+
     `(defun ,(enumerator-name name M) (,ivar)
        ,@enum-decls
        ,enum-body)))
@@ -252,10 +282,10 @@ B is the builtin combinator table."
        (kwd-alist (append kwd-alist top-kwd-alist))
        (avoid-lst (append (forbidden-names) (strip-cars N)))
        (ivar (if (member-eq 'i avoid-lst) 'i (acl2::generate-variable 'i avoid-lst nil nil wrld)))
-       (enum/acc-body (make-enum/acc-I ndef ivar kwd-alist M C B wrld))
+       (enum/acc-body (make-enum/acc-I ndef ivar kwd-alist (strip-cars new-types) M C B wrld))
        (enum/acc-decls (make-enum/acc-declare-forms ivar kwd-alist))
        )
-    
+
     `(defun ,(get2 name :enum/acc M) (,ivar _SEED)
        ,@enum/acc-decls
        ,enum/acc-body)))
@@ -273,4 +303,4 @@ B is the builtin combinator table."
       enum/acc-events)))
 
 
-(add-pre-post-hook defdata-defaults-table :cgen-hook-fns '(enumerator-events enumerator/acc-events))
\ No newline at end of file
+(add-pre-post-hook defdata-defaults-table :cgen-hook-fns '(enumerator-events enumerator/acc-events))
