@@ -51,7 +51,7 @@ data last modified: [2014-08-06]
         (cons (car terms) (find-x-terms-=-depth (cdr terms) x depth))
       (find-x-terms-=-depth (cdr terms) x depth)))))
 
- 
+
 (defun find-x-terms->=-depth (terms x depth)
   "find terms having x at greater than or equal to given depth"
   (if (endp terms)
@@ -87,8 +87,8 @@ data last modified: [2014-08-06]
 
 
 
-       
-  
+
+
 ;Tell J this problem TODO
 (DEFUN DUMBer-NEGATE-LIT (TERM)
   (COND ((acl2::VARIABLEP TERM)
@@ -134,15 +134,16 @@ data last modified: [2014-08-06]
            (if cons-x ;dest-elim 1 round successful
                (cond (remaining-x-es
                       (prog2$ ;check this
-                       (cw "~| Presence of ~x0 precludes a tau characterization of ~x1~%" remaining-x-es P)
-                       "Multiple sig terms i.e. (P1 (f x1 ...)) \/ (P2 (f x1 ...)) not allowed in conclusion of signature rule"))
+                       (cw? nil "~| Presence of ~x0 precludes a tau characterization of ~x1~%" remaining-x-es P)
+"Multiple sig terms i.e. (P1 (f x1 ...)) \/ (P2 (f x1 ...))
+ not allowed in conclusion of signature rule."))
                      ((nested-functional-terms-with-vars-p dest-es x1--xk)
                       (prog2$
-                       (cw "~| Nested destructors precludes a tau characterization of ~x0~%" P)
+                       (cw? nil "~| Nested destructors precludes a tau characterization of ~x0~%" P)
                        "Nesting i.e. (P (f ... (g x1 ...) ...) not allowed in conclusion of signature rule"))
                      (t `(IMPLIES (AND . ,dest-es) (,P ,cons-x))))
              (prog2$
-              (cw "~| Non-dest-eliminable AND nest ~x0 precludes a tau characterization of ~x1~%" fes P)
+              (cw? nil "~| Non-dest-eliminable AND nest ~x0 precludes a tau characterization of ~x1~%" fes P)
               "Illegal tau rule")))
        (if fes ;there is one nested term
            (if (= (depth-var x (car fes)) 2)
@@ -150,7 +151,7 @@ data last modified: [2014-08-06]
              "Nesting i.e. (P (f ... (g x1 ...) ...) not allowed in conclusion of signature rule")
 ;poss simple or conj rule
          `(IMPLIES (AND . ,es) (,P ,x))))))
-  
+
 
 
 (defloop tau-rules-DNF=>Px (conj-clauses Px C)
@@ -172,21 +173,23 @@ data last modified: [2014-08-06]
        (conjunctive-clauses (acl2::cnf-dnf t te nil)) ;get dnf form
        (rules (tau-rules-DNF=>Px conjunctive-clauses Px C)))
     rules))
-    
+
 
 
 (defun tau-rules-Px=>OR-terms (terms P x)
   (if (consp terms)
       (b* ((fes2 (find-x-terms->=-depth terms x 2)))
         (cond ((null fes2) ;no nesting, return simple or conj rule
-          (if (consp (cdr terms)) ;len > 1 
+          (if (consp (cdr terms)) ;len > 1
               `(IMPLIES (AND . ,(cons (list P x) (dumber-negate-lit-lst (cdr terms)))) ,(car terms))
             `(IMPLIES (,P ,x) ,(car terms))))
          ((not (consp (cdr fes2))) ;exactly one sig-like term
           (if (= (depth-var x (car fes2)) 2)
               `(IMPLIES (AND . ,(cons (list P x) (dumber-negate-lit-lst (set-difference-equal terms fes2)))) ,(car fes2)) ;sig rule
             "Nesting i.e. (P (f ... (g x1 ...) ...) not allowed in conclusion of signature rule"))
-         (t "Multiple sig terms i.e. (P1 (f x1 ...)) \/ (P2 (f x1 ...)) not allowed in conclusion of signature rule")))
+         (t
+"Multiple sig terms i.e. (P1 (f x1 ...)) \/ (P2 (f x1 ...))
+ not allowed in conclusion of signature rule")))
     "Impossible: Empty clause"))
 
 (defloop tau-rules-Px=>CNF (clauses Px)
@@ -213,7 +216,7 @@ data last modified: [2014-08-06]
 
 (defloop get-first-eq-constant-dont-change (terms wrld)
   (for ((term in terms)) (thereis (get-eq-constant-dont-change term wrld))))
-  
+
 
 (defun tau-rule-Px-recog=>prod (and-terms P x C wrld)
   (declare (ignorable wrld))
@@ -228,7 +231,7 @@ data last modified: [2014-08-06]
 ;TODO -- revisit this!! The and-terms eaten up tau-rule-Px-=>EQ-NIL-Hack should not
 ; in the first place come here.
 
-       
+
        (dterms (remove-equal recog-exp and-terms)))
 
      (if (find-x-terms->=-depth dterms x 3)
@@ -239,7 +242,7 @@ data last modified: [2014-08-06]
 (defloop tau-rules-Px=>SoP (sop P x C wrld)
   "Given sum-of-products pred expr sop, return a list of characterizing tau rules"
   (for ((prod in sop)) (append (tau-rule-Px-recog=>prod prod P x C wrld))))
-       
+
 (defun tau-rule-Px-=>EQ-NIL-Hack (and-terms P x wrld)
   (b* ((eq-exp (get-first-eq-constant-dont-change and-terms wrld))
        ((unless eq-exp) ;hack
@@ -253,7 +256,7 @@ data last modified: [2014-08-06]
                     nil))
 ;       (- (cw "recog = ~x0 " recog-exp))
        ((unless recog-exp) nil))
-                               
+
     `((IMPLIES (AND (,P ,x) ,recog-exp) ,eq-exp))))
 
 (defloop tau-rules-Px=>EQ-constants (sop P x wrld)
@@ -262,19 +265,17 @@ data last modified: [2014-08-06]
 
 
 
-
-(defun recognizer-call (call C)
+(defun recognizer-call (call C wrld)
   (case-match call
-    ((P x) (and (proper-symbolp x) (get-conx-name P C) call))
+    ((P x) (and (proper-symbolp x) (or (get-conx-name P C) (tau-predicate-p P wrld)) call))
     (& nil)))
 
-(defloop governing-recognizer-call (terms C) ;cheat: just give the first
-  (for ((term in terms)) (thereis (recognizer-call term C))))
-
+(defloop governing-recognizer-call (terms C wrld) ;cheat: just give the first
+  (for ((term in terms)) (thereis (recognizer-call term C wrld))))
 
 (defun disjoint-clause2-p (cl1 cl2 C wrld)
-  (b* ((P1x (governing-recognizer-call cl1 C))
-       (P2x (governing-recognizer-call cl2 C))
+  (b* ((P1x (governing-recognizer-call cl1 C wrld))
+       (P2x (governing-recognizer-call cl2 C wrld))
        (evg1-term (get-first-eq-constant cl1 wrld))
        (evg2-term (get-first-eq-constant cl2 wrld))
        (evg1 (and evg1-term (third evg1-term)))
@@ -282,12 +283,12 @@ data last modified: [2014-08-06]
 ;       (- (cw "~| evg1-term = ~x0 evg2-term = ~x1 P1x = ~x2 P2x = ~x3" evg1-term evg2-term P1x P2x))
        )
     (cond ((and evg1-term evg2-term) (not (equal evg1 evg2)))
-          ((and evg2-term P1x (equal (second P1x) (second evg2-term))) 
+          ((and evg2-term P1x (equal (second P1x) (second evg2-term)))
            (b* (((mv erp res) (acl2::ev-fncall-w (car P1x) (list evg2) wrld nil nil t t nil)))
              (and (not erp) (not res))))
-          ((and evg1-term P2x (equal (second P2x) (second evg1-term))) 
+          ((and evg1-term P2x (equal (second P2x) (second evg1-term)))
            (b* (((mv erp res) (acl2::ev-fncall-w (car P2x) (list evg1) wrld nil nil t t nil)))
-             (and (not erp) (not res)))) 
+             (and (not erp) (not res))))
           ((and P1x P2x) (disjoint-p (car P1x) (car P2x) wrld))
           (t nil))))
 
@@ -319,12 +320,12 @@ data last modified: [2014-08-06]
 
 (defun tau-rules-Px=>form (form Px s  new-fns-and-args ctx C wrld)
   (b* (((mv erp te) (acl2::pseudo-translate form new-fns-and-args wrld))
-       ((when erp) 
+       ((when erp)
         (prog2$
          (cw "~| ~x0: Error in translate: ~x1" ctx te)
          (list "Error in translate in tau-rules => direction")))
        (te (expand-lambda te)) ;eliminate let/lambda
-;       (vars (all-vars1-lst te '())) 
+;       (vars (all-vars1-lst te '()))
 ;       (- (assert$ (= 1 (len vars)) nil))) ;monadic
        )
     (if (shallow-union-of-prods-p s wrld)
@@ -334,11 +335,11 @@ data last modified: [2014-08-06]
                       ;;TODO The conj clauses eaten/consumed by above should be
                       ;;excluded from below call!!
                       (tau-rules-Px=>SoP conj-clauses (car Px) (cadr Px) C wrld))
-            (list "Dont know how to characterize (using tau rules) a non-disjoint union type")))
+            (list "Unable to characterize (using tau rules) a non-disjoint union type")))
 
       (b* ((clauses (acl2::cnf-dnf t te t))) ;get cnf
         (tau-rules-Px=>CNF clauses Px)))))
-     
+
 
 (defloop filter-strings (xs)
   (for ((x in xs)) (append (and (stringp x) (list x)))))
@@ -351,14 +352,14 @@ data last modified: [2014-08-06]
            (mv msgs (cons 'AND rules)))
           ((consp rules) (mv msgs (car rules))) ;single rule
           (t (mv msgs nil)))))
-  
+
 
 
 (defun all-1-arity-fns1 (conx-al)
     (b* ((dest-pred-alist (get1 :dest-pred-alist conx-al))
          (recog (get1 :recog conx-al)))
       (cons recog (strip-cars dest-pred-alist))))
-         
+
 (defloop all-1-arity-fns (new-constructors)
   (for ((cx in new-constructors)) (append (all-1-arity-fns1 (cdr cx)))))
 
@@ -367,7 +368,7 @@ data last modified: [2014-08-06]
          (arity (get1 :arity conx-al))
          (v1--vk (numbered-vars x arity)))
       (cons conx v1--vk)))
-         
+
 (defloop all-conx-fns-args (new-constructors x)
   (for ((cx in new-constructors)) (collect (all-conx-fns-args1 cx x))))
 
@@ -381,8 +382,8 @@ data last modified: [2014-08-06]
         (t (let ((fn (caar fn-args-lst)))
                  ;(formals (cdar fn-args-lst)))
              (putprop
-              fn 'acl2::tau-pair (if (is-allp-alias fn wrld) 
-                                     (cons nil fn) 
+              fn 'acl2::tau-pair (if (is-allp-alias fn wrld)
+                                     (cons nil fn)
                                    (cons 0 fn))
               (extend-wrld-with-fn-args-list-with-tau-pair (cdr fn-args-lst) wrld))))))
 
@@ -391,7 +392,7 @@ data last modified: [2014-08-06]
                 (acl2::strip-force-and-case-split-in-hyps-of-pairs (acl2::unprettyify term))))
         (wrld1 (extend-wrld-with-fn-args-list-with-tau-pair fn-args-lst wrld)))
     (acl2::acceptable-tau-rulesp :all pairs wrld1)))
-       
+
 
 (defun tau-characterization-events1 (pair top-kwd-alist ctx wrld)
   (b* (((cons name A) pair)
@@ -431,34 +432,35 @@ data last modified: [2014-08-06]
        (msgs=> (remove-duplicates-equal (append (and (not rule-=>-Px-tau-acceptable-p) (list unacceptable-tau-rule-msg)) msgs=>)))
        (msgs<= (remove-duplicates-equal (append (and (not rule-Px-=>-tau-acceptable-p) (list unacceptable-tau-rule-msg)) msgs<=)))
        (?recp (get1 :recp kwd-alist))
+       (yes (get1 :print-commentary kwd-alist))
        )
-    
 
-    
-    (append (and msgs<= `((value-triple (cw "~| Incomplete tau characterization of ~x0 <= body. ~% Failure messages: ~x1 ~%" ',Px ',msgs<=))))
-            (and msgs=> `((value-triple (cw "~| Incomplete tau characterization of ~x0 => body. ~% Failure messages: ~x1 ~%" ',Px ',msgs=>))))
+
+
+    (append (and msgs<= `((commentary ,yes "~| ~x0 <= body -- not complete. ~|Reasons: ~x1 ~%" ',Px ',msgs<=)))
+            (and msgs=> `((commentary ,yes "~| ~x0 => body -- not complete. ~|Reasons: ~x1 ~%" ',Px ',msgs=>)))
             (and (not msgs=>) (not msgs<=) rule-Px-=>-tau-acceptable-p rule-Px-=>-tau-acceptable-p
-                 `((value-triple (cw "~|Defdata/Note: Complete tau characterization of ~x0 successful.~%" ',Px))))
-            (and rule-=>-Px 
+                 `((commentary ,yes "~|Defdata/Note: ~x0 relatively complete for Tau.~%" ',(car Px))))
+            (and rule-=>-Px
                  `((DEFTHM ,(symbol-fns::prefix 'def '=> name)
                      ,rule-=>-Px
                      :HINTS (("Goal" :IN-THEORY (e/d (,(car Px)) (,@disabled ,@(strip-cars new-constructors)))))
                      :RULE-CLASSES (,@(and rule-=>-Px-tau-acceptable-p (list :TAU-SYSTEM)) :REWRITE))))
             (and rule-Px-=>
-                 `((DEFTHM ,(symbol-fns::suffix name '=> 'def) 
+                 `((DEFTHM ,(symbol-fns::suffix name '=> 'def)
                      ,rule-Px-=>
                      :HINTS (("Goal" :IN-THEORY (e/d (,(car Px)) (,@disabled))))
-                     :RULE-CLASSES (,@(and rule-Px-=>-tau-acceptable-p '(:TAU-SYSTEM)) 
+                     :RULE-CLASSES (,@(and rule-Px-=>-tau-acceptable-p '(:TAU-SYSTEM))
                                     ;,@(:REWRITE
-                                    ;:backchain-limit-lst 10) 
+                                    ;:backchain-limit-lst 10)
 ;HARSH: Check if it is a record (product?) type and also add a rewrite rule here
 ; Disable the constructor
                                     (:forward-chaining :trigger-terms (,Px))
                                     ; Do we want forward chaining here?
                                     )))))))
-     
 
-   
+
+
 
 (defloop tau-characterization-events0 (ps kwd-alist wrld)
   (for ((p in ps)) (append (tau-characterization-events1 p kwd-alist 'tau-characterization wrld))))
