@@ -830,6 +830,7 @@ details.</p>"
 
 
 (define vl-emodwire-fix ((x vl-emodwire-p))
+  :parents (vl-emodwire-p)
   :returns (x-prime vl-emodwire-p)
   :inline t
   :hooks nil
@@ -1110,9 +1111,7 @@ index of @('|reset|') is @('nil').</p>"
 
 ;; Introduce defaggregate like make-vl-emodwire and change-vl-emodwire macros.
 
-(make-event (std::da-make-maker-fn 'vl-emodwire '(basename index) nil))
-(make-event (std::da-make-maker 'vl-emodwire '(basename index)))
-(make-event (std::da-make-changer-fn 'vl-emodwire '(basename index)))
+(make-event (std::da-make-maker 'vl-emodwire '(basename index) nil))
 (make-event (std::da-make-changer 'vl-emodwire '(basename index)))
 
 
