@@ -68,7 +68,7 @@
                         (cw "GLCP interpreter error:~%~@0~%" msg)
                         (break$))))
     '(untrace$ glcp-interp-error-trace)))
-    
+
 
 (defmacro glcp-interp-abort (msg &key (nvals '1))
   `(mv ,@(make-list-ac nvals nil nil)
@@ -225,7 +225,7 @@
                   :stobjs ,*glcp-stobjs*))
         (b* ((pathcond (lbfr-hyp-fix pathcond))
              ((when (zp clk))
-              (glcp-interp-error "The clock ran out.~%"))
+              (glcp-interp-error "The clock ran out."))
              ((glcp-er xobj)
               (interp-term x alist contexts . ,*glcp-common-inputs*))
              ((unless (glcp-term-obj-p xobj))
@@ -249,7 +249,7 @@
                            :in-theory (e/d** ((:rules-of-class :executable-counterpart :here)
                                               acl2::open-nat-list-<
                                               len nfix fix
-                                              acl2::acl2-count-of-cons
+                                              acl2::acl2-count-of-cons-greater
                                               acl2::acl2-count-of-sum
                                               acl2-count-of-general-consp-car
                                               acl2-count-of-general-consp-cdr
@@ -775,7 +775,7 @@ but its arity is ~x3.  Its formal parameters are ~x4."
              ((when successp)
               (interp-test term bindings intro-bvars
                            . ,*glcp-common-inputs*))
-             
+
              (x (g-apply fn args))
              (look (get-term->bvar x bvar-db))
 
@@ -1038,7 +1038,7 @@ but its arity is ~x3.  Its formal parameters are ~x4."
             ((when contra)
              (cw "Constraints unsatisfiable~%")
              (glcp-value-nopathcond t))
-            
+
             ((unless pathcond-bfr)
              (glcp-value-nopathcond t))
             (pathcond-bfr (bfr-to-param-space pathcond-bfr pathcond-bfr)))
@@ -1159,13 +1159,16 @@ In ~@0: The hyp contains the following unbound variables: ~x1~%"
 In ~@0: the bindings don't satisfy shape-spec-bindingsp: ~x1"
                          id bindings)))
             (obj (strip-cadrs bindings))
-            ((unless (and (acl2::fast-no-duplicatesp (shape-spec-list-indices obj))
-                          (acl2::fast-no-duplicatesp-equal (shape-spec-list-vars obj))))
-             (flush-hons-get-hash-table-link obligs)
+            ((unless (acl2::fast-no-duplicatesp (shape-spec-list-indices obj)))
              (glcp-error
               (acl2::msg "~
-In ~@0: the indices or variables contain duplicates in bindings ~x1"
-                         id bindings)))
+In ~@0: the shape spec indices contain duplicates: ~x1"
+                         id (acl2::duplicated-members (shape-spec-list-indices obj)))))
+            ((unless (acl2::fast-no-duplicatesp (shape-spec-list-vars obj)))
+             (glcp-error
+              (acl2::msg "~
+In ~@0: the shape spec vars contain duplicates: ~x1"
+                         id (acl2::duplicated-members (shape-spec-list-vars obj)))))
             ((unless (subsetp-equal vars bound-vars))
              (flush-hons-get-hash-table-link obligs)
              (glcp-error
