@@ -31,6 +31,7 @@
 (in-package "GL")
 (include-book "bfr")
 (include-book "bfr-param")
+(include-book "bfr-reasoning")
 (include-book "centaur/misc/absstobjs" :dir :system)
 (include-book "tools/clone-stobj" :dir :system)
 (include-book "std/lists/index-of" :dir :system)
@@ -114,7 +115,7 @@
 ;;         (calist-remassoc x (cdr calist))
 ;;       (cons (car calist)
 ;;             (calist-remassoc x (cdr calist)))))
-                    
+
 ;;   ///
 ;;   (defthm calist-lookup-of-calist-remassoc
 ;;     (equal (calist-lookup k1 (calist-remassoc k2 calist))
@@ -308,7 +309,7 @@
                   t
                 (if (consp (car x))
                     (let ((rest (eval-constraint-alist
-                                 (calist-remassocs (cdr x) 
+                                 (calist-remassocs (cdr x)
                                                    (list (caar x)))
                                  env)))
                       (if (cdar x)
@@ -329,7 +330,7 @@
            (eval-constraint-alist x env))
     :hints(("Goal" :in-theory (enable eval-constraint-alist-aux
                                       calist-remassocs))))
-  
+
   (verify-guards eval-constraint-alist)
 
 
@@ -456,7 +457,7 @@
     (equal (constraint-alist->aig-aux x nil)
            (constraint-alist->aig x))
     :hints(("Goal" :in-theory (enable constraint-alist->aig-aux))))
-  
+
   (verify-guards constraint-alist->aig)
 
   (local (defthm set-equiv-of-cons-redundant
@@ -536,10 +537,10 @@
         (if (cdar x)
             (or (set::in (nfix v) (acl2::aig-vars (caar x)))
                 (constr-alist-depends-on
-                 v (calist-remassocs (cdr x) 
+                 v (calist-remassocs (cdr x)
                                      (list (caar x)))))
           (constr-alist-depends-on
-           v (calist-remassocs (cdr x) 
+           v (calist-remassocs (cdr x)
                                (list (caar x)))))
       (constr-alist-depends-on v (cdr x))))
   ///
@@ -555,7 +556,7 @@
   ;;          (constr-alist-depends-on v x))
   ;;   :hints(("Goal" :in-theory (enable constr-alist-depends-on-aux
   ;;                                     calist-remassocs))))
-  
+
   ;; (verify-guards constr-alist-depends-on)
 
 
@@ -617,11 +618,13 @@
        (car (aig-under-constraint-alist (car xabs) calist))
        ((when (eq car nil)) nil)
        (cdr (aig-under-constraint-alist (cdr xabs) calist))
-       (newx (acl2::aig-and car cdr))
+       ;; BOZO.  The new, smarter AIG-AND algorithm *might* work here if we can come
+       ;; up with a suitable measure.  For now, just use the dumb algorithm.
+       (newx (acl2::aig-and-dumb car cdr))
        (look (calist-lookup newx calist))
        ((when look) (eql 1 look)))
     newx)
-  /// 
+  ///
 
   (defthm aig-under-constraint-alist-of-shrink-constraint-alist
     (equal (aig-under-constraint-alist x (shrink-constraint-alist calist))
@@ -654,7 +657,7 @@
            (aig-under-constraint-alist x calist))
     :hints (("goal" :induct (aig-under-constraint-alist x calist))
             (and stable-under-simplificationp
-                 '(:in-theory (enable acl2::aig-and)))))
+                 '(:in-theory (enable acl2::aig-and-dumb)))))
 
   (defthm aig-under-constraint-alist-of-t-and-nil
     (and (equal (aig-under-constraint-alist t calist) t)
@@ -669,7 +672,7 @@
 ;;         (acl2::aig-not )))
 ;;     )
 
-;;   /// 
+;;   ///
 
 ;;   (defthm aig-under-constraint-alist-of-shrink-constraint-alist
 ;;     (equal (aig-under-constraint-alist x (shrink-constraint-alist calist))
@@ -744,7 +747,7 @@
     :hints(("Goal" :in-theory (enable shrink-constraint-alist calist-remassocs pairlis$)
             :expand ((shrink-constraint-alist calist)
                      (:free (seen) (calist-remassocs calist seen))))))
-  
+
   ;; (defcong constraint-alist-equiv constraint-alist-equiv
   ;;   (constraint-alist-delete-keys keys calist) 2)
 
@@ -874,7 +877,7 @@
                                       (cons k lst))
                     (calist-remassocs (shrink-constraint-alist calist)
                                       lst)))
-    :hints(("Goal" 
+    :hints(("Goal"
             :expand ((shrink-constraint-alist calist)
                      (:free (a b lst) (calist-remassocs (cons a b) lst))
                      (:free (lst) (calist-remassocs nil lst)))
@@ -1088,7 +1091,7 @@
   (defthm eval-of-bfr-hyp-init$a
     (equal (bfr-hyp-eval (bfr-hyp-init$a hyp$a) env) t)
     :hints(("Goal" :in-theory (enable bfr-hyp-eval)))))
-             
+
 
 
 (define bfr-assume$c (x hyp$c)
@@ -1399,7 +1402,7 @@
   (bfr-case
    :bdd nil
    :aig (shrink-constraint-alist nil)))
-  
+
 (define hyp$ap (hyp$a)
   (equal hyp$a (bfr-hyp-fix hyp$a))
   ///
