@@ -32,72 +32,85 @@
 
 (in-package "GL")
 (include-book "bvecs")
-(include-book "std/util/bstar" :dir :system)
-(include-book "tools/mv-nth" :dir :system)
-(include-book "ihs/logops-definitions" :dir :system)
+(include-book "bfr-reasoning")
 (local (include-book "clause-processors/find-subterms" :dir :system))
-(local (include-book "centaur/bitops/ihs-extensions" :dir :system))
+(local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (local (include-book "arith-lemmas"))
 
+(local (defthm equal-complexes-rw
+         (implies (and (acl2-numberp x)
+                       (rationalp a)
+                       (rationalp b))
+                  (equal (equal (complex a b) x)
+                         (and (equal a (realpart x))
+                              (equal b (imagpart x)))))
+         :hints (("goal" :use ((:instance realpart-imagpart-elim))))))
+
+
+(defsection symbolic-arithmetic
+  :parents (reference)
+  :short "Internal operations for computing on symbolic bit vectors."
+  :long "<p>Naming convention:</p>
+<ul>
+<li>B stands for a boolean variable.</li>
+<li>S stands for a signed bvec.</li>
+<li>U stands for an unsigned bvec.</li>
+<li>V stands for a generic bvec where signedness doesn't matter.</li>
+<li>N stands for a known natural constant.</li>
+</ul>
+
+<p>For instance, @('bfr-ite-bss-fn') has @('bss'), indicating that it's
+for computing:</p>
+
+@({
+     (ite Boolean Signed-Bvec Signed-Bvec)
+})")
+
+(local (xdoc::set-default-parents symbolic-arithmetic))
 
 ;;---------------- Misc function definitions and lemmas -------------------
 
-(defthm equal-complexes-rw
-  (implies (and (acl2-numberp x)
-                (rationalp a)
-                (rationalp b))
-           (equal (equal (complex a b) x)
-                  (and (equal a (realpart x))
-                       (equal b (imagpart x)))))
-  :hints (("goal" :use ((:instance realpart-imagpart-elim)))))
-
-(defun binary-- (x y)
-  (declare (xargs :guard (and (acl2-numberp x)
-                              (acl2-numberp y))))
-  (- x y))
-
-(defund int-set-sign (negp i)
-  (declare (xargs :guard (integerp i)))
+(define int-set-sign ((negp "True if we should set the sign bit to 1.")
+                      (i    integerp "The integer to modify."))
+  :short "Change the sign bit of an integer to a new value."
+  :returns (new-i integerp :rule-classes :type-prescription)
   (let ((i (lifix i)))
-    (acl2::logapp (integer-length i) i (if negp -1 0))))
+    (acl2::logapp (integer-length i) i (if negp -1 0)))
+  ///
+  (defthm sign-of-int-set-sign
+    (iff (< (int-set-sign negp i) 0)
+         negp)
+    :hints(("Goal" :in-theory (e/d* (int-set-sign)
+                                    (acl2::logapp
+                                     acl2::ifix-under-int-equiv))))))
 
-(defthm sign-of-int-set-sign
-  (iff (< (int-set-sign negp i) 0)
-       negp)
-  :hints(("Goal" :in-theory (e/d* (int-set-sign)
-                                  (acl2::logapp
-                                   acl2::ifix-under-int-equiv)))))
-
-(defthm int-set-sign-integerp
-  (integerp (int-set-sign negp i))
-  :rule-classes :type-prescription)
-
-(defund non-int-fix (x)
+(define non-int-fix (x)
+  :short "Identity for non-integers; coerces any integers to @('nil')."
   (declare (xargs :guard t))
-  (and (not (integerp x)) x))
+  (and (not (integerp x))
+       x)
+  ///
+  (defthm non-int-fix-when-non-integer
+    (implies (not (integerp x))
+             (equal (non-int-fix x) x))
+    :hints(("Goal" :in-theory (enable non-int-fix)))
+    :rule-classes ((:rewrite :backchain-limit-lst 0))))
 
-(defthm non-int-fix-when-non-integer
-  (implies (not (integerp x))
-           (equal (non-int-fix x) x))
-  :hints(("Goal" :in-theory (enable non-int-fix)))
-  :rule-classes ((:rewrite :backchain-limit-lst 0)))
-
-(defund maybe-integer (i x intp)
-  (declare (xargs :guard (integerp i)))
+(define maybe-integer ((i integerp) x intp)
   (if intp
       (ifix i)
-    (non-int-fix x)))
+    (non-int-fix x))
+  ///
+  (defthm maybe-integer-t
+    (equal (maybe-integer i x t)
+           (ifix i))
+    :hints(("Goal" :in-theory (enable maybe-integer))))
 
-(defthm maybe-integer-t
-  (equal (maybe-integer i x t)
-         (ifix i))
-  :hints(("Goal" :in-theory (enable maybe-integer))))
-
-(defthm maybe-integer-nil
-  (equal (maybe-integer i x nil)
-         (non-int-fix x))
-  :hints(("Goal" :in-theory (enable maybe-integer))))
+  (defthm maybe-integer-nil
+    (equal (maybe-integer i x nil)
+           (non-int-fix x))
+    :hints(("Goal" :in-theory (enable maybe-integer)))))
 
 ;;-------------------------- DEFSYMBOLIC -----------------------------------
 
@@ -302,7 +315,7 @@
                     :guard (and ,@(let ((guard (cadr (assoc-keyword :guard other-kws))))
                                     (and guard `(,guard)))
                                 . ,(defsymbolic-guards formals))))
-                  
+
                   (local (defun ,name ,formal-vars
                            (,exec-name . ,formal-vars)))
 
@@ -325,7 +338,7 @@
                 (defattach ,name ,exec-name)))
 
        (table defsymbolic-forms ',name ',args))))
-                  
+
 (defmacro defsymbolic (name &rest args)
   (defsymbolic-fn name args))
 
@@ -339,70 +352,113 @@
                            acl2::logext-identity
                            truncate)))
 
-(defthm true-listp-of-bfr-ucons
-  (implies (true-listp b)
-           (true-listp (bfr-ucons a b)))
-  :hints(("Goal" :in-theory (enable bfr-ucons)))
-  :rule-classes :type-prescription)
 
-(defthm true-listp-of-bfr-scons
-  (implies (true-listp b)
-           (true-listp (bfr-scons a b)))
-  :hints(("Goal" :in-theory (enable bfr-scons)))
-  :rule-classes :type-prescription)
+(defmacro car/cdr (x)
+  `(let* ((a ,x))
+     (mbe :logic (mv (car a) (cdr a))
+          :exec (if (atom a) (mv nil nil) (mv (car a) (cdr a))))))
+
+(defsymbolic bfr-ite-bvv-fn-aux ((c b) ;; name c, type b (boolean)
+                                 (v1 u) ;; unsigned
+                                 (v0 u))
+  :returns (vv u (if c v1 v0))
+  :abstract nil
+  :measure (+ (acl2-count v1) (acl2-count v0))
+  (b* (((when (and (atom v1) (atom v0)))
+        nil)
+       ((mv v11 v1r) (car/cdr v1))
+       ((mv v01 v0r) (car/cdr v0))
+       (tail (bfr-ite-bvv-fn-aux c v1r v0r))
+       (head (bfr-ite c v11 v01)))
+    (bfr-ucons head tail)))
 
 (defsymbolic bfr-ite-bvv-fn ((c b) ;; name c, type b (boolean)
                              (v1 u) ;; unsigned
                              (v0 u))
   :returns (vv u (if c v1 v0))
   :abstract nil
-  :measure (+ (acl2-count v1) (acl2-count v0))
-  (if (and (atom v1) (atom v0))
-      nil
-    (b* (((mv v11 v1r) (car/cdr v1))
-         ((mv v01 v0r) (car/cdr v0))
-         (tail (bfr-ite-bvv-fn c v1r v0r))
-         (head (bfr-ite c v11 v01)))
-      (bfr-ucons head tail))))
+  (if c
+      (if (eq c t)
+          (list-fix v1)
+        (bfr-ite-bvv-fn-aux c v1 v0))
+    (list-fix v0)))
+
+(defthm bfr-ite-bvv-fn-of-const-tests
+  (and (equal (bfr-ite-bvv-fn t v1 v0) (list-fix v1))
+       (equal (bfr-ite-bvv-fn nil v1 v0) (list-fix v0)))
+  :hints(("Goal" :in-theory (enable bfr-ite-bvv-fn))))
+
+(defthm bfr-ite-bvv-fn-aux-elim
+  (implies (and (not (equal c t))
+                c)
+           (equal (bfr-ite-bvv-fn-aux c v1 v0)
+                  (bfr-ite-bvv-fn c v1 v0)))
+  :hints(("Goal" :in-theory (enable bfr-ite-bvv-fn))))
 
 (defmacro bfr-ite-bvv (c v1 v0)
-  `(let ((bfr-ite-bvv-test ,c))
-     (if bfr-ite-bvv-test
-         (if (eq bfr-ite-bvv-test t)
-             ,v1
-           (bfr-ite-bvv-fn bfr-ite-bvv-test ,v1 ,v0))
-       ,v0)))
+  `(mbe :logic (bfr-ite-bvv-fn ,c ,v1 ,v0)
+        :exec (let ((bfr-ite-bvv-test ,c))
+                (if bfr-ite-bvv-test
+                    (if (eq bfr-ite-bvv-test t)
+                        (list-fix ,v1)
+                      (bfr-ite-bvv-fn-aux bfr-ite-bvv-test ,v1 ,v0))
+                  (list-fix ,v0)))))
 
 (add-macro-alias bfr-ite-bvv bfr-ite-bvv-fn)
 
-(defsymbolic bfr-ite-bss-fn ((c b) ;; name c, type b (boolean)
-                             (v1 s) ;; signed
-                             (v0 s))
+(defsymbolic bfr-ite-bss-fn-aux ((c  b) ;; name c, type b (boolean)
+                                 (v1 s) ;; signed
+                                 (v0 s))
   :returns (vv s (if c v1 v0))
   :abstract nil
   :measure (+ (acl2-count v1) (acl2-count v0))
   (b* (((mv head1 tail1 end1) (first/rest/end v1))
-       ((mv head0 tail0 end0) (first/rest/end v0)))
-    (if (and end1 end0)
-        (bfr-sterm (bfr-ite-fn c head1 head0))
-      (let ((rst (bfr-ite-bss-fn c tail1 tail0))
-            (head (bfr-ite c head1 head0)))
-        (bfr-scons head rst)))))
+       ((mv head0 tail0 end0) (first/rest/end v0))
+       ((when (and end1 end0))
+        (bfr-sterm (bfr-ite-fn c head1 head0)))
+       (rst (bfr-ite-bss-fn-aux c tail1 tail0))
+       (head (bfr-ite c head1 head0)))
+    (bfr-scons head rst)))
+
+(defsymbolic bfr-ite-bss-fn ((c  b) ;; name c, type b (boolean)
+                             (v1 s) ;; signed
+                             (v0 s))
+  :returns (vv s (if c v1 v0))
+  :abstract nil
+  (if c
+      (if (eq c t)
+          (list-fix v1)
+        (bfr-ite-bss-fn-aux c v1 v0))
+    (list-fix v0)))
+
+(defthm bfr-ite-bss-fn-of-const-tests
+  (and (equal (bfr-ite-bss-fn t v1 v0) (list-fix v1))
+       (equal (bfr-ite-bss-fn nil v1 v0) (list-fix v0)))
+  :hints(("Goal" :in-theory (enable bfr-ite-bss-fn))))
+
+(defthm bfr-ite-bss-fn-aux-elim
+  (implies (and (not (equal c t))
+                c)
+           (equal (bfr-ite-bss-fn-aux c v1 v0)
+                  (bfr-ite-bss-fn c v1 v0)))
+  :hints(("Goal" :in-theory (enable bfr-ite-bss-fn))))
 
 (defmacro bfr-ite-bss (c v1 v0)
-  `(let ((bfr-ite-bss-test ,c))
-     (if bfr-ite-bss-test
-         (if (eq bfr-ite-bss-test t)
-             ,v1
-           (bfr-ite-bss-fn bfr-ite-bss-test ,v1 ,v0))
-       ,v0)))
+  `(mbe :logic (bfr-ite-bss-fn ,c ,v1 ,v0)
+        :exec (let ((bfr-ite-bss-test ,c))
+                (if bfr-ite-bss-test
+                    (if (eq bfr-ite-bss-test t)
+                        (list-fix ,v1)
+                      (bfr-ite-bss-fn-aux bfr-ite-bss-test ,v1 ,v0))
+                  (list-fix ,v0)))))
 
 (add-macro-alias bfr-ite-bss bfr-ite-bss-fn)
-                        
+
 (defsymbolic bfr-loghead-ns ((n n)  ;; name n, type n (natp)
                              (x s)) ;; name x, type s (signed bvec)
   :returns (xx s (loghead n x))     ;; return name, type (signed bvec), spec
-  (b* (((when (zp n)) (bfr-sterm nil))
+  (b* (((when (zp n))
+        (bfr-sterm nil))
        ((mv head tail ?end) (first/rest/end x)))
     (bfr-scons head (bfr-loghead-ns (1- n) tail))))
 
@@ -410,20 +466,20 @@
                             (x s)) ;; name x, type s (signed bvec)
   :returns (xx s (acl2::logext n x))     ;; return name, type (signed bvec), spec
   :measure (acl2::pos-fix n)
-  (b* ((n (mbe :logic (acl2::pos-fix n) :exec n))
+  (b* ((n (lposfix n))
        ((mv head tail ?end) (first/rest/end x))
-       ((when end) (rlist-fix x))
+       ((when end) (list-fix x))
        ((when (eql n 1)) (bfr-sterm head)))
     (bfr-scons head (bfr-logext-ns (1- n) tail)))
   :correct-hints (("goal" :induct (bfr-logext-ns n x))
                   (And stable-under-simplificationp
-                       '(:expand ((:free (x) (acl2::logext (acl2::pos-fix n) x)))))))
+                       '(:expand ((:free (x) (logext (pos-fix n) x)))))))
 
 (defsymbolic bfr-logtail-ns ((place n)
                              (x s))
   :returns (xx s (logtail place x))
   (if (or (zp place) (s-endp x))
-      (rlist-fix x)
+      (list-fix x)
     (bfr-logtail-ns (1- place) (scdr x))))
 
 (defsymbolic bfr-+-ss ((c b)
@@ -434,23 +490,35 @@
   (b* (((mv head1 tail1 end1) (first/rest/end v1))
        ((mv head2 tail2 end2) (first/rest/end v2))
        (axorb (bfr-xor head1 head2))
-       (s (bfr-xor c axorb)))
-    (if (and end1 end2)
+       (s     (bfr-xor c axorb))
+       ((when (and end1 end2))
         (let ((last (bfr-ite axorb (bfr-not c) head1)))
-          (bfr-scons s (bfr-sterm last)))
-      (let* ((c (bfr-or (bfr-and c axorb)
-                        (bfr-and head1 head2)))
-             (rst (bfr-+-ss c tail1 tail2)))
-        (bfr-scons s rst))))
+          (bfr-scons s (bfr-sterm last))))
+       ;; BOZO think about this.  Using axorb here seems like a good idea since
+       ;; we're already computing it anyway in order to compute S.  However, we
+       ;; could instead do something like:
+       ;;    (c   (bfr-or  (bfr-and c head1)
+       ;;                  (bfr-and c head2)
+       ;;                  (bfr-and head1 head2)))
+       ;; This wouldn't share the same structure but might result in a simpler
+       ;; carry in being delivered to the rest of the sum, which might be a win.
+       ;; It's hard to guess whether this would be better or worse, so for now
+       ;; we'll just leave it alone...
+       (c (bfr-or (bfr-and c axorb)
+                  (bfr-and head1 head2)))
+       (rst (bfr-+-ss c tail1 tail2)))
+    (bfr-scons s rst))
   :correct-hints ('(:in-theory (enable logcons))))
 
+
+
 (defsymbolic bfr-lognot-s ((x s))
   :returns (nx s (lognot x))
-  (b* (((mv head tail end) (first/rest/end x)))
-    (if end
-        (bfr-sterm (bfr-not head))
-      (bfr-scons (bfr-not head)
-                 (bfr-lognot-s tail)))))
+  (b* (((mv head tail end) (first/rest/end x))
+       ((when end)
+        (bfr-sterm (bfr-not head))))
+    (bfr-scons (bfr-not head)
+               (bfr-lognot-s tail))))
 
 (defsymbolic bfr-unary-minus-s ((x s))
   :returns (ms s (- x))
@@ -462,25 +530,19 @@
   :returns (xab s (logxor a b))
   :measure (+ (len a) (len b))
   (b* (((mv af ar aend) (first/rest/end a))
-       ((mv bf br bend) (first/rest/end b)))
-    (if (and aend bend)
-        (bfr-sterm (bfr-xor af bf))
-      (b* ((c (bfr-xor af bf))
-           (r (bfr-logxor-ss ar br)))
-        (bfr-scons c r)))))
+       ((mv bf br bend) (first/rest/end b))
+       (c (bfr-xor af bf))
+       ((when (and aend bend))
+        (bfr-sterm c))
+       (r (bfr-logxor-ss ar br)))
+    (bfr-scons c r)))
 
 (defsymbolic bfr-sign-s ((x s))
   :returns (sign b (< x 0))
-  (b* (((mv first rest endp) (first/rest/end x)))
-    (if endp
-        first
-      (bfr-sign-s rest))))
-
-(defthm not-s-endp-compound-recognizer
-  (implies (not (s-endp x))
-           (consp x))
-  :hints(("Goal" :in-theory (enable s-endp)))
-  :rule-classes :compound-recognizer)
+  (b* (((mv first rest endp) (first/rest/end x))
+       ((when endp)
+        first))
+    (bfr-sign-s rest)))
 
 (defsymbolic bfr-integer-length-s1 ((offset p)
                                     (x s))
@@ -497,27 +559,23 @@
                                (equal (bfr-eval (car x) env)
                                       (equal 1 (logcar c)))))))
   (b* (((mv first rest end) (first/rest/end x))
-       (offset (mbe :logic (acl2::pos-fix offset) :exec offset)))
-    (if end
-        (mv nil nil)
-      (mv-let (changed res)
-        (bfr-integer-length-s1 (1+ offset) rest)
-        (if (eq changed t)
-            (mv t res)
-          (let ((change (bfr-xor first (car rest))))
-            (mv (bfr-or changed change)
-                (bfr-ite-bss changed
-                             res
-                             (bfr-ite-bss change
-                                          (i2v offset)
-                                          nil))))))))
+       (offset (lposfix offset))
+       ((when end)
+        (mv nil nil))
+       ((mv changed res)
+        (bfr-integer-length-s1 (1+ offset) rest))
+       ((when (eq changed t))
+        (mv t res))
+       (change (bfr-xor first (car rest))))
+    (mv (bfr-or changed change)
+        (bfr-ite-bss changed
+                     res
+                     (bfr-ite-bss change (i2v offset) nil))))
   :correct-hints ((bfr-reasoning)))
 
 (defsymbolic bfr-integer-length-s ((x s))
   :returns (ilen s (integer-length x))
-  (mv-let (ign res)
-    (bfr-integer-length-s1 1 x)
-    (declare (ignore ign))
+  (b* (((mv ?changed res) (bfr-integer-length-s1 1 x)))
     res))
 
 (define integer-length-bound-s (x)
@@ -556,26 +614,28 @@
                                      (integer-length-bound-s-correct
                                       bfr-sign-s-correct
                                       acl2::ihsext-redefs)))))
-                   
+
 (defsymbolic bfr-=-uu ((a u) (b u))
   :returns (a=b b (equal a b))
   :measure (+ (len a) (len b))
-  (if (and (atom a) (atom b))
-      t
-    (b* (((mv head1 tail1) (car/cdr a))
-         ((mv head2 tail2) (car/cdr b)))
-      (bfr-and (bfr-iff head1 head2)
-               (bfr-=-uu tail1 tail2)))))
+  (b* (((when (and (atom a) (atom b)))
+        t)
+       ((mv head1 tail1) (car/cdr a))
+       ((mv head2 tail2) (car/cdr b))
+       (first-eq (bfr-iff head1 head2)))
+    (bfr-and first-eq
+             (bfr-=-uu tail1 tail2))))
 
 (defsymbolic bfr-=-ss ((a s) (b s))
   :returns (a=b b (equal a b))
   :measure (+ (len a) (len b))
   (b* (((mv head1 tail1 end1) (first/rest/end a))
-       ((mv head2 tail2 end2) (first/rest/end b)))
-    (if (and end1 end2)
-        (bfr-iff head1 head2)
-      (bfr-and (bfr-iff head1 head2)
-               (bfr-=-ss tail1 tail2)))))
+       ((mv head2 tail2 end2) (first/rest/end b))
+       ((when (and end1 end2))
+        (bfr-iff head1 head2))
+       (first-eq (bfr-iff head1 head2)))
+    (bfr-and first-eq
+             (bfr-=-ss tail1 tail2))))
 
 (local (add-bfr-pat (bfr-sign-s . &)))
 (local (add-bfr-pat (bfr-=-ss . &)))
@@ -584,15 +644,15 @@
 (defsymbolic bfr-*-ss ((v1 s) (v2 s))
   :measure (+ (len v1) (len v2))
   :returns (prod s (* v1 v2))
-  (b* (((mv dig1 rest end1) (first/rest/end v1)))
-    (if end1
+  (b* (((mv dig1 rest end1) (first/rest/end v1))
+       ((when end1)
         (bfr-ite-bss dig1
                      (bfr-unary-minus-s v2)
-                     nil)
-      (let ((rest (bfr-*-ss rest v2)))
-        (bfr-+-ss nil
+                     nil))
+       (rest (bfr-*-ss rest v2)))
+    (bfr-+-ss nil
               (bfr-ite-bss dig1 v2 nil)
-              (bfr-scons nil rest)))))
+              (bfr-scons nil rest)))
   :correct-hints ('(:in-theory (enable logcons))))
 
 (defsymbolic bfr-<-=-ss ((a s) (b s))
@@ -600,42 +660,41 @@
   :returns (mv (a<b b (< a b))
                (a=b b (= a b)))
   (b* (((mv head1 tail1 end1) (first/rest/end a))
-       ((mv head2 tail2 end2) (first/rest/end b)))
-    (if (and end1 end2)
+       ((mv head2 tail2 end2) (first/rest/end b))
+       ((when (and end1 end2))
         (mv (bfr-and head1 (bfr-not head2))
-            (bfr-iff head1 head2))
-      (mv-let (rst< rst=)
-        (bfr-<-=-ss tail1 tail2)
-        (mv (bfr-or rst< (bfr-and rst= head2 (bfr-not head1)))
-            (bfr-and rst= (bfr-iff head1 head2)))))))
+            (bfr-iff head1 head2)))
+       ((mv rst< rst=)
+        (bfr-<-=-ss tail1 tail2)))
+    (mv (bfr-or rst< (bfr-and rst= head2 (bfr-not head1)))
+        (bfr-and rst= (bfr-iff head1 head2)))))
 
 (defsymbolic bfr-<-ss ((a s) (b s))
   :returns (a<b b (< a b))
   (b* (((mv head1 tail1 end1) (first/rest/end a))
-       ((mv head2 tail2 end2) (first/rest/end b)))
-    (if (and end1 end2)
-        (bfr-and head1 (bfr-not head2))
-      (mv-let (rst< rst=)
-        (bfr-<-=-ss tail1 tail2)
-        (bfr-or rst< (bfr-and rst= head2 (bfr-not head1)))))))
+       ((mv head2 tail2 end2) (first/rest/end b))
+       ((when (and end1 end2))
+        (bfr-and head1 (bfr-not head2)))
+       ((mv rst< rst=) (bfr-<-=-ss tail1 tail2)))
+    (bfr-or rst< (bfr-and rst= head2 (bfr-not head1)))))
 
 (defsymbolic bfr-logapp-nss ((n n)
                              (a s)
                              (b s))
   :returns (a-app-b s (logapp n a b))
-  (if (zp n)
-      (rlist-fix b)
-    (b* (((mv first rest &) (first/rest/end a)))
-      (bfr-scons first (bfr-logapp-nss (1- n) rest b)))))
+  (b* (((when (zp n))
+        (list-fix b))
+       ((mv first rest &) (first/rest/end a)))
+    (bfr-scons first (bfr-logapp-nss (1- n) rest b))))
 
 (defsymbolic bfr-logapp-nus ((n n)
-                        (a u)
-                        (b s))
+                             (a u)
+                             (b s))
   :returns (a-app-b s (logapp n a b))
-  (if (zp n)
-      (rlist-fix b)
-    (b* (((mv first rest) (car/cdr a)))
-      (bfr-scons first (bfr-logapp-nus (1- n) rest b)))))
+  (b* (((when (zp n))
+        (list-fix b))
+       ((mv first rest) (car/cdr a)))
+    (bfr-scons first (bfr-logapp-nus (1- n) rest b))))
 
 (defsymbolic bfr-ash-ss ((place p)
                     (n s)
@@ -661,15 +720,15 @@
                      (equal (+ n (- (* 2 n)) m)
                             (+ (- n) m))))))
   (b* (((mv shdig shrst shend) (first/rest/end shamt))
-       (place (mbe :logic (acl2::pos-fix place) :exec place)))
-    (if shend
+       (place (lposfix place))
+       ((when shend)
         (bfr-ite-bss shdig
                      (bfr-logtail-ns 1 n)
-                     (bfr-logapp-nss (1- place) nil n))
-      (let ((rst (bfr-ash-ss (* 2 place) n shrst)))
-        (bfr-ite-bss shdig
-                     rst
-                     (bfr-logtail-ns place rst)))))
+                     (bfr-logapp-nss (1- place) nil n)))
+       (rst (bfr-ash-ss (* 2 place) n shrst)))
+    (bfr-ite-bss shdig
+                 rst
+                 (bfr-logtail-ns place rst)))
   :correct-hints ('(:expand ((:free (b) (logcons b (bfr-list->s (scdr shamt) env)))
                              (bfr-ash-ss place n shamt))
                     :in-theory (disable acl2::logtail-identity
@@ -681,12 +740,12 @@
   :returns (bit b (logbitp (* place digit) n))
   :measure (len digit)
   (b* (((mv first & end) (first/rest/end n))
-       (place (mbe :logic (acl2::pos-fix place) :exec place)))
-    (if (or (atom digit) end)
-        first
-      (bfr-ite (car digit)
-               (bfr-logbitp-n2v (* 2 place) (cdr digit) (bfr-logtail-ns place n))
-               (bfr-logbitp-n2v (* 2 place) (cdr digit) n))))
+       (place (lposfix place))
+       ((when (or (atom digit) end))
+        first))
+    (bfr-ite (car digit)
+             (bfr-logbitp-n2v (* 2 place) (cdr digit) (bfr-logtail-ns place n))
+             (bfr-logbitp-n2v (* 2 place) (cdr digit) n)))
   :correct-hints ((and stable-under-simplificationp
                        '(:in-theory (enable logcons acl2::bool->bit)))))
 
@@ -695,36 +754,36 @@
   :returns (a&b s (logand a b))
   :measure (+ (len a) (len b))
   (b* (((mv af ar aend) (first/rest/end a))
-       ((mv bf br bend) (first/rest/end b)))
-    (if (and aend bend)
-        (bfr-sterm (bfr-and af bf))
-      (b* ((c (bfr-and af bf))
-           (r (bfr-logand-ss ar br)))
-        (bfr-scons c r)))))
+       ((mv bf br bend) (first/rest/end b))
+       (c (bfr-and af bf))
+       ((when (and aend bend))
+        (bfr-sterm c))
+       (r (bfr-logand-ss ar br)))
+    (bfr-scons c r)))
 
 (defsymbolic bfr-logior-ss ((a s)
                             (b s))
   :returns (avb s (logior a b))
   :measure (+ (len a) (len b))
   (b* (((mv af ar aend) (first/rest/end a))
-       ((mv bf br bend) (first/rest/end b)))
-    (if (and aend bend)
-        (bfr-sterm (bfr-or af bf))
-      (b* ((c (bfr-or af bf))
-           (r (bfr-logior-ss ar br)))
-        (bfr-scons c r)))))
+       ((mv bf br bend) (first/rest/end b))
+       (c (bfr-or af bf))
+       ((when (and aend bend))
+        (bfr-sterm c))
+       (r (bfr-logior-ss ar br)))
+    (bfr-scons c r)))
 
 (defsymbolic bfr-logeqv-ss ((a s)
                             (b s))
   :returns (a=b s (logeqv a b))
   :measure (+ (len a) (len b))
   (b* (((mv af ar aend) (first/rest/end a))
-       ((mv bf br bend) (first/rest/end b)))
-    (if (and aend bend)
-        (bfr-sterm (bfr-not (bfr-xor af bf)))
-      (b* ((c (bfr-not (bfr-xor af bf)))
-           (r (bfr-logeqv-ss ar br)))
-        (bfr-scons c r)))))
+       ((mv bf br bend) (first/rest/end b))
+       (c (bfr-iff af bf))
+       ((when (and aend bend))
+        (bfr-sterm c))
+       (r (bfr-logeqv-ss ar br)))
+    (bfr-scons c r)))
 
 (defsymbolic bfr-floor-ss-aux ((a s)
                              (b s)
@@ -958,19 +1017,19 @@
   :guard-hints ('(:expand ((:free (not-b) (bfr-floor-ss-aux a b not-b)))))
   (mbe :logic (non-exec (mv-nth 1 (bfr-floor-ss-aux a b not-b)))
        :exec (b* (((mv first rest endp) (first/rest/end a))
-                  (not-b (mbe :logic (bfr-lognot-s b) :exec not-b)))
-               (if endp
+                  (not-b (mbe :logic (bfr-lognot-s b) :exec not-b))
+                  ((when endp)
                    (bfr-ite-bss
                     first
                     (bfr-+-ss nil '(t) b) ;; (mod -1 b) = b-1 with b > 0
-                    '(nil)) ;; (mod  0  b) = 0
-                 (b* ((rm (bfr-mod-ss-aux rest b not-b))
-                      (rm (bfr-scons first rm))
-                      (less (bfr-<-ss rm b)))
-                   (bfr-ite-bss
-                    less rm
-                    (bfr-loghead-ns (integer-length-bound-s b)
-                                    (bfr-+-ss t not-b rm))))))))
+                    '(nil)))               ;; (mod  0  b) = 0
+                  (rm (bfr-mod-ss-aux rest b not-b))
+                  (rm (bfr-scons first rm))
+                  (less (bfr-<-ss rm b)))
+               (bfr-ite-bss
+                less rm
+                (bfr-loghead-ns (integer-length-bound-s b)
+                                (bfr-+-ss t not-b rm))))))
 
 (defsymbolic bfr-sign-abs-not-s ((x s))
   :returns (mv (s b (< x 0))
@@ -998,7 +1057,7 @@
   :returns (m s (mod a b))
   :prepwork ((local (in-theory (enable bfr-sign-abs-not-s))))
   (bfr-ite-bss (bfr-=-ss b nil)
-               (rlist-fix a)
+               (list-fix a)
                (bfr-logext-ns (integer-length-bound-s b)
                               (b* (((mv bsign babs bneg) (bfr-sign-abs-not-s b))
                                    (anorm (bfr-ite-bss bsign (bfr-unary-minus-s a) a))
@@ -1025,11 +1084,10 @@
   :prepwork ((local (in-theory (disable integer-length-of-between-abs-and-minus-abs
                                         logext-of-integer-length-bound
                                         rem
-                                        bitops::logbitp-upper-bound
                                         acl2::integer-length**)))
              (local (in-theory (enable bfr-sign-abs-not-s))))
   (bfr-ite-bss (bfr-=-ss b nil)
-               (rlist-fix a)
+               (list-fix a)
                (b* (((mv & babs bneg) (bfr-sign-abs-not-s b))
                     ((mv asign aabs &) (bfr-sign-abs-not-s a))
                     (m (bfr-mod-ss-aux aabs babs bneg)))
@@ -1045,10 +1103,130 @@
                                           integer-length-of-mod))))))
 
 
+(defun s-take (n x)
+  (declare (xargs :guard (natp n)))
+  (b* (((when (zp n)) (bfr-sterm nil))
+       ((mv first rest &) (first/rest/end x)))
+    (bfr-ucons first (s-take (1- n) rest))))
+
+(defthm deps-of-s-take
+  (implies (not (pbfr-list-depends-on k p x))
+           (not (pbfr-list-depends-on k p (s-take n x)))))
+
+
+(defthm s-take-correct
+  (equal (bfr-list->u (s-take n x) env)
+         (loghead n (bfr-list->s x env)))
+  :hints (("goal" :induct (s-take n x)
+           :in-theory (enable* acl2::ihsext-recursive-redefs))))
 
 
 
+(defsymbolic bfr-logapp-uss ((w n)
+                             (n u)
+                             (x s)
+                             (y s))
+  :returns (app s (logapp (* n w) x y))
+  :prepwork ((local (in-theory (enable logcons)))
+             (local (defthm logapp-loghead-logtail
+                      (implies (equal z (logapp w1 (logtail w x) y))
+                               (equal (logapp w (loghead w x) z)
+                                      (logapp (+ (nfix w) (nfix w1)) x y)))
+                      :hints(("Goal" :in-theory (enable* bitops::ihsext-recursive-redefs
+                                                         bitops::ihsext-inductions))))))
+  (if (atom n)
+      (list-fix y)
+    (if (b* (((mv x1 & xend) (first/rest/end x))
+             ((mv y1 & yend) (first/rest/end y)))
+          (and xend
+               yend
+               (equal x1 y1)))
+        (list-fix x)
+      (bfr-ite-bss
+       (car n)
+       (bfr-logapp-nus (lnfix w) (s-take w x)
+                       (bfr-logapp-uss
+                        (ash (lnfix w) 1) (cdr n) (bfr-logtail-ns w x)
+                        y))
+       (bfr-logapp-uss (ash (lnfix w) 1) (cdr n) x y)))))
 
 
 
+(local
+ (defsection expt-lemmas
+   (defthm expt-of-0
+     (equal (expt base 0) 1)
+     :hints(("Goal" :in-theory (enable expt))))
+
+
+   (defthm expt-of-*-2
+     (implies (natp exp)
+              (equal (expt base (* 2 exp))
+                     (* (expt base exp)
+                        (expt base exp))))
+     :hints(("Goal" :in-theory (enable expt))))))
+
+
+;; (local
+;;  (defthm expt-base-decompose
+;;    (implies (and (posp exp) (integerp base))
+;;             (equal (expt base exp)
+;;                    (* (if (eql (logcar exp) 1) base 1)
+;;                       (expt (* base base) (logcdr exp)))))
+;;    :hints (("goal" :use ((:instance acl2::logcar-logcdr-elim
+;;                           (i exp))
+;;                          (:instance acl2::exponents-add-for-nonneg-exponents
+;;                           (r base) (i (* 2 (logcdr exp))) (j (logcar exp)))
+;;                          (:instance acl2::exponents-add-for-nonneg-exponents
+;;                           (r base) (i (logcdr exp)) (j (logcdr exp))))
+;;             :in-theory (e/d (logcons) (acl2::logcar-logcdr-elim
+;;                                        bitops::logcons-destruct
+;;                                        acl2::exponents-add-for-nonneg-exponents))))))
+
+;; (define expt-impl ((base integerp)
+;;                    (exp natp))
+;;   :measure (nfix exp)
+;;   (if (mbe :logic (or (zp exp) (zp (logcdr exp)))
+;;            :exec (zp (logcdr exp)))
+;;       (if (eql (logcar exp) 1) base 1)
+;;     (let ((rest (expt-impl (* base base) (logcdr exp))))
+;;       (if (eql (logcar exp) 1)
+;;           (* base rest)
+;;         rest)))
+;;   ///
+
+;;   (defthm expt-impl-correct
+;;     (implies (and (integerp base) (natp exp))
+;;              (equal (expt-impl base exp)
+;;                     (expt base exp)))))
+
+
+(define all-nil ((x))
+  (if (atom x)
+      t
+    (and (eq (car x) nil)
+         (all-nil (cdr x))))
+  ///
+  (defthm all-nil-when-atom
+    (implies (atom x) (all-nil x)))
+
+  (defthmd zero-when-all-nil
+    (implies (all-nil x)
+             (equal (bfr-list->u x env) 0))))
+
+;; Note: We don't have a symbolic counterpart for expt yet, but this is used in
+;; SV and it could easily be used in GL as well so we wrote it here.
+(defsymbolic bfr-expt-su ((b s)
+                          (e u))
+  :measure (len e)
+  :returns (b^e s (expt b e))
+  (b* (((when (all-nil e)) '(t nil))
+       ((when (all-nil (cdr e)))
+        (bfr-ite-bss (car e) b '(t nil)))
+       (rest (bfr-expt-su (bfr-*-ss b b) (cdr e))))
+    (bfr-ite-bss (car e)
+                 (bfr-*-ss b rest)
+                 rest))
+  :correct-hints ('(:in-theory (enable zero-when-all-nil
+                                       logcons))))
 
