@@ -1,16 +1,20 @@
-;;;; $Id: usocket.lisp 719 2013-06-17 16:52:12Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.6.1/usocket.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
 (in-package :usocket)
 
 (defparameter *wildcard-host* #(0 0 0 0)
-  "Hostname to pass when all interfaces in the current system are to be bound.")
+  "Hostname to pass when all interfaces in the current system are to
+  be bound.  If this variable is passed to socket-listen, IPv6 capable
+  systems will also listen for IPv6 connections.")
 
 (defparameter *auto-port* 0
   "Port number to pass when an auto-assigned port number is wanted.")
 
+(defparameter *version* #.(asdf:component-version (asdf:find-system :usocket)))
+
 (defconstant +max-datagram-packet-size+ 65507
   "The theoretical maximum amount of data in a UDP datagram.
 
@@ -381,7 +385,7 @@ the values documented in usocket.lisp in the usocket class."
   `(octet-buffer-to-integer ,buffer 4 :start ,start))
 
 ;;
-;; IP(v4) utility functions
+;; IPv4 utility functions
 ;;
 
 (defun list-of-strings-to-integers (list)
@@ -443,13 +447,98 @@ such as 3232235777."
 (defmethod host-byte-order ((int integer))
   int)
 
+;;
+;; IPv6 utility functions
+;;
+
+(defun vector-to-ipv6-host (vector)
+  (with-output-to-string (*standard-output*)
+    (loop with zeros-collapsed-p
+          with collapsing-zeros-p
+          for i below 16 by 2
+          for word = (+ (ash (aref vector i) 8)
+                        (aref vector (1+ i)))
+          do (cond
+               ((and (zerop word)
+                     (not collapsing-zeros-p)
+                     (not zeros-collapsed-p))
+                (setf collapsing-zeros-p t))
+               ((or (not (zerop word))
+                    zeros-collapsed-p)
+                (when collapsing-zeros-p
+                  (write-string ":")
+                  (setf collapsing-zeros-p nil
+                        zeros-collapsed-p t))
+                (format t "~:[~;:~]~X" (plusp i) word)))
+          finally (when collapsing-zeros-p
+                    (write-string "::")))))
+
+(defun split-ipv6-address (string)
+  (let ((pos 0)
+        word
+        double-colon-seen-p
+        words-before-double-colon
+        words-after-double-colon)
+    (loop
+      (multiple-value-setq (word pos) (parse-integer string :radix 16 :junk-allowed t :start pos))
+      (labels ((at-end-p ()
+                 (= pos (length string)))
+               (looking-at-colon-p ()
+                 (char= (char string pos) #\:))
+               (ensure-colon ()
+                 (unless (looking-at-colon-p)
+                   (error "unsyntactic IPv6 address string ~S, expected a colon at position ~D"
+                          string pos))
+                 (incf pos)))
+        (cond
+          ((null word)
+           (when double-colon-seen-p
+             (error "unsyntactic IPv6 address string ~S, can only have one double-colon filler mark"
+                    string))
+           (setf double-colon-seen-p t))
+          (double-colon-seen-p
+           (push word words-after-double-colon))
+          (t
+           (push word words-before-double-colon)))
+        (if (at-end-p)
+            (return (list (nreverse words-before-double-colon) (nreverse words-after-double-colon)))
+            (ensure-colon))))))
+
+(defun ipv6-host-to-vector (string)
+  (assert (> (length string) 2) ()
+          "Unsyntactic IPv6 address literal ~S, expected at least three characters" string)
+  (destructuring-bind (words-before-double-colon words-after-double-colon)
+      (split-ipv6-address (concatenate 'string
+                                       (when (eql (char string 0) #\:)
+                                         "0")
+                                       string
+                                       (when (eql (char string (1- (length string))) #\:)
+                                         "0")))
+    (let ((number-of-words-specified (+ (length words-before-double-colon) (length words-after-double-colon))))
+      (assert (<= number-of-words-specified 8) ()
+              "Unsyntactic IPv6 address literal ~S, too many colon separated address components" string)
+      (assert (or (= number-of-words-specified 8) words-after-double-colon) ()
+              "Unsyntactic IPv6 address literal ~S, too few address components and no double-colon filler found" string)
+      (loop with vector = (make-array 16 :element-type '(unsigned-byte 8))
+            for i below 16 by 2
+            for word in (append words-before-double-colon
+                                (make-list (- 8 number-of-words-specified) :initial-element 0)
+                                words-after-double-colon)
+            do (setf (aref vector i) (ldb (byte 8 8) word)
+                     (aref vector (1+ i)) (ldb (byte 8 0) word))
+            finally (return vector)))))
+
 (defun host-to-hostname (host)
-  "Translate a string or vector quad to a stringified hostname."
+  "Translate a string, vector quad or 16 byte IPv6 address to a
+stringified hostname."
   (etypecase host
     (string host)
     ((or (vector t 4)
          (array (unsigned-byte 8) (4)))
      (vector-quad-to-dotted-quad host))
+    ((or (vector t 16)
+         (array (unsigned-byte 8) (16)))
+     (vector-to-ipv6-host host))
     (integer (hbo-to-dotted-quad host))
     (null "0.0.0.0")))
 
@@ -457,12 +546,10 @@ such as 3232235777."
   (etypecase ip1
     (string (string= ip1 (host-to-hostname ip2)))
     ((or (vector t 4)
-         (array (unsigned-byte 8) (4)))
-     (or (eq ip1 ip2)
-         (and (= (aref ip1 0) (aref ip2 0))
-              (= (aref ip1 1) (aref ip2 1))
-              (= (aref ip1 2) (aref ip2 2))
-              (= (aref ip1 3) (aref ip2 3)))))
+         (array (unsigned-byte 8) (4))
+         (vector t 16)
+         (array (unsigned-byte 8) (16)))
+     (equalp ip1 ip2))
     (integer (= ip1 (host-byte-order ip2)))))
 
 (defun ip/= (ip1 ip2)
@@ -559,11 +646,11 @@ Returns a usocket object.")
 ;;###FIXME: extend with default-element-type
 (setf (documentation 'socket-listen 'function)
       "Bind to interface `host' on `port'. `host' should be the
-representation of an ready-interface address.  The implementation is not
-required to do an address lookup, making no guarantees that hostnames
-will be correctly resolved.  If `*wildcard-host*' is passed for `host',
-the socket will be bound to all available interfaces for the IPv4
-protocol in the system.  `port' can be selected by the IP stack by
+representation of an ready-interface address.  The implementation is
+not required to do an address lookup, making no guarantees that
+hostnames will be correctly resolved.  If `*wildcard-host*' or NIL is
+passed for `host', the socket will be bound to all available
+interfaces for the system.  `port' can be selected by the IP stack by
 passing `*auto-port*'.
 
 Returns an object of type `stream-server-usocket'.
