@@ -64,7 +64,7 @@ This is especially useful in @(see acl2::hardware-verification).</p>
 off-the-shelf SAT solvers like <a
 href='http://fmv.jku.at/lingeling/'>lingeling</a>, <a
 href='https://www.lri.fr/~simon/?page=glucose'>glucose</a>, and so on; see
-@(see sat-solver-options).  It provides:</p>
+@(see sat-solver-options) for download and build help.  It provides:</p>
 
 <ul>
 
@@ -141,6 +141,26 @@ e.g., to print counterexamples.)</p>
 <li>Verify that @('glucose-3.0/simp/glucose --help') prints a help message</li>
 </ul>
 
+<p>(NOTE for Mac users: If you are building Glucose 3.0 or 4.0 on a Mac, the
+build might fail.  In that case, a solution may be to make the two replacements
+shown below, where the the first in each pair (@('<')) is the Mac version,
+while the second in each pair (@('>')) is the original source.  (Thanks to
+Marijn Heule for these suggestions.)</p>
+
+@({
+<     // friend Lit mkLit(Var var, bool sign = false);
+---
+>     friend Lit mkLit(Var var, bool sign = false);
+
+< inline  Lit  mkLit     (Var var, bool sign = false) { Lit p; p.x =
+var + var + (int)sign; return p; }
+---
+> inline  Lit  mkLit     (Var var, bool sign) { Lit p; p.x = var + var + (int)sign; return p; }
+
+})
+
+<p>End of NOTE for Mac users.)</p>
+
 <p>Now create a shell script, somewhere in your @('$PATH'), named @('glucose'):</p>
 
 @({
@@ -287,8 +307,8 @@ and the various files in @('centaur/regression').</p>")
 
   :long "<p>For higher confidence (at some cost to runtime), some SAT solvers
 are able to produce UNSAT proofs.  Small programs such as <a
-href='http://www.cs.utexas.edu/~marijn/drup/'>drup-trim</a> can check these
-proofs, to ensure the SAT solver reasoned correctly.</p>
+href='http://www.cs.utexas.edu/~marijn/drat-trim/'>drat-trim</a> can check
+these proofs, to ensure the SAT solver reasoned correctly.</p>
 
 <p>Satlink now includes Perl scripts that can make use of this capability for
 the Glucose and Riss3g solvers.  In particular, see the following scripts:</p>
@@ -307,8 +327,8 @@ the Glucose and Riss3g solvers.  In particular, see the following scripts:</p>
 <li>When Glucose reports SAT, we just exit (because Satlink can check the
 satisfying assignment itself); or</li>
 
-<li>When Glucose reports UNSAT, we check the proof using the Drup-Trim unsat
-proof checker.  We only print an \"s UNSATISFIABLE\" line if Drup-Trim says
+<li>When Glucose reports UNSAT, we check the proof using the Drat-Trim unsat
+proof checker.  We only print an \"s UNSATISFIABLE\" line if Drat-Trim says
 that the proof is ok.</li>
 
 </ul>
@@ -326,8 +346,9 @@ solver and the verifier in real time, interrupt it, etc.</p>
 <li>Install @('glucose') and/or @('riss3g') as described in @(see
 sat-solver-options), and</li>
 
-<li>Install the <a href='http://www.cs.utexas.edu/~marijn/drup/'>drup-trim</a>
-program as @('drup-trim') somewhere in your PATH.</li>
+<li>Install the <a
+href='http://www.cs.utexas.edu/~marijn/drat-trim/'>drat-trim</a> program as
+@('drat-trim') somewhere in your PATH.</li>
 
 </ul>
 
