@@ -27,44 +27,56 @@
 ;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
-;
-; doc.lisp -- this is a home for high level documentation topics; keeping
-; this basically separated from the VL source code is just a way to let me
-; edit these topics without having to rebuild stuff as often.
 
 (in-package "VL")
 (include-book "xdoc/top" :dir :system)
 
+; -----------------------------------------------------------------------------
+;
+;                   High-Level VL Documentation Topics
+;
+; -----------------------------------------------------------------------------
+;
+; This file is just a home for high level documentation topics.  Most of these
+; topics could easily be put elsewhere, but keeping them separated from the VL
+; source code is convenient because it allows us to edit them without having to
+; rebuild any of the rest of VL.
+
 (defxdoc vl
   :parents (hardware-verification)
   :short "The VL Verilog Toolkit is a large ACL2 library for working with <a
-href='http://en.wikipedia.org/wiki/Verilog'>Verilog</a> and <a
-href='http://en.wikipedia.org/wiki/SystemVerilog'>SystemVerilog</a> source
+href='http://en.wikipedia.org/wiki/SystemVerilog'>SystemVerilog</a> (and also
+regular <a href='http://en.wikipedia.org/wiki/Verilog'>Verilog</a>) source
 code, developed at <a href='http://www.centtech.com/'>Centaur Technology</a> by
 Jared Davis and Sol Swords.  It serves as a frontend for many Verilog tools."
 
-  :long "<box><p><b>Note</b>: this documentation is mainly a reference manual.
-If you are new to VL, please see @(see getting-started) first.</p></box>
+  :long "<box><p><b><color rgb='#ff0000'>ALPHA VERSION</color></b>.  The new
+development version of VL is not yet ready for public use and may change in
+drastic ways without any warning.  Users who want to be on the bleeding edge
+should follow the github project to try to keep up to date.  Alternately, see
+@(see vl2014::vl2014) for a more stable (but less fully featured) version of
+VL.</p></box>
 
-<p>Warning: VL is currently under heavy development and is not in a
-particularly usable state.  In the meantime, please see @(see vl2014::vl2014)
-for a more stable version.</p>")
+<box><p><b>Note</b>: this documentation is mainly a reference manual.
+If you are new to VL, please see @(see getting-started) first.</p></box>")
 
 (defxdoc getting-started
   :parents (vl)
-  :short "Getting started with VL."
+  :short "An introduction to VL, with suggested starting points for how to get
+started with evaluating it for use in your own projects."
 
   :long "<h3>Introduction</h3>
 
 <p><b>VL</b> is an @(see acl2::acl2) library for working with <a
-href='http://en.wikipedia.org/wiki/Verilog'>Verilog</a> and <a
-href='http://en.wikipedia.org/wiki/SystemVerilog'>SystemVerilog</a> source
-code.  It includes:</p>
+href='http://en.wikipedia.org/wiki/SystemVerilog'>SystemVerilog</a> (and also
+regular <a href='http://en.wikipedia.org/wiki/Verilog'>Verilog</a>) source
+code, developed at <a href='http://www.centtech.com/'>Centaur Technology</a> by
+Jared Davis and Sol Swords.  At a high level, VL includes:</p>
 
 <ul>
  <li>An internal representation for Verilog @(see syntax),</li>
  <li>A @(see loader) for parsing Verilog source code into this representation,</li>
- <li>Utilities for inspecting and analyzing parsed designs,</li>
+ <li>Utilities for inspecting, analyzing, and manipulating these designs,</li>
  <li>Various @(see transforms) that can simplify these designs, and</li>
  <li>Pretty-printing and other report-generation functions.</li>
 </ul>
@@ -75,18 +87,19 @@ to implement a family of Verilog-related tools.  Here are some examples:</p>
 
 <ul>
 
-<li>VL can build @(see esim) or @(see acl2::svex) models of Verilog modules for
-formal verification with ACL2.  This is the basis for much of Centaur's formal
+<li>VL can build @(see acl2::sv) models of Verilog modules for formal
+verification with ACL2.  This is the basis for much of Centaur's formal
 verification efforts.</li>
 
-<li>The VL @(see kit) is a standalone command-line executable that you can
-build on top of ACL2 and VL.  It includes commands for @(see lint)ing Verilog
-designs, converting Verilog modules into a JSON format, and other
-commands.</li>
+<li>The VL @(see kit) is a standalone command-line program that you can build
+on top of ACL2 and VL.  It provides various high-level commands to, e.g., @(see
+lint)ing, preprocess, collect source code into a single file, and so on.  It
+also provides an interactive shell for an instant way to start up ACL2 with VL
+already loaded.</li>
 
-<li>VL has been used to build a web-based \"module browser\" that lets you see
+<li>VL has been used to build a web-based ``module browser'' that lets you see
 the source code for our modules with, e.g., hyperlinks for navigating between
-wires and following wires.  This is now integrated into the VL @(see kit);
+wires and following wires.  This is now integrated into the VL @(see kit); see
 @(see server).</li>
 
 <li>(unreleased) We have used VL to implement <i>samev</i>, a sequential
@@ -103,8 +116,9 @@ href='http://www.cs.bham.ac.uk/research/projects/formare/events/aisb2013/'>Do-Fo
 
 </ul>
 
-<p>We imagine that other users of VL may wish to reuse its parsing and
-transformations to easily implement other tools.</p>
+<p>We imagine that parts of VL may be useful for implementing other
+SystemVerilog processing tools.</p>
+
 
 <h3>Starting Points</h3>
 
@@ -123,9 +137,9 @@ vl-ppcs-module) and perhaps more generally the VL @(see printer).</p>
 
 <p>After getting a feel for how modules are represented, it would be good to
 look at the available @(see transforms).  For instance, you might look at the
-code for @(see vl-simplify) to see the transforms used in the @(see esim) flow.
-You could also look at @('run-vl-lint-main') which uses a different
-transformation sequence for toward linting.</p>
+code for @('run-vl-lint-main') to see a transformation sequence geared toward
+linting.  You might also see @(see vl-design->svex-design) to see how the new
+@(see sv) flow works.</p>
 
 <p>If you are going to write any Verilog-processing tools of your own, you
 should probably read through how VL deals with @(see warnings) and then take a
@@ -143,8 +157,8 @@ level.</p>")
 href='http://standards.ieee.org/findstds/standard/1364-2005.html'>Verilog-2005
 standard, IEEE Std 1364-2005</a>.  When page and section numbers are used
 throughout the VL documentation, they are often in reference to this document.
-VL now also supports some fragment of SystemVerilog, based on our reading of
-the <a
+VL now also supports a significant fragment of SystemVerilog, based on our
+reading of the <a
 href='http://standards.ieee.org/findstds/standard/1800-2012.html'>SystemVerilog-2012
 Standard, IEEE 1800-2012</a>.</p>
 
@@ -159,7 +173,7 @@ designs.  It primarily handles RTL-based designs.  It has trouble with
 transistor-level constructs, hierarchical identifiers, inout ports, and fancy
 procedural statements.  It lacks support for most SystemVerilog features.</li>
 
-<li>The newer @(see acl2::svex) flow provides much better support for
+<li>The newer @(see acl2::sv) flow provides much better support for
 SystemVerilog features like structures, arrays, interfaces, and hierarchical
 identifiers.  It does not currently handle transistor-level constructs or
 simulation constructs like dynamic arrays, tasks, classes, etc.</li>
@@ -172,14 +186,16 @@ truly understand them.</li>
 </ul>
 
 <p>Regardless of the particular flow you are using, all VL tools reuse the same
-loader and internal representation.  The loader can read files as either
-Verilog-2005 or SystemVerilog-2012.  Regardless of the input type, we arrive at
-the same internal representation (essentially SystemVerilog).</p>
+loader, internal representation of Verilog @(see syntax).  The loader can read
+files as either Verilog-2005 or SystemVerilog-2012.  Regardless of the input
+type, we arrive at the same internal representation (essentially
+SystemVerilog), and supporting functionality like @(see mlib) and our @(see
+printer) all work on this same representation.</p>
 
 <p>VL's @(see preprocessor) is somewhat incomplete.  It basically just supports
 @('`define') and @('`ifdef')-related stuff.  It can @('`include') files in the
 style of the @('+incdir') options for tools like Verilog-XL, NCVerilog, and
-VCS.  VL also supports a notion of search paths, which are similar to
+VCS.  It also supports a notion of search paths, which are similar to
 @('+libdir') arguments.</p>
 
 <p>The @(see lexer) is essentially complete.</p>
@@ -220,9 +236,10 @@ modules.")
 working with Verilog modules.")
 
 
-(defxdoc warnings
-  :parents (vl)
-  :short "Support for handling warnings and errors."
+(defxdoc warning-basics
+  :parents (warnings)
+  :short "General introduction to @(see vl-warning) objects and error handling
+in VL."
 
   :long "<h3>Introduction</h3>
 
@@ -249,11 +266,11 @@ different sizes.</li>
 
 </ul>
 
-<p>Handling these many kinds of cases is tricky.  In the early days of VL, our
-approach to warnings and errors was quite ad-hoc.  We sometimes printed warning
-messages to standard output using the @(see cw) function.  For more serious
-conditions, we sometimes caused errors using @(see er).  This approach had a
-number of problems.  In particular,</p>
+<p>Handling these many kinds of cases is tricky.  In the earliest days of VL,
+our approach to warnings and errors was quite ad-hoc.  We sometimes printed
+warning messages to standard output using the @(see cw) function.  For more
+serious conditions, we sometimes caused errors using @(see er).  This ad-hoc
+approach had a number of problems.  In particular,</p>
 
 <ul>
 
@@ -287,28 +304,26 @@ approach to dealing with warnings and errors.</p>
 <h3>Warning Objects</h3>
 
 <p>Our new approach to warning and error handling centers around explicit @(see
-vl-warning-p) objects.</p>
-
-<p>These objects are in many ways similar to the <a
+vl-warning) objects.  These objects are in many ways similar to the <a
 href='https://en.wikipedia.org/wiki/Exception_handling'>Exception</a> objects
 found in other programming languages.  Each warning has a <b>type</b> and a
 <b>message</b> that describes the error.  These messages can conveniently make
 use of VL's @(see printer), so you can directly pretty-print arbitrary Verilog
 constructs when writing warning messages.</p>
 
-<p>We use @('vl-warning-p') objects universally, for all kinds of warnings and
-errors.  That is, everything from the most minor of code smells (@('wire foo')
-is never used for anything), to the most severe problems (the module you're
-instantiating isn't defined) results in a warning.  Since it is useful to
-distinguish minor commentary from severe problems, our warning objects include
-a <b>fatalp</b> field.</p>
+<p>We use @(see vl-warning) objects universally, for all kinds of warnings and
+errors.  That is, everything from the most minor of code smells (e.g., @('wire
+foo') is never used for anything), to the most severe problems (e.g., the
+module you're instantiating isn't defined) results in a warning.  To
+distinguish minor oddities from severe problems, our warning objects include a
+<b>fatalp</b> field.</p>
 
 <p>As a general philosophy or strategy for using these warning objects:</p>
 
 <ul>
 
-<li>Warnings messages should never be printed to standard output.  Instead, we
-should create a @(see vl-warning-p) object that gives the context, and explains
+<li>Warning messages should never be printed to standard output.  Instead, we
+should create a @(see vl-warning-p) object that provides context and explains
 the problem as clearly and concisely as possible.</li>
 
 <li>Errors should not cause sudden, unrecoverable exits.  That is, @(see er)
@@ -345,11 +360,10 @@ elements (e.g., module, package, interface, etc.) where it was caused.</li>
 
 <p>This approach allows us to easily do many practically useful things with the
 warnings.  For instance, it lets us easily filter out any modules that have
-fatal warnings&mdash;an important operation for @(see vl-simplify).  As another
-example, we can create reports, e.g., about all of the warnings for some
-particular module, or all the warnings of some particular type throughout all
-of the modules, etc.  These capability is used in tools like @(see
-vl-lint).</p>
+fatal warnings; see @(see propagating-errors).  As another example, we can
+create reports such as a @(see vl-reportcard) that summarize the warnings in
+our design.  These kinds of capabilities are especially useful in tools like
+@(see vl-lint).</p>
 
 <p>Practically implementing this philosophy is slightly tricky.</p>
 
@@ -377,3 +391,186 @@ of the module, and meanwhile the warnings are perhaps extended.  Finally, the
 function returns a new @(see vl-module-p) which is updated with the extended
 list of warnings.</p>")
 
+
+(defxdoc warnings
+  :parents (vl)
+  :short "Support for handling warnings and errors."
+
+  :long "<p>Many parts of VL can run into situations where we want to issue
+warnings or cause errors.  In these situations, VL creates @(see vl-warning)
+objects.  These warnings can be attached to major design elements such as
+modules and interfaces.  They can later be collected, examined, filtered,
+etc.</p>
+
+<p><b>New users:</b> You may wish to see @(see warning-basics) for a broad
+discussion of VL's philosophy toward handling warnings before looking at the
+functions below.</p>
+
+<h3>Quick Guide</h3>
+
+<p>VL includes many functions for creating, collecting, printing, filtering,
+and otherwise working with warnings.</p>
+
+<ul>
+
+<li>The core warning data structure is a @(see vl-warning), but more often
+these are found in a @(see vl-warninglist), which are often called <i>warnings
+accumulators</i>.</li>
+
+<li>For <b>creating warnings</b>, there are some convenient macros for extend
+warnings accumulators with new warnings; see @(see ok), @(see warn), @(see
+fatal).  As a new alternative to warnings accumulators, see also @(see vmsg),
+@(see vl-warning-add-ctx), and the special @(see patbind-wmv) binder for @(see
+b*).</li>
+
+<li>Functions for <b>displaying, filtering, etc.</b> a warning list include:
+<ul>
+  <li>@(see vl-warnings-to-string) and @(see vl-print-warnings)</li>
+  <li>@(see vl-clean-warnings) and @(see vl-warning-sort)</li>
+  <li>@(see vl-keep-warnings) and @(see vl-remove-warnings)</li>
+  <li>@(see vl-some-warning-fatalp) and @(see vl-some-warning-of-type-p).</li>
+</ul></li>
+
+<li>You can get a @(see vl-reportcard) that <b>summarizes the warnings</b> that
+are attached to design elements such as modules, interfaces, etc.  A report
+card can also be printed, filtered, etc., see @(see vl-reportcard) for details.
+You can alternately extract all of the warnings as a list of @(see
+flat-warnings).</li>
+
+<li>Fatal warnings can be <b>propagated</b> throughout a design to transitively
+any superior design elements as having fatal warnings somewhere below.  See
+@(see propagating-errors).</li>
+
+<li>VL hackers may want to know about @(see vl-trace-warnings), which just
+<b>traces warning creation</b> in a nice way so that you can see warnings as
+they are created (e.g., between print statements or other kinds of
+tracing.)</li>
+
+<li>Warnings are heavily used in VL's @(see lint)er, which features some
+special mechanisms for <b>suppressing warnings</b>; see @(see
+lint-warning-suppression).</li>
+
+</ul>")
+
+
+
+(defxdoc vl-patternkey-ambiguity
+  :parents (vl-patternkey supported-constructs)
+  :short "Notes about our handling of @(see vl-patternkey)s."
+
+  :long "<p>The keys in assignment patterns can be expressions (which should
+resolve to array indexes, for array patterns), structure member names, type
+names, or the special keyword @('default').  Here are some examples:</p>
+
+@({
+    '{ 0: a, 1: b, 2: c, default: 0 }    // assign to some array indices, default others...
+    '{ foo: 3, bar: 5 }                  // assign to struct members by name (maybe)
+    '{ integer: 5, opcode_t: 7 }         // assign to struct members by type (maybe)
+})
+
+<p>Simple names are particularly ambiguous here.  For instance, a name like
+@('place') might perhaps refer to any of (1) a parameter value to be used as an
+array index, as in:</p>
+
+@({
+     int arr [3:0];
+     parameter place = 0;
+     assign arr = '{place: 3, default: 0 };
+})
+
+<p>or (2) a structure member name, as in:</p>
+
+@({
+     typedef struct { int place; int value; } slot_t;
+     slot_t s = '{ place: 3, value: 4 };
+})
+
+<p>or (3) the name of some type, as in:</p>
+
+@({
+     typedef logic [3:0] place;
+     typedef struct { place src; place dest; int data; } msg_t;
+     msg_t m = '{ place: 0, data: 0 };
+})
+
+<p>This ambiguity is problematic in early parts of @(see annotate), such as
+@(see shadowcheck), where we want to check that, e.g., wires are used declared
+before they are used.  Here, because of type parameters, we might not be able
+to tell the names of a structure's members until elaboration time.  For
+example, suppose there is no type named @('place') and that, in some module, we
+are have a type parameter @('mytype_t') and an assignment like:</p>
+
+@({
+     mytype_t foo = '{ place: 0, ... }
+})
+
+<p>In this case, we won't know whether @('place') is the name of a structure
+field of @('mytype_t') until elaboration provides us with a definition for
+@('mytype_t').  If @('mytype_t') ends up having a member named @('place'), then
+it's fine for @('place') not to be declared here.  But if it doesn't have such
+a member, then the only way this makes sense is for @('place') to be the name
+of some parameter that's being used as an array index, in which case @('place')
+needs to be declared here.</p>
+
+<p>Rather than further intertwine shadowcheck and elaborate, our approach is to
+avoid this ambiguity by require that all simple names used in assignment
+pattern keys <b>must</b> be either the names of types or structure members.
+Although NCVerilog doesn't impose this restriction, it appears that our
+behavior matches that of VCS.  For instance, when we submit the following to
+VCS J-2014.12-SP3-1:</p>
+
+@({
+    module foo1 ;
+      int arr [3:0];
+      parameter place = 0;
+      assign arr = '{place: 3, default: 0 };
+    endmodule
+})
+
+<p>we find that it reports the following error:</p>
+
+@({
+    Error-[IAP] Illegal assignment pattern
+    test.sv, 5
+    foo1, \"place:3\"
+      Assignment pattern is illegal due to: Assignment Pattern with named fields 
+      cannot be assigned to a non-structure target
+})
+
+<h3>Implementation notes, disambiguation strategy</h3>
+
+<p>Upon parsing a @(see vl-patternkey) there are four possibilities:</p>
+
+<ol>
+<li>It is the @('default') keyword, which is no problem.</li>
+<li>It is unambiguously an index expression, e.g., @('3 + 5').</li>
+<li>It is unambiguously a built-in type expression, e.g., @('integer').</li>
+<li>It is ambiguously a simple name like @('foo'), which might be a structure
+member, type name, or parameter name.</li>
+</ol>
+
+<p>To reduce insanity we are going to assume that any such @('foo') must not
+be a parameter.</p>
+
+<p>At parse time, after dealing with @('default'), we try to parse an
+expression and then (to account for core type names like @('integer') which
+aren't expressions) fall back to trying to parse a type.  If we get anything
+other than a simple name then it's already unambiguous.</p>
+
+<p>On the other hand, if we get an expression which is a simple name, then we
+will <b>immediately convert it to a structmem</b> instead of an <b>expr</b>
+@(see vl-patternkey).  Note that we don't yet know whether it's a structure
+name or a type name; structmems will need to be further disambiguated before
+we're sure they aren't types.</p>
+
+<p>During shadowcheck, we have enough information to tell whether a structmem
+is actually a type.  We can then check for tricky shadowing as per usual.  Type
+disambiguation can then make the final conversion of structmems to types as
+necessary.</p>
+
+<p>Later on, in svex conversion: if we encounter a struct pattern, we should
+probably also explicitly check that any type keys such as @('foo_t') are NOT
+also the names of structure members.  If so, there might be confusion about
+what we are assigning to.  We might want this to be smart enough to handle
+things like @('struct mystruct { foo foo; bar bar; }'), or maybe those aren't
+handled by other Verilog tools anyway.</p>")
