@@ -35,10 +35,11 @@
 (include-book "../lint/check-namespace")
 (include-book "../mlib/hierarchy")
 
-(include-book "../lint/drop-missing-submodules")
+;(include-book "../lint/drop-missing-submodules")
 (include-book "../lint/drop-user-submodules")
-(include-book "../lint/lint-stmt-rewrite")
-(include-book "../lint/remove-toohard")
+;(include-book "../lint/lint-stmt-rewrite")
+;; Not used anymore (?)
+;; (include-book "../lint/remove-toohard")
 (include-book "../lint/suppress-warnings")
 (include-book "../lint/condcheck")
 (include-book "../lint/duplicate-detect")
@@ -46,34 +47,31 @@
 (include-book "../lint/duperhs")
 (include-book "../lint/leftright")
 (include-book "../lint/oddexpr")
-(include-book "../lint/portcheck")
 (include-book "../lint/qmarksize-check")
 (include-book "../lint/selfassigns")
 (include-book "../lint/skip-detect")
+(include-book "../lint/logicassign")
 
 (include-book "../transforms/cn-hooks")
 (include-book "../transforms/unparam/top")
-(include-book "../transforms/annotate/argresolve")
-(include-book "../transforms/annotate/resolve-indexing")
-(include-book "../transforms/annotate/origexprs")
-(include-book "../transforms/annotate/make-implicit-wires")
-(include-book "../transforms/annotate/portdecl-sign")
-(include-book "../transforms/annotate/udp-elim")
+(include-book "../transforms/annotate/top")
+(include-book "../transforms/addnames")
 
-(include-book "../transforms/assign-trunc")
-(include-book "../transforms/blankargs")
-(include-book "../transforms/clean-params")
+;(include-book "../transforms/assign-trunc")
+;(include-book "../transforms/blankargs")
+;(include-book "../transforms/clean-params")
 (include-book "../transforms/clean-warnings")
-(include-book "../transforms/drop-blankports")
-(include-book "../transforms/expr-split")
-(include-book "../transforms/expand-functions")
-(include-book "../transforms/oprewrite")
-(include-book "../transforms/resolve-ranges")
-(include-book "../transforms/replicate-insts")
-(include-book "../transforms/selresolve")
-(include-book "../transforms/sizing")
-(include-book "../transforms/unused-vars")
+;(include-book "../transforms/drop-blankports")
+;(include-book "../transforms/expr-split")
+;(include-book "../transforms/expand-functions")
+;(include-book "../transforms/oprewrite")
+;(include-book "../transforms/resolve-ranges")
+;(include-book "../transforms/replicate-insts")
+;(include-book "../transforms/selresolve")
+;(include-book "../transforms/sizing")
+;(include-book "../transforms/unused-vars")
 
+(include-book "centaur/sv/vl/moddb" :dir :system)
 (include-book "../../misc/sneaky-load")
 
 (include-book "../mlib/json")
@@ -385,7 +383,7 @@ shown.</p>"
                    NO-DUPLICATESP-EQUAL-WHEN-SAME-LENGTH-MERGESORT
                    SUBSETP-OF-VL-MODINSTLIST->MODNAMES-WHEN-SUBSETP
                    CDR-OF-VL-MODULELIST-DUPERHS-CHECK
-                   CDR-OF-VL-MODULELIST-ORIGEXPRS
+                   ;; CDR-OF-VL-MODULELIST-ORIGEXPRS
                    vl-modulelist-p-of-append
                    NO-DUPLICATESP-EQUAL-OF-APPEND
                    acl2::no-duplicatesp-equal-append-iff
@@ -451,7 +449,24 @@ shown.</p>"
   :returns (result vl-lintresult-p :hyp :fguard)
 
   (b* (((vl-lintconfig config) config)
-       (design (cwtime (vl-design-drop-user-submodules design config.dropmods)))
+       (design (vl-annotate-design design))
+
+       ;; We originally did this before annotate, but that doesn't work in the
+       ;; new world of loaditems.  we need to annotate first.
+       (design (xf-cwtime (vl-design-drop-user-submodules design config.dropmods)))
+
+       ;; BOZO where did lvaluecheck go??
+
+       ;; Taken care of by annotate
+       ;; (design (cwtime (vl-design-resolve-ansi-portdecls design)))
+       ;; (design (cwtime (vl-design-resolve-nonansi-interfaceports design)))
+       ;; (design (cwtime (vl-design-add-enumname-declarations design)))
+       ;; (design (cwtime (vl-design-make-implicit-wires design)))
+       ;; (design (cwtime (vl-design-portdecl-sign design)))
+       ;; (design (cwtime (vl-design-udp-elim design)))
+
+       ;; BOZO I have no idea why we're doing this.
+       ;; Old comments:
 
        ;; You might expect that we'd immediately throw out modules that we
        ;; don't need for topmods.  Historically we did that.  But then we found
@@ -462,33 +477,38 @@ shown.</p>"
        ;; mods0, so do that now:
        (design0 (vl-design-remove-unnecessary-modules config.topmods design))
 
-       (- (cw "~%vl-lint: initial processing...~%"))
-       (design (cwtime (vl-design-make-implicit-wires design)))
-       (design (cwtime (vl-design-portdecl-sign design)))
-       (design (cwtime (vl-design-udp-elim design)))
-       (design (cwtime (vl-design-portcheck design)))
-       (design (cwtime (vl-design-argresolve design)))
-       (design (cwtime (vl-design-resolve-indexing design)))
+       (design (xf-cwtime (vl-design-duplicate-detect design)))
+
+       ;; Note: don't want to addnames before duplicate-detect, because it
+       ;; would name unnamed duplicated blocks in different ways.
+       (design (xf-cwtime (vl-design-addnames design)))
+
+       ;; all done in annotate now:
+       ;; (design (cwtime (vl-design-portcheck design)))
+       ;; (design (cwtime (vl-design-argresolve design)))
+       ;; (design (cwtime (vl-design-type-disambiguate design)))
+
+       (design (xf-cwtime (vl-design-oddexpr-check design)))
+
+       ;; this goes away (design (cwtime (vl-design-resolve-indexing design)))
 
        ;; Pre-unparameterization Lucidity Check.
-       (design (cwtime (vl-design-lucid design
-                                        ;; This is a good time to check parameter uses
-                                        :paramsp t
-                                        ;; This is a bad time to check generates
-                                        :generatesp nil)))
+       (design (xf-cwtime (vl-design-lucid design
+                                           :modportsp t ;; Good time to check modports
+                                           :paramsp t   ;; Good time to check params
+                                           :generatesp nil ;; Bad time to check generates
+                                           )))
+
+       (design (xf-cwtime (vl-design-check-namespace design)))
+       (design (xf-cwtime (vl-design-check-case design)))
+       (design (xf-cwtime (vl-design-duperhs-check design)))
+       (design (xf-cwtime (vl-design-condcheck design)))
+       (design (xf-cwtime (vl-design-leftright-check design)))
+       (design (xf-cwtime (vl-design-dupeinst-check design)))
+       (design (xf-cwtime (vl-centaur-seqcheck-hook design)))
+       (design (xf-cwtime (vl-design-logicassign design)))
 
-       (- (cw "~%vl-lint: starting general checks...~%"))
-       (design (cwtime (vl-design-check-namespace design)))
-       (design (cwtime (vl-design-check-case design)))
-       (design (cwtime (vl-design-duperhs-check design)))
-       (design (cwtime (vl-design-duplicate-detect design)))
-       (design (cwtime (vl-design-condcheck design)))
-       (design (cwtime (vl-design-leftright-check design)))
-       (design (cwtime (vl-design-origexprs design)))
-       (design (cwtime (vl-design-dupeinst-check design)))
-       (design (cwtime (vl-centaur-seqcheck-hook design)))
 
-       (- (cw "~%vl-lint: elaborating the design...~%"))
 
        ;; BOZO we need to do something to throw away instances with unresolved
        ;; arguments to avoid programming-errors in drop-blankports... and actually
@@ -497,32 +517,37 @@ shown.</p>"
        ;;(design (cwtime (vl-design-follow-hids design)))
        ;; (design (cwtime (vl-design-clean-params design)))
        ;; (design (cwtime (vl-design-check-good-paramdecls design)))
-       (design (cwtime (vl-design-unparameterize design)))
-       (design (cwtime (vl-design-rangeresolve design)))
-       (design (cwtime (vl-design-selresolve design)))
+       (design (xf-cwtime (vl-design-elaborate design)))
+
+       ;; these are part of elaboration now
+       ;;(design (cwtime (vl-design-rangeresolve design)))
+       ;;(design (cwtime (vl-design-selresolve design)))
 
        (design
         ;; Running another dupeinst check here, after unparameterization, may
         ;; create redundant warnings, but it may also help to catch things that
         ;; become duplicates after unparameterization.
-        (cwtime (vl-design-dupeinst-check design)))
+        (xf-cwtime (vl-design-dupeinst-check design)))
 
        ;; Post-unparameterization Lucidity Check -- this is a bad time for
-       ;; checking parameters (because they've been eliminated) but it's a
-       ;; much better time to do bit-level analysis, because things like
-       ;; foo[width-1:0] should hopefully be resolved now.  Also we can
-       ;; sensibly check generates now.
-       (design (cwtime (vl-design-lucid design
-                                        :paramsp nil
-                                        :generatesp t)))
+       ;; checking parameters (because they've been eliminated) and modports
+       ;; (because the interface names have changed and we won't be able to
+       ;; find them correctly).  But it's a much better time to do bit-level
+       ;; analysis, because things like foo[width-1:0] should hopefully be
+       ;; resolved now.  Also we can sensibly check generates now.
+       (design (xf-cwtime (vl-design-lucid design
+                                           :modportsp nil
+                                           :paramsp nil
+                                           :generatesp t)))
 
-       (design
-        ;; Best not to do this until after lucid checking.
-        (cwtime (vl-design-drop-missing-submodules design)))
+;;*** do we want to do thsi???  I can't think of why
+       ;; (design
+       ;;  ;; Best not to do this until after lucid checking.
+       ;;  (cwtime (vl-design-drop-missing-submodules design)))
 
        ;; BOZO do we even need to do this?
        ;; BOZO not exactly sure where this should go, maybe this will work.
-       
+
 
        ;; (design
        ;;  ;; Bug fixed 2014-12-19: don't do this until after argresolve, because
@@ -532,26 +557,32 @@ shown.</p>"
        ;;  ;; are we even doing this?
        ;;  (cwtime (vl-design-elim-unused-vars design)))
 
-       (design (cwtime (vl-design-check-selfassigns design)))
-       (design (cwtime (vl-design-lint-stmt-rewrite design)))
-       (design (cwtime (vl-design-stmtrewrite design 1000)))
+       (design (xf-cwtime (vl-design-check-selfassigns design)))
+       (design (xf-cwtime (vl-design-qmarksize-check design)))
+       (sd-probs (xf-cwtime (sd-analyze-design design0)))
+
+;; Not sure we care abotu this for anything
+       ;; (design (cwtime (vl-design-lint-stmt-rewrite design)))
+       ;; (design (cwtime (vl-design-stmtrewrite design 1000)))
        ;;(design (cwtime (vl-design-hid-elim design)))
 
-       ;; Now that HIDs are gone, we can throw away any modules we don't care
-       ;; about, if we have been given any topmods.
-       (design (b* ((names1 (vl-modulelist->names (vl-design->mods design)))
-                    (design (vl-design-drop-missing-submodules design))
-                    (names2 (vl-modulelist->names (vl-design->mods design)))
-                    (lost   (difference (mergesort names1) (mergesort names2))))
-                 (or (not lost)
-                     (cw "BOZO lost ~x0 modules somewhere (probably unparameterizing): ~x1~%"
-                         (len lost) lost))
-                 design))
+;; maaaaybe we don't watn this?
+       ;; ;; Now that HIDs are gone, we can throw away any modules we don't care
+       ;; ;; about, if we have been given any topmods.
+       ;; (design (b* ((names1 (vl-modulelist->names (vl-design->mods design)))
+       ;;              (design (vl-design-drop-missing-submodules design))
+       ;;              (names2 (vl-modulelist->names (vl-design->mods design)))
+       ;;              (lost   (difference (mergesort names1) (mergesort names2))))
+       ;;           (or (not lost)
+       ;;               (cw "BOZO lost ~x0 modules somewhere (probably unparameterizing): ~x1~%"
+       ;;                   (len lost) lost))
+       ;;           design))
 
-       (design (cwtime (vl-design-remove-unnecessary-modules config.topmods design)))
+       ((mv reportcard ?modalist) (xf-cwtime (vl-design->svex-modalist design)))
+       (design (xf-cwtime (vl-apply-reportcard design reportcard)))
+
+       (design (xf-cwtime (vl-design-remove-unnecessary-modules config.topmods design)))
 
-       (- (cw "~%vl-lint: processing expressions...~%"))
-       (design (cwtime (vl-design-oddexpr-check design)))
 
        ;; [Jared] -- Trying to NOT do oprewrite anymore.  Our sizing warnings
        ;; do better if we can tell the difference between == and ~^ operators,
@@ -560,28 +591,31 @@ shown.</p>"
 
 
        ;; Sizing doesn't do well unless we expand functions
-       (design (cwtime (vl-design-expand-functions design)))
-       (design (cwtime (vl-design-exprsize design)))
-       (design (cwtime (vl-design-constcheck-hook design config.cclimit)))
-       (design (cwtime (vl-design-qmarksize-check design)))
+;       (design (cwtime (vl-design-expand-functions design)))
+;       (design (cwtime (vl-design-exprsize design)))
+;       (design (cwtime (vl-design-constcheck-hook design config.cclimit)))
 
-       (- (cw "~%vl-lint: processing assignments...~%"))
-       (design (cwtime (vl-design-split design)))
-       (design (cwtime (vl-design-replicate design)))
-       (design (cwtime (vl-design-blankargs design)))
-       (design (cwtime (vl-design-trunc design)))
 
-       (- (cw "~%vl-lint: finding skipped and multiply driven wires...~%"))
+;       (- (cw "~%vl-lint: processing assignments...~%"))
+;       (design (cwtime (vl-design-split design)))
+
+; BOZO argument width checking?
+;       (design (cwtime (vl-design-replicate design)))
+
+;; BOZO are we checking for connecting blanks to interface ports
+;;        (design (cwtime (vl-design-blankargs design)))
+;;       (design (cwtime (vl-design-trunc design)))
+
+;       (- (cw "~%vl-lint: finding skipped and multiply driven wires...~%"))
        ;; NOTE: use design0, not design, if you ever want this to finish. :)
-       (sd-probs (cwtime (sd-analyze-design design0)))
 
-       (- (cw "~%vl-lint: cleaning up...~%"))
-       (design   (cwtime (vl-design-clean-warnings design)))
-       (design   (cwtime (vl-design-suppress-lint-warnings design)))
-       (design   (cwtime (vl-design-lint-ignoreall design config.ignore)))
-       (design   (cwtime (vl-lint-apply-quiet config.quiet design)))
-       (sd-probs (cwtime (vl-delete-sd-problems-for-modnames config.quiet sd-probs)))
-       (reportcard  (cwtime (vl-design-origname-reportcard design))))
+
+       (design   (xf-cwtime (vl-design-clean-warnings design)))
+       (design   (xf-cwtime (vl-design-suppress-lint-warnings design)))
+       (design   (xf-cwtime (vl-design-lint-ignoreall design config.ignore)))
+       (design   (xf-cwtime (vl-lint-apply-quiet config.quiet design)))
+       (sd-probs (xf-cwtime (vl-delete-sd-problems-for-modnames config.quiet sd-probs)))
+       (reportcard  (xf-cwtime (vl-design-origname-reportcard design))))
 
     (make-vl-lintresult :design design
                         :design0 design0
@@ -754,7 +788,9 @@ shown.</p>"
         :vl-warn-instances-same
         :vl-warn-case-sensitive-names
         :vl-warn-same-rhs
-        :vl-const-expr))
+        :vl-const-expr
+        :vl-warn-vardecl-assign
+        ))
 
 (defconst *smell-minor-warnings*
   (list :vl-warn-partselect-same
