@@ -92,15 +92,27 @@
  `(defstobj tokstream
     (tokens :type (satisfies vl-tokenlist-p)
             :initially nil)
+    (position :type (integer 0 *)
+              :initially 0)
     (pstate :type (satisfies vl-parsestate-p)
             :initially ,(make-vl-parsestate))
     :inline t
     :non-memoizable t
     :renaming
-    ((tokens        vl-tokstream->tokens-raw)
-     (pstate        vl-tokstream->pstate-raw)
-     (update-tokens vl-tokstream-update-tokens-fn)
-     (update-pstate vl-tokstream-update-pstate-fn))))
+    ((tokens          vl-tokstream->tokens-raw)
+     (pstate          vl-tokstream->pstate-raw)
+     (position        vl-tokstream->position-raw)
+     (update-tokens   vl-tokstream-update-tokens-fn)
+     (common-lisp::update-position vl-tokstream-update-position-fn)
+     (update-pstate   vl-tokstream-update-pstate-fn)
+     (common-lisp::positionp       vl-tokstream-positionp))))
+
+(defthm vl-tokstream-positionp-rw
+  (equal (vl-tokstream-positionp x)
+         (natp x))
+  :rule-classes (:rewrite :compound-recognizer))
+
+(in-theory (disable vl-tokstream-positionp))
 
 (define vl-tokstream->tokens (&key (tokstream 'tokstream))
   :returns (tokens vl-tokenlist-p)
@@ -108,6 +120,11 @@
   (mbe :logic (vl-tokenlist-fix (vl-tokstream->tokens-raw tokstream))
        :exec (vl-tokstream->tokens-raw tokstream)))
 
+(define vl-tokstream->position (&key (tokstream 'tokstream))
+  :returns (position natp :rule-classes :type-prescription)
+  :inline t
+  (lnfix (vl-tokstream->position-raw tokstream)))
+
 (define vl-tokstream->pstate (&key (tokstream 'tokstream))
   :returns (pstate vl-parsestate-p)
   :inline t
@@ -120,18 +137,31 @@
 (defmacro vl-tokstream-update-pstate (pstate &key (tokstream 'tokstream))
   `(vl-tokstream-update-pstate-fn ,pstate ,tokstream))
 
+(defmacro vl-tokstream-update-position (position &key (tokstream 'tokstream))
+  `(vl-tokstream-update-position-fn ,position ,tokstream))
+
 (add-macro-alias vl-tokstream-update-tokens vl-tokstream-update-tokens-fn)
 (add-macro-alias vl-tokstream-update-pstate vl-tokstream-update-pstate-fn)
+(add-macro-alias vl-tokstream-update-position vl-tokstream-update-position-fn)
 
 (in-theory (disable vl-tokstream-update-tokens
-                    vl-tokstream-update-pstate))
+                    vl-tokstream-update-pstate
+                    vl-tokstream-update-position))
+
+(define vl-tokstream-pop (&key (tokstream 'tokstream))
+  :guard (consp (vl-tokstream->tokens))
+  :inline t
+  (b* ((tokstream (vl-tokstream-update-tokens (cdr (vl-tokstream->tokens)))))
+    (vl-tokstream-update-position (+ 1 (vl-tokstream->position)))))
 
 (encapsulate
   ()
   (local (in-theory (enable vl-tokstream->tokens
                             vl-tokstream->pstate
+                            vl-tokstream->position
                             vl-tokstream-update-tokens
-                            vl-tokstream-update-pstate)))
+                            vl-tokstream-update-pstate
+                            vl-tokstream-update-position)))
 
   (defthm vl-tokstream->tokens-of-vl-tokstream-update-tokens
     (equal (vl-tokstream->tokens :tokstream (vl-tokstream-update-tokens new-tokens))
@@ -145,22 +175,35 @@
     (equal (vl-tokstream->tokens :tokstream (vl-tokstream-update-pstate new-pstate))
            (vl-tokstream->tokens :tokstream tokstream)))
 
+  (defthm vl-tokstream->tokens-of-vl-tokstream-update-position
+    (equal (vl-tokstream->tokens :tokstream (vl-tokstream-update-position new-position))
+           (vl-tokstream->tokens :tokstream tokstream)))
+
   (defthm vl-tokstream->pstate-of-vl-tokstream-update-pstate
     (equal (vl-tokstream->pstate :tokstream (vl-tokstream-update-pstate new-pstate))
-           (vl-parsestate-fix new-pstate))))
+           (vl-parsestate-fix new-pstate)))
+
+  (defthm vl-tokstream->tokens-of-vl-tokstream-pop
+    (equal (vl-tokstream->tokens :tokstream (vl-tokstream-pop))
+           (cdr (vl-tokstream->tokens :tokstream tokstream)))
+    :hints(("Goal" :in-theory (enable vl-tokstream-pop)))))
+
+
 
 (defmacro vl-tokstream-measure (&key (tokstream 'tokstream))
   `(len (vl-tokstream->tokens :tokstream ,tokstream)))
 
 (define vl-tokstream-fix (&key (tokstream 'tokstream))
   (mbe :logic (b* ((tokens (vl-tokstream->tokens))
-                   (pstate (vl-tokstream->pstate)))
+                   (pstate (vl-tokstream->pstate))
+                   (position (vl-tokstream->position)))
                 (non-exec
-                 (list tokens pstate)))
+                 (list tokens position pstate)))
        :exec tokstream)
   :prepwork
   ((local (in-theory (enable vl-tokstream->pstate
-                             vl-tokstream->tokens))))
+                             vl-tokstream->tokens
+                             vl-tokstream->position))))
   ///
   (defthm vl-tokstream->pstate-of-vl-tokstream-fix
     (equal (vl-tokstream->pstate :tokstream (vl-tokstream-fix))
@@ -170,6 +213,10 @@
     (equal (vl-tokstream->tokens :tokstream (vl-tokstream-fix))
            (vl-tokstream->tokens)))
 
+  (defthm vl-tokstream->position-of-vl-tokstream-fix
+    (equal (vl-tokstream->position :tokstream (vl-tokstream-fix))
+           (vl-tokstream->position)))
+
   (defthm vl-tokstream-measure-of-vl-tokstream-fix
     (equal (vl-tokstream-measure :tokstream (vl-tokstream-fix))
            (vl-tokstream-measure))))
@@ -178,11 +225,13 @@
 (defaggregate vl-tokstream-backup
   :tag nil
   ((tokens vl-tokenlist-p)
+   (position natp :rule-classes :type-prescription)
    (pstate vl-parsestate-p)))
 
 (define vl-tokstream-save (&key (tokstream 'tokstream))
   :returns (backup vl-tokstream-backup-p)
   (make-vl-tokstream-backup :tokens (vl-tokstream->tokens)
+                            :position (vl-tokstream->position)
                             :pstate (vl-tokstream->pstate))
   ///
   (defthm vl-tokstream-backup->tokens-of-vl-tokstream-save
@@ -202,13 +251,16 @@
             (vl-parsestate-free curr-pstate)))
        (new-pstate (vl-parsestate-restore backup.pstate))
        (tokstream (vl-tokstream-update-pstate new-pstate))
+       (tokstream (vl-tokstream-update-position backup.position))
        (tokstream (vl-tokstream-update-tokens backup.tokens)))
     (vl-tokstream-fix))
   ///
   (local (in-theory (enable vl-tokstream->tokens
                             vl-tokstream->pstate
+                            vl-tokstream->position
                             vl-tokstream-update-tokens
                             vl-tokstream-update-pstate
+                            vl-tokstream-update-position
                             vl-tokstream-fix
                             vl-tokstream-save
                             vl-parsestate-restore)))
@@ -216,11 +268,25 @@
     (equal (vl-tokstream-restore (vl-tokstream-save) :tokstream anything)
            (vl-tokstream-fix))))
 
+(define vl-add-context-to-parser-warning ((warning vl-warning-p)
+                                          &key (tokstream 'tokstream))
+  :returns (new-warning vl-warning-p)
+  (b* ((tokens  (vl-tokstream->tokens))
+       (context (cat "  Near: \""
+                     (vl-tokenlist->string-with-spaces
+                      (take (min 4 (len tokens))
+                            (list-fix tokens)))
+                     (if (> (len tokens) 4) "..." "")
+                     "\""))
+       (new-msg (cat (vl-warning->msg warning)
+                     (str::strsubst "~" "~~" context))))
+    (change-vl-warning warning :msg new-msg)))
 
 (define vl-tokstream-add-warning ((warning vl-warning-p)
                                   &key (tokstream 'tokstream))
   :returns (new-tokstream)
-  (b* ((pstate (vl-tokstream->pstate))
+  (b* ((warning (vl-add-context-to-parser-warning warning))
+       (pstate (vl-tokstream->pstate))
        (pstate (vl-parsestate-add-warning warning pstate))
        (tokstream (vl-tokstream-update-pstate pstate)))
     tokstream)
@@ -672,6 +738,7 @@ frequently.</p>")
   :short "Check whether the current token has a particular type."
   ((type "Type of token to match.")
    &key (tokstream 'tokstream))
+  :guard (symbolp type)
   :returns bool
   :inline t
   (b* ((tokens (vl-tokstream->tokens)))
@@ -765,19 +832,22 @@ execution) that includes the current location."
    ((function symbolp) '__function__)
    (tokstream 'tokstream))
   :returns
-  (mv (errmsg   vl-warning-p)
+  (mv (errmsg   vl-warning-p
+                "Note that such a warning is not automatically extended with
+                 ``nearby'' context, because for backtracking these are often
+                 ephemeral.")
       (value    "Always just @('nil')." (equal value nil))
       (new-tokstream (equal new-tokstream tokstream)))
-  (b* ((tokens (vl-tokstream->tokens)))
-    (mv (make-vl-warning :type :vl-parse-error
+  (b* ((tokens  (vl-tokstream->tokens))
+       (warning (make-vl-warning :type :vl-parse-error
                          :msg "Parse error at ~a0: ~s1"
                          :args (list (if (consp tokens)
                                          (vl-token->loc (car tokens))
                                        "EOF")
                                      description)
                          :fn function
-                         :fatalp t)
-        nil tokstream))
+                         :fatalp t)))
+    (mv warning nil tokstream))
   ///
   (more-returns (errmsg (iff errmsg t)
                         :name vl-parse-error-0-under-iff)))
@@ -843,7 +913,7 @@ the start of the input stream."
                              :fatalp t
                              :fn function)
             nil tokstream))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil token1 tokstream))
 
   :prepwork
@@ -915,7 +985,7 @@ acceptable types."
                              :fatalp t
                              :fn function)
             nil tokstream))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil token1 tokstream))
 
   :prepwork
@@ -967,7 +1037,7 @@ the given type, but if not, don't consume anything and return nil."
        (token1 (car tokens))
        ((unless (eq type (vl-token->type token1)))
         (mv nil nil tokstream))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil token1 tokstream))
 
   :prepwork
@@ -1079,6 +1149,17 @@ the @('member') term never arose.</p>
                   (vl-type-of-matched-token (remove-equal exclude types) tokens)))
   :hints(("Goal" :in-theory (enable vl-type-of-matched-token))))
 
+(defthm magically-reduce-possible-types-from-vl-token->type-of-car-of-tokens
+  (implies (and (equal (vl-token->type (car (vl-tokstream->tokens))) free-type)
+                (syntaxp (quotep types))
+                (syntaxp (quotep free-type))
+                (member free-type types))
+           (equal (vl-type-of-matched-token types (vl-tokstream->tokens))
+                  (if (member free-type types)
+                      free-type
+                    nil)))
+  :hints(("Goal" :in-theory (enable vl-type-of-matched-token))))
+
 (defthm magically-resolve-vl-is-some-token?
   (implies (and (equal (vl-type-of-matched-token types2 (vl-tokstream->tokens)) value)
                 (member-equal value types)
@@ -1141,7 +1222,7 @@ guard)."
                                  (consp (vl-tokstream->tokens))))
                (new-tokstream))
   (b* ((tokens    (vl-tokstream->tokens))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil (car tokens) tokstream))
   ///
   ;; (defthm vl-match-of-vl-tokenlist-fix
@@ -1186,7 +1267,14 @@ guard)."
              (equal (vl-token->type (mv-nth 1 (vl-match)))
                     (vl-type-of-matched-token types (vl-tokstream->tokens))))
     :hints(("Goal" :in-theory (enable vl-type-of-matched-token
-                                      vl-is-some-token?)))))
+                                      vl-is-some-token?))))
+
+  (defthm more-tokens-after-vl-match-because-lookahead-sees-something
+    (implies (vl-lookahead-is-token? token (cdr (vl-tokstream->tokens)))
+             (consp (vl-tokstream->tokens :tokstream (mv-nth 2 (vl-match)))))
+    :rule-classes ((:forward-chaining :trigger-terms ((vl-lookahead-is-token? token (cdr (vl-tokstream->tokens))))))
+    :hints(("Goal" :in-theory (enable vl-lookahead-is-token?)))))
+
 
 
 (define vl-match-any
@@ -1203,7 +1291,7 @@ kind of token it is.  Causes an error on EOF."
   (b* ((tokens (vl-tokstream->tokens))
        ((when (atom tokens))
         (vl-parse-error "Unexpected EOF." :function function))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil (car tokens) tokstream))
   ///
   ;; (defthm vl-match-any-of-vl-tokenlist-fix
@@ -1265,7 +1353,7 @@ listed types.  Causes an error on EOF."
                              :fn function
                              :fatalp t)
             nil tokstream))
-       (tokstream (vl-tokstream-update-tokens (cdr tokens))))
+       (tokstream (vl-tokstream-pop)))
     (mv nil token1 tokstream))
   ///
   (local (in-theory (enable vl-is-some-token?)))
@@ -1366,3 +1454,60 @@ parser when we encounter such an ending.</p>"
                                          (vl-idtoken->name endname)))
             nil
             tokstream))))
+
+
+(define vl-choose-parse-error ((pos1 natp) (err1 vl-warning-p)
+                               (pos2 natp) (err2 vl-warning-p))
+  :returns (mv (best-pos natp :rule-classes :type-prescription)
+               (best-err vl-warning-p))
+  (if (<= (lnfix pos1) (lnfix pos2))
+      (mv (lnfix pos1) (vl-warning-fix err1))
+    (mv (lnfix pos2) (vl-warning-fix err2)))
+  ///
+  (defret vl-choose-parse-error-under-iff
+    best-err))
+
+
+(defun clause-which-flag (clause)
+  (declare (xargs :mode :program))
+  (b* (((when (atom clause))
+        nil)
+       (lit1 (car clause)))
+    (case-match lit1
+      (('not ('acl2::flag-is ('quote flag)))
+       flag)
+      (&
+       (clause-which-flag (cdr clause))))))
+
+(defun expand-only-the-flag-function-hint (clause state)
+  ;; This computed hint is a much more restrictive alternative to
+  ;; flag::expand-calls-computed-hint.  We look for calls ONLY of the function
+  ;; whose flag we are currently on.  This was particularly useful for
+  ;; statement parsing, where we have a situation sort of like this:
+  ;;
+  ;;   (mutual-recursion
+  ;;      (defun f1 ...)
+  ;;      (defun f2 ...)
+  ;;      ...
+  ;;      (defun fN ...)
+  ;;      (defun g (args)
+  ;;         (cond ((cond1 args) (f1 args))
+  ;;               ((cond2 args) (f2 args))
+  ;;               ...
+  ;;               ((condN args) (fN args)))))
+  ;;
+  ;; Here, when we get to proving a theorem about G, it is a really bad idea to
+  ;; use expand-calls-computed-hint on all of the clique members (which
+  ;; normally works fine), because they're all being invoked on the original
+  ;; arguments, so it thinks they're all suitable for expansion and just opens
+  ;; them all up, leading to a massive case split.
+  ;;
+  ;; So the idea here is: don't open up all of the functions, only open up the
+  ;; one that is explicitly mentioned in the flag::flag-is hyp.  This doesn't
+  ;; seem to make much of any difference for, e.g., expression parsing, but it
+  ;; makes a big difference for statements.
+  (declare (xargs :mode :program :stobjs state))
+  (let ((flag (clause-which-flag clause)))
+    (and flag
+         (let ((fn (acl2::deref-macro-name flag (macro-aliases (w state)))))
+           (std::expand-calls-computed-hint clause (list fn))))))
