@@ -207,7 +207,7 @@ quotes."
 
   :long "<p>We assume that @('x') has a simple enough name to print without any
 encoding.  This is generally true for keyword constants used as tags and in
-basic enumerations like @(see vl-exprtype-p).  We print only the symbol name,
+basic enumerations like @(see vl-exprsign-p).  We print only the symbol name,
 in lower-case.</p>"
 
   (vl-ps-seq (vl-print "\"")
@@ -293,6 +293,76 @@ encoding.</p>"
                 (vl-println? "}"))))
 
 
+(define vl-jp-warning ((x vl-warning-p) &key (ps 'ps))
+  :parents (json-encoders)
+  :short "Special, custom JSON encoder for warnings."
+
+  :long "<p>We probably don't want to use the ordinary aggregate-encoding stuff
+to print @(see vl-warning-p) objects, since the types in the @(':args') field
+are dynamic and, besides, who wants to reimplement @(see vl-cw) in other
+languages.  Instead, it's probably more convenient to just go ahead and convert
+the warning into a printed message here.  We'll include both HTML and plain
+TEXT versions of the message.</p>"
+
+  (b* (((vl-warning x) x)
+       (text (with-local-ps (vl-cw-obj x.msg x.args)))
+       (html (with-local-ps
+               (vl-ps-update-htmlp t)
+               ;; For the module browser, the warnings get word wrapped by the
+               ;; browser anyway, so don't try to wrap them ourselves or things
+               ;; get weird looking.
+               (vl-ps-update-autowrap-col 100000)
+               (vl-cw-obj x.msg x.args))))
+    (jp-object :tag    (vl-print "\"warning\"")
+               :fatalp (jp-bool x.fatalp)
+               :type   (jp-str (symbol-name x.type))
+               :fn     (jp-str (symbol-name x.fn))
+               :text   (jp-str text)
+               :html   (jp-str html))))
+
+(add-json-encoder vl-warning-p jp-warning)
+
+(defmacro def-vl-jp-list (type &key newlines)
+  (declare (xargs :guard (maybe-natp newlines)))
+  (b* ((mksym-package-symbol 'vl::foo)
+       (list-p         (mksym 'vl- type 'list-p))
+       (elem-print     (mksym 'vl-jp- type))
+       (list-print-aux (mksym 'vl-jp- type 'list-aux))
+       (list-print     (mksym 'vl-jp- type 'list))
+       (list-p-str     (symbol-name list-p)))
+    `(encapsulate ()
+       (define ,list-print-aux ((x ,list-p) &key (ps 'ps))
+         :parents (,list-print)
+         :short ,(cat "Prints out the elements of a @(see " list-p-str
+                      ") without the enclosing brackets.")
+         (if (atom x)
+             ps
+           (vl-ps-seq (,elem-print (car x))
+                      (if (atom (cdr x))
+                          ps
+                        ,(if newlines
+                             `(vl-ps-seq (vl-println ",")
+                                         (vl-indent ,newlines))
+                           `(vl-println? ", ")))
+                      (,list-print-aux (cdr x)))))
+       (define ,list-print ((x ,list-p) &key (ps 'ps))
+         :parents (json-encoders)
+         :short ,(cat "Prints out the elements of a @(see " list-p-str
+                      ") with the enclosing brackets.")
+         (vl-ps-seq (vl-print "[")
+                    (,list-print-aux x)
+                    (vl-println? "]")))
+       (add-json-encoder ,list-p ,list-print))))
+
+(def-vl-jp-list warning :newlines 4)
+
+
+
+
+#|||
+
+;; BOZO once upon a time we had a working vl->json translation.
+;; If we ever want it back, maybe we'll think about resurrecting this code.
 
 ; Fancy automatic json encoding of std structures
 
@@ -365,6 +435,18 @@ encoding.</p>"
                             :guard (list x1.type x1.name))
           (convert-flexprod-fields-into-eformals (cdr x)))))
 
+(define def-vl-jp-prod-fn (prod omit )
+
+  (vl-ps-seq
+        (vl-print "{\"tag\": ")
+        (jp-sym ',tag)
+        ,(if newlines
+             `(vl-ps-seq (vl-println ", ")
+                         (vl-indent ,newlines))
+           `(vl-print ", "))
+        ,@main
+        (vl-println? "}"))
+
 (define def-vl-jp-aggregate-fn (type omit overrides long newlines world)
   (b* ((mksym-package-symbol 'vl::foo)
        (elem-print     (mksym 'vl-jp- type))
@@ -437,43 +519,13 @@ encoding.</p>"
 (logic)
 
 
-(defmacro def-vl-jp-list (type &key newlines)
-  (declare (xargs :guard (maybe-natp newlines)))
-  (b* ((mksym-package-symbol 'vl::foo)
-       (list-p         (mksym 'vl- type 'list-p))
-       (elem-print     (mksym 'vl-jp- type))
-       (list-print-aux (mksym 'vl-jp- type 'list-aux))
-       (list-print     (mksym 'vl-jp- type 'list))
-       (list-p-str     (symbol-name list-p)))
-    `(encapsulate ()
-       (define ,list-print-aux ((x ,list-p) &key (ps 'ps))
-         :parents (,list-print)
-         :short ,(cat "Prints out the elements of a @(see " list-p-str
-                      ") without the enclosing brackets.")
-         (if (atom x)
-             ps
-           (vl-ps-seq (,elem-print (car x))
-                      (if (atom (cdr x))
-                          ps
-                        ,(if newlines
-                             `(vl-ps-seq (vl-println ",")
-                                         (vl-indent ,newlines))
-                           `(vl-println? ", ")))
-                      (,list-print-aux (cdr x)))))
-       (define ,list-print ((x ,list-p) &key (ps 'ps))
-         :parents (json-encoders)
-         :short ,(cat "Prints out the elements of a @(see " list-p-str
-                      ") with the enclosing brackets.")
-         (vl-ps-seq (vl-print "[")
-                    (,list-print-aux x)
-                    (vl-println? "]")))
-       (add-json-encoder ,list-p ,list-print))))
+
 
 
 
 ;; Real Verilog JSON Encoding
 
-(define vl-jp-exprtype ((x vl-exprtype-p) &key (ps 'ps))
+(define vl-jp-exprsign ((x vl-exprsign-p) &key (ps 'ps))
   :parents (json-encoders)
   :inline t
   (jp-sym x))
@@ -529,7 +581,7 @@ encoding.</p>"
   :inline t
   (jp-sym x))
 
-(add-json-encoder vl-exprtype-p         vl-jp-exprtype)
+(add-json-encoder vl-exprsign-p         vl-jp-exprsign)
 (add-json-encoder vl-cstrength-p        vl-jp-cstrength)
 (add-json-encoder vl-dstrength-p        vl-jp-dstrength)
 (add-json-encoder vl-direction-p        vl-jp-direction)
@@ -542,11 +594,11 @@ encoding.</p>"
 (add-json-encoder vl-alwaystype-p       vl-jp-alwaystype)
 
 
-(define vl-jp-maybe-exprtype ((x vl-maybe-exprtype-p) &key (ps 'ps))
+(define vl-jp-maybe-exprsign ((x vl-maybe-exprsign-p) &key (ps 'ps))
   :parents (json-encoders)
   :inline t
   (if x
-      (vl-jp-exprtype x)
+      (vl-jp-exprsign x)
     (vl-print "null")))
 
 (define vl-jp-maybe-cstrength ((x vl-maybe-cstrength-p) &key (ps 'ps))
@@ -563,7 +615,7 @@ encoding.</p>"
       (vl-jp-direction x)
     (vl-print "null")))
 
-(add-json-encoder vl-maybe-exprtype-p  vl-jp-maybe-exprtype)
+(add-json-encoder vl-maybe-exprsign-p  vl-jp-maybe-exprsign)
 (add-json-encoder vl-maybe-cstrength-p vl-jp-maybe-cstrength)
 (add-json-encoder vl-maybe-direction-p vl-jp-maybe-direction)
 
@@ -656,23 +708,29 @@ which could not hold such large values.</p>")
 
 (add-json-encoder vl-atomguts-p vl-jp-atomguts)
 
+(define vl-jp-value ((x vl-value-p) &key (ps 'ps))
+  (vl-value-case x
+    :vl-constint (vl-jp-constint x)
+
 (defines vl-jp-expr
   :parents (json-encoders)
 
   (define vl-jp-expr ((x vl-expr-p) &key (ps 'ps))
     :measure (two-nats-measure (vl-expr-count x) 0)
+    (vl-expr-case x
+      :vl-literal (vl-jp-value x.val)
     (vl-expr-case x
       :atom
       (jp-object :tag (jp-sym :atom)
                  :guts (vl-jp-atomguts x.guts)
                  :finalwidth (jp-maybe-nat x.finalwidth)
-                 :finaltype (vl-jp-maybe-exprtype x.finaltype))
+                 :finaltype (vl-jp-maybe-exprsign x.finaltype))
       :nonatom
       (jp-object :tag        (jp-sym :nonatom)
                  :atts       (vl-jp-atts x.atts)
                  :args       (vl-jp-exprlist x.args)
                  :finalwidth (jp-maybe-nat x.finalwidth)
-                 :finaltype  (vl-jp-maybe-exprtype x.finaltype))))
+                 :finaltype  (vl-jp-maybe-exprsign x.finaltype))))
 
   (define vl-jp-atts ((x vl-atts-p) &key (ps 'ps))
     :measure (two-nats-measure (vl-atts-count x) 1)
@@ -725,6 +783,9 @@ which could not hold such large values.</p>")
 (add-json-encoder vl-atts-p       vl-jp-atts)
 (add-json-encoder vl-maybe-expr-p vl-jp-maybe-expr)
 
+
+
+
 (def-vl-jp-aggregate range)
 (def-vl-jp-list range)
 
@@ -819,16 +880,6 @@ which could not hold such large values.</p>")
   ///
   (add-json-encoder vl-maybe-nettypename-p vl-jp-maybe-nettypename))
 
-(define vl-jp-enumbasekind ((x vl-enumbasekind-p) &key (ps 'ps))
-  :guard-hints(("Goal" :in-theory (enable vl-enumbasekind-p)))
-  (if (stringp x)
-      (jp-object :tag (jp-sym :user-defined-type)
-                 :name (jp-str x))
-    (jp-sym x)))
-
-(add-json-encoder vl-enumbasekind-p vl-jp-enumbasekind)
-
-(def-vl-jp-aggregate enumbasetype)
 (def-vl-jp-aggregate enumitem)
 (def-vl-jp-list enumitem)
 
@@ -861,7 +912,7 @@ which could not hold such large values.</p>")
                 :members (vl-jp-structmemberlist x.members))
      :vl-enum
      (jp-object :tag      (jp-sym :vl-enum)
-                :basetype (vl-jp-enumbasetype x.basetype)
+                :basetype (vl-jp-datatype x.basetype)
                 :items    (vl-jp-enumitemlist x.items)
                 :pdims    (vl-jp-packeddimensionlist x.pdims)
                 :udims    (vl-jp-packeddimensionlist x.udims))
@@ -1062,9 +1113,10 @@ which could not hold such large values.</p>")
                   :lvalue (vl-jp-expr x.lvalue)
                   :atts   (vl-jp-atts x.atts))
 
-       :vl-enablestmt
+       :vl-callstmt
        (jp-object :tag    (jp-sym kind)
                   :id     (vl-jp-expr x.id)
+                  :
                   :args   (vl-jp-exprlist x.args)
                   :atts   (vl-jp-atts x.atts))
 
@@ -1073,6 +1125,14 @@ which could not hold such large values.</p>")
                   :id     (vl-jp-expr x.id)
                   :atts   (vl-jp-atts x.atts))
 
+       :vl-breakstmt
+       (jp-object :tag    (jp-sym kind)
+                  :atts   (vl-jp-atts x.atts))
+
+       :vl-continuestmt
+       (jp-object :tag    (jp-sym kind)
+                  :atts   (vl-jp-atts x.atts))
+
        :vl-returnstmt
        (jp-object :tag    (jp-sym kind)
                   :atts   (vl-jp-atts x.atts)
@@ -1127,7 +1187,7 @@ which could not hold such large values.</p>")
 
        :vl-blockstmt
        (jp-object :tag        (jp-sym kind)
-                  :sequential (jp-bool x.sequentialp)
+                  :blocktype  (jp-sym x.blocktype)
                   :name       (jp-maybe-string x.name)
                   :imports    (vl-jp-importlist x.imports)
                   :paramdecls (vl-jp-paramdecllist x.paramdecls)
@@ -1215,40 +1275,6 @@ which could not hold such large values.</p>")
 
 
 
-(define vl-jp-warning ((x vl-warning-p) &key (ps 'ps))
-  :parents (json-encoders)
-  :short "Special, custom JSON encoder for warnings."
-
-  :long "<p>We probably don't want to use the ordinary aggregate-encoding stuff
-to print @(see vl-warning-p) objects, since the types in the @(':args') field
-are dynamic and, besides, who wants to reimplement @(see vl-cw) in other
-languages.  Instead, it's probably more convenient to just go ahead and convert
-the warning into a printed message here.  We'll include both HTML and plain
-TEXT versions of the message.</p>"
-
-  (b* (((vl-warning x) x)
-       (text (with-local-ps (vl-cw-obj x.msg x.args)))
-       (html (with-local-ps
-               (vl-ps-update-htmlp t)
-               ;; For the module browser, the warnings get word wrapped by the
-               ;; browser anyway, so don't try to wrap them ourselves or things
-               ;; get weird looking.
-               (vl-ps-update-autowrap-col 100000)
-               (vl-cw-obj x.msg x.args))))
-    (jp-object :tag    (vl-print "\"warning\"")
-               :fatalp (jp-bool x.fatalp)
-               :type   (jp-str (symbol-name x.type))
-               :fn     (jp-str (symbol-name x.fn))
-               :text   (jp-str text)
-               :html   (jp-str html))))
-
-(add-json-encoder vl-warning-p jp-warning)
-
-(def-vl-jp-list warning :newlines 4)
-
-
-
-
 (define vl-jp-commentmap-aux ((x vl-commentmap-p) &key (ps 'ps))
   (b* (((when (atom x))
         ps))
@@ -1360,3 +1386,4 @@ TEXT versions of the message.</p>"
 
 ")
                (vl-jp-individual-modules (cdr x)))))
+||#
