@@ -33,6 +33,7 @@
 (include-book "../mlib/allexprs")
 (include-book "../mlib/find")
 (include-book "../mlib/modnamespace")
+(include-book "../mlib/fmt")
 (local (include-book "../util/arithmetic"))
 
 (defxdoc vl-describe
@@ -55,7 +56,7 @@ to describe all of the places that @('name') is used.</p>")
   (b* (((when (atom x))
         nil)
        (exprs (vl-modinst-allexprs (car x)))
-       (names (vl-exprlist-names exprs))
+       (names (vl-exprlist-varnames exprs))
        ((when (member-equal name names))
         (cons (vl-modinst-fix (car x))
               (vl-modinsts-that-mention name (cdr x)))))
@@ -67,7 +68,7 @@ to describe all of the places that @('name') is used.</p>")
   (b* (((when (atom x))
         nil)
        (exprs (vl-assign-allexprs (car x)))
-       (names (vl-exprlist-names exprs))
+       (names (vl-exprlist-varnames exprs))
        ((when (member-equal name names))
         (cons (vl-assign-fix (car x))
               (vl-assigns-that-mention name (cdr x)))))
@@ -80,7 +81,7 @@ to describe all of the places that @('name') is used.</p>")
   (b* (((vl-plainarg x) x)
        ((unless x.expr)
         ps)
-       (names (vl-expr-names x.expr))
+       (names (vl-expr-varnames x.expr))
        ((unless (member-equal name names))
         ps)
        (directp (and (vl-idexpr-p x.expr)
@@ -129,7 +130,7 @@ to describe all of the places that @('name') is used.</p>")
   (b* (((vl-namedarg x) x)
        ((unless x.expr)
         ps)
-       (names (vl-expr-names x.expr))
+       (names (vl-expr-varnames x.expr))
        ((unless (member-equal name names))
         ps)
        (directp (and (vl-idexpr-p x.expr)
@@ -258,8 +259,8 @@ to describe all of the places that @('name') is used.</p>")
                                &key (ps 'ps))
   ;; We assume that name is mentioned somewhere in X.
   (b* (((vl-assign x) x)
-       (lhs-names (vl-expr-names x.lvalue))
-       (rhs-names (vl-expr-names x.expr))
+       (lhs-names (vl-expr-varnames x.lvalue))
+       (rhs-names (vl-expr-varnames x.expr))
        (writtenp  (member-equal name lhs-names))
        (readp     (member-equal name rhs-names))
        (htmlp     (vl-ps->htmlp)))
@@ -353,7 +354,7 @@ to describe all of the places that @('name') is used.</p>")
   (b* (((when (atom x))
         nil)
        (exprs (vl-gateinst-allexprs (car x)))
-       (names (vl-exprlist-names exprs))
+       (names (vl-exprlist-varnames exprs))
        ((when (member-equal name names))
         (cons (vl-gateinst-fix (car x))
               (vl-gateinsts-that-mention name (cdr x)))))
@@ -388,6 +389,7 @@ to describe all of the places that @('name') is used.</p>")
 (define vl-pp-describe ((name stringp)
                         (x    vl-module-p)
                         &key (ps 'ps))
+  :guard-debug t
   (b* (((vl-module x) x)
 
        (some-uses-will-not-be-displayed-p
@@ -400,7 +402,7 @@ to describe all of the places that @('name') is used.</p>")
                                         :ports nil
                                         :portdecls nil
                                         :gateinsts nil))
-             (others  (vl-exprlist-names (vl-module-allexprs x-prime))))
+             (others  (vl-exprlist-varnames (vl-module-allexprs x-prime))))
           (member-equal name others)))
 
        (htmlp (vl-ps->htmlp))
@@ -416,8 +418,12 @@ to describe all of the places that @('name') is used.</p>")
                   ps))
              ps))
 
+       (ss (make-vl-scopestack-null) ;; bozo stupid hack
+           )
+       (ss (vl-scopestack-push x ss))
+
        (portdecl (vl-find-portdecl name x.portdecls))
-       (item     (vl-find-moduleitem name x))
+       (item     (vl-scopestack-find-item name ss))
 
        (ps       (if item
                      (vl-ps-seq
@@ -426,10 +432,12 @@ to describe all of the places that @('name') is used.</p>")
                       (vl-when-html (vl-println-markup "</h4>"))
                       (vl-when-html (vl-println-markup "<div class=\"vl_src\">"))
                       (if (and portdecl (not (equal item portdecl)))
-                          (vl-ps-seq
-                           (vl-pp-portdecl portdecl)
-                           (vl-pp-moduleitem item))
-                        (vl-pp-moduleitem item))
+                          (vl-ps-seq (vl-cw "~a0" portdecl)
+                                     (vl-cw "~a0" item))
+                        ps)
+                      (if item
+                          (vl-cw "~a0" item)
+                        ps)
                       (vl-when-html (vl-println-markup "</div>")))
                    (vl-ps-seq
                     (vl-when-html (vl-print-markup "<h4>"))
