@@ -90,7 +90,7 @@ viewing Verilog designs.")
   :prepwork ((local (in-theory (enable vl-descalist-okp))))
   (b* (((vl-zip zip))
        (orig           (cwtime (hons-copy (cwtime (vl-annotate-design zip.design)))))
-       (orig-descalist (fast-alist-free (vl-descalist (vl-design-descriptions orig)))))
+       (orig-descalist (fast-alist-free (vl-make-descalist (vl-design-descriptions orig)))))
     (make-vls-data
      :name           zip.name
      :date           zip.date
@@ -185,7 +185,7 @@ viewing Verilog designs.")
     (case (tag x)
       (:vl-module    (vl-ppc-module    x ss))
       (:vl-udp       (vl-ppc-udp       x))
-      (:vl-interface (vl-ppc-interface x))
+      (:vl-interface (vl-ppc-interface x ss))
       (:vl-package   (vl-ppc-package   x))
       (:vl-program   (vl-ppc-program   x))
       (:vl-config    (vl-ppc-config    x))
@@ -196,6 +196,9 @@ viewing Verilog designs.")
       (:vl-paramdecl  (vl-pp-paramdecl  x))
       (:vl-import     (vl-pp-import     x))
       (:vl-fwdtypedef (vl-pp-fwdtypedef x))
+      (:vl-vardecl    (vl-pp-vardecl    x))
+      (:vl-dpiimport  (vl-pp-dpiimport  x))
+      (:vl-dpiexport  (vl-pp-dpiexport  x))
       (otherwise      (vl-pp-typedef    x)))))
 
 (define vl-description-summary ((x vl-description-p))
@@ -285,6 +288,11 @@ viewing Verilog designs.")
   (b* ((desctypes (vl-descalist->descriptions/types (vls-data->orig-descalist data))))
     (vls-success :json (bridge::json-encode desctypes))))
 
+(local (defthm vl-module-p-by-tag-when-vl-description-p-unlimited
+         (implies (and (vl-description-p x)
+                       (eq (tag x) :vl-module))
+                  (vl-module-p x))))
+
 (define-vls-html vls-describe (origname what data)
   (b* (((vls-data data))
        (desc (cdr (hons-assoc-equal origname data.orig-descalist)))
@@ -361,6 +369,9 @@ declaration, ...)."
       (:vl-import     nil)
       (:vl-fwdtypedef nil)
       (:vl-typedef    nil)
+      (:vl-vardecl    nil)
+      (:vl-dpiimport  nil)
+      (:vl-dpiexport  nil)
       (otherwise      (impossible)))))
 
 (define vls-data-origname-reportcard ((data vls-data-p))
