@@ -34,7 +34,7 @@
 (include-book "centaur/fty/basetypes" :dir :system)
 
 (defprod vl-simpconfig
-  :parents (vl-simplify)
+  :parents (vl-design->svex-design)
   :short "Options for how to simplify Verilog modules."
   :tag :vl-simpconfig
 
@@ -48,17 +48,15 @@
                   cause some kind of problems."
                  :default nil)
 
-   (clean-params-p booleanp
-                   "Should we clean parameters with the @(see clean-params) transform
-                    before unparameterizing?"
-                   :rule-classes :type-prescription
-                   :default t)
-
    (unroll-limit natp
                  "Maximum number of iterations to unroll for loops, etc., when
                   rewriting statements.  This is just a safety valve."
                  :rule-classes :type-prescription
-                 :default 1000)))
+                 :default 1000)
+
+   (already-annotated booleanp
+                      "Denotes that we've already annotated the design and shouldn't
+                       do it again.")))
 
 (defconst *vl-default-simpconfig*
   (make-vl-simpconfig))
