@@ -215,7 +215,10 @@ fast-alist-free).</p>"
 
 
 
-(define fast-memberp (a x (alist (equal alist (make-lookup-alist x))))
+(define fast-memberp (a
+                      x
+                      (alist (set-equiv (alist-keys alist)
+                                        (list-fix x))))
   :parents (utilities)
   :short "Fast list membership using @(see make-lookup-alist)."
 
@@ -358,27 +361,6 @@ from @('alist')."
     :enable acl2-count))
 
 
-(define redundant-list-fix (x)
-  :parents (utilities)
-  :short "@(call redundant-list-fix) is the same as @('(list-fix x)'), but
-avoids consing when @('x') is already a true-listp."
-
-  :long "<p>I sometimes want to @('list-fix') something that I know is almost
-certainly already a @('true-listp') in practice.  In such cases,
-@('redundant-list-fix') may be a better choice than @('list-fix'), because
-checking @('true-listp') is much cheaper than re-consing the a list.</p>
-
-<p>I leave this function enabled.  Logically it is just an alias for
-@('list-fix'), so you should never need to reason about it.</p>"
-
-  :enabled t
-
-  (mbe :logic (list-fix x)
-       :exec (if (true-listp x)
-                 x
-               (list-fix x))))
-
-
 (deflist string-list-listp (x)
   (string-listp x)
   :guard t
@@ -836,3 +818,31 @@ versions of the standard.  We currently have some support for:</p>
 <li>@(':system-verilog-2012') corresponds to IEEE Std 1800-2012.</li>
 </ul>")
 
+
+
+
+;; (defoption maybe-string stringp :pred acl2::maybe-stringp$inline
+;;   ;; BOZO misplaced, also has documentation issues
+;;   :parents nil
+;;   :fix maybe-string-fix
+;;   :equiv maybe-string-equiv)
+
+(define maybe-string-fix ((x maybe-stringp))
+  :returns (xx maybe-stringp)
+  :hooks nil
+  (mbe :logic (and x (str-fix x))
+       :exec x)
+  ///
+  (defthm maybe-string-fix-when-maybe-stringp
+    (implies (maybe-stringp x)
+             (equal (maybe-string-fix x) x)))
+
+  (defthm maybe-string-fix-under-iff
+    (iff (maybe-string-fix x) x))
+
+  (fty::deffixtype maybe-string :pred maybe-stringp :fix maybe-string-fix
+    :equiv maybe-string-equiv :define t :forward t)
+
+  (defrefinement maybe-string-equiv streqv
+    :hints ((and stable-under-simplificationp
+                 '(:in-theory (enable streqv))))))
