@@ -31,15 +31,12 @@
 (in-package "VL")
 
 ;; This showed up in the critical path, so we try to reduce our dependencies
+(include-book "locations")
 (include-book "ihs/basic-definitions" :dir :system)
-(include-book "centaur/fty/deftypes" :dir :system)
-(include-book "centaur/fty/basetypes" :dir :system)
 (include-book "std/util/defaggregate" :dir :system)
 (include-book "std/util/defprojection" :dir :system)
 (include-book "std/util/defval" :dir :system)
 (include-book "std/util/deflist" :dir :system)
-(include-book "std/strings/cat" :dir :system)
-(include-book "std/strings/decimal" :dir :system)
 (include-book "centaur/nrev/pure" :dir :system)
 (include-book "xdoc/alter" :dir :system)
 (local (include-book "centaur/misc/arith-equivs" :dir :system))
@@ -83,67 +80,6 @@ extended character as an @(see vl-echar-p), a structure that associates a
 character with its location.</p>")
 
 
-(defprod vl-location
-  :parents (extended-characters)
-  :short "Representation of a point in a source file."
-  :tag :vl-location
-  :layout :tree
-
-  ((filename stringp :rule-classes :type-prescription)
-   (line     posp    :rule-classes :type-prescription)
-   (col      natp    :rule-classes :type-prescription))
-
-  :long "<p>Each vl-location-p represents some location in a source code file.
-These locations are attached to characters and module items to provide context
-during error reporting.</p>")
-
-(defval *vl-fakeloc*
-  :parents (vl-location)
-  :short "A \"fake\" @(see vl-location-p) which we use when generating our
-own @(see extended-characters) and module items."
-
-  (vl-location "[[[ fake location ]]]" 1 0))
-
-(fty::deflist vl-locationlist :elt-type vl-location)
-
-
-(define vl-location-string ((loc vl-location-p))
-  :parents (vl-location)
-  :short "Convert an @(see vl-location-p) into a string."
-  :long "<p>@(call vl-location-string) is often useful in generating warning
-or error messages.  It converts a @(see vl-location-p) object into a string
-of the form <i>filename:line:col</i>.</p>"
-  :returns (str stringp :rule-classes :type-prescription)
-  (cat (vl-location->filename loc)
-       ":"
-       (natstr (vl-location->line loc))
-       ":"
-       (natstr (vl-location->col loc))))
-
-
-(define vl-location-between-p ((x vl-location-p)
-                               (min vl-location-p)
-                               (max vl-location-p))
-  :parents (vl-location)
-  :short "@(call vl-location-between-p) is true exactly when @('x') is in the
-same file as @('min') and @('max'), and inclusively falls between these
-bounds."
-
-  (b* (((vl-location x) x)
-       ((vl-location low) min) ;; bozo awful symbol problems with min/max
-       ((vl-location high) max))
-
-      (and (equal x.filename low.filename)
-           (equal x.filename high.filename)
-
-           (or (< low.line x.line)
-               (and (eql low.line x.line)
-                    (<= low.col x.col)))
-
-           (or (< x.line high.line)
-               (and (eql x.line high.line)
-                    (<= x.col high.col))))))
-
 (defsection vl-echar-p
   :autodoc nil
 
@@ -384,9 +320,7 @@ bounds."
    (let ((name 'vl-echar)
          (fields '(char loc)))
      `(progn
-        ,(std::da-make-maker-fn name fields nil)
-        ,(std::da-make-maker name fields)
-        ,(std::da-make-changer-fn name fields)
+        ,(std::da-make-maker name fields nil)
         ,(std::da-make-changer name fields)
         ,(std::da-make-binder name fields))))
 
