@@ -209,6 +209,7 @@ fatal warnings instead of non-fatal warnings.</p>"
     :hints(("Goal" :in-theory (enable string<)))))
 
 (defsection vl-warning-sort
+  :parents (vl-clean-warnings)
   :short "Mergesort warnings using @(see vl-warning-<)"
 
   (ACL2::defsort :comparablep vl-warning-p
@@ -267,13 +268,13 @@ fatal warnings instead of non-fatal warnings.</p>"
     :hints(("Goal" :cases ((consp x))))))
 
 (define vl-clean-warnings
-  :parents (warnings clean-warnings)
+  :parents (clean-warnings)
   :short "Sort and remove duplicates from a list of warnings."
   ((x vl-warninglist-p))
   :returns (ans vl-warninglist-p)
   (ACL2::remove-adjacent-duplicates
    (vl-warning-sort
-    (redundant-list-fix
+    (list-fix
      (vl-warninglist-fix x))))
   ///
   (defthm vl-clearn-warnings-under-iff
@@ -374,3 +375,65 @@ particular interest.</p>"
              (t
               (vl-some-warning-of-type-p types (cdr x))))))
 
+
+;; this is just a product of msg (string) and args (true-list) but if there are
+;; no args we just use the string.  It's convenient for a bare string to be a
+;; vl-msg.
+(fty::defflexsum vl-msg
+  :short "Format string and args for small messages not constituting a whole warning"
+  :kind nil
+  (:msg :cond t
+   :type-name vl-msg
+   :shape (or (atom x)
+              (cdr x))
+   :fields ((msg :type stringp :acc-body (if (atom x) x (car x))
+                 :rule-classes :type-prescription)
+            (args :type true-listp :acc-body (and (consp x) (cdr x))
+                  :rule-classes :type-prescription))
+   :ctor-body (if args (cons msg args) msg)))
+
+(defsection vmsg
+  :parents (warnings)
+  :short "Similar to @(see acl2::msg); constructs a @(see vl-msg) that can be used
+          with @('~@') directives in VL's @(see formatted-printing) routines."
+  :long "@(def vmsg)"
+
+  (defmacro vmsg (msg &rest args)
+    `(make-vl-msg :msg ,msg :args (list . ,args))))
+
+
+(define vl-warning-add-ctx ((x vl-warning-p)
+                            (ctx))
+  :returns (new-x vl-warning-p)
+  (b* (((vl-warning x)))
+    (change-vl-warning x
+                       :msg "~a0: ~@1"
+                       :args (list ctx (vl-msg x.msg x.args)))))
+
+(defprojection vl-warninglist-add-ctx ((x vl-warninglist-p)
+                                       (ctx))
+  :returns (new-x vl-warninglist-p)
+  (vl-warning-add-ctx x ctx))
+
+#!acl2
+(def-b*-binder vl::wmv
+  :parents (warnings)
+  :short "B* binder to automatically append together returned warnings"
+  :body
+  (b* (((mv ctx args)
+        (b* ((mem (member :ctx args)))
+          (if mem
+              (mv (cadr mem)
+                  (append (take (- (len args) (len mem)) args)
+                          (cddr mem)))
+            (mv nil args)))))
+    `(b* (,(if (equal args '(vl::warnings))
+               `(vl::__tmp__warnings . ,forms)
+             `((mv . ,(subst 'vl::__tmp__warnings 'vl::warnings args)) . ,forms))
+          (vl::warnings (append-without-guard
+                         ,(if ctx
+                              `(vl::vl-warninglist-add-ctx vl::__tmp__warnings
+                                                       ,ctx)
+                            'vl::__tmp__warnings)
+                         vl::warnings)))
+       ,rest-expr)))
