@@ -77,29 +77,31 @@ sorting the elements by type; see @(see vl-sort-genelements).</p>
       :tag :vl-genblob
       :layout :tree)
 
-    (define vl-modelement-loc ((x vl-modelement-p))
-      :parents (vl-modelement)
-      :short "Get the location of any @(see vl-modelement-p)."
-      :returns (loc vl-location-p
-                    :hints(("Goal" :in-theory (enable vl-modelement-fix
-                                                      (tau-system)))))
-      (b* ((x (vl-modelement-fix x)))
-        (case (tag x)
-          . ,(project-over-modelement-types
-              '(:vl-__type__       (vl-__type__->loc x))))))
+    ;; [Jared] no, use vl-modelement->loc instead.
+    ;; (define vl-modelement-loc ((x vl-modelement-p))
+    ;;   :parents (vl-modelement)
+    ;;   :short "Get the location of any @(see vl-modelement-p)."
+    ;;   :returns (loc vl-location-p
+    ;;                 :hints(("Goal" :in-theory (enable vl-modelement-fix
+    ;;                                                   (tau-system)))))
+    ;;   (b* ((x (vl-modelement-fix x)))
+    ;;     (case (tag x)
+    ;;       . ,(project-over-modelement-types
+    ;;           '(:vl-__type__       (vl-__type__->loc x))))))
 
-    (define vl-genelement-loc ((x vl-genelement-p))
-      :parents (vl-genelement)
-      :short "Get the location of any @(see vl-genelement-p)."
-      :returns (loc vl-location-p
-                    :hints(("Goal" :in-theory (enable vl-genelement-fix))))
-      (vl-genelement-case x
-        :vl-genbase (vl-modelement-loc x.item)
-        :vl-genloop   x.loc
-        :vl-genif     x.loc
-        :vl-gencase   x.loc
-        :vl-genblock  x.loc
-        :vl-genarray  x.loc))
+    ;; [Jared] no, we already have vl-genelement->loc
+    ;; (define vl-genelement-loc ((x vl-genelement-p))
+    ;;   :parents (vl-genelement)
+    ;;   :short "Get the location of any @(see vl-genelement-p)."
+    ;;   :returns (loc vl-location-p
+    ;;                 :hints(("Goal" :in-theory (enable vl-genelement-fix))))
+    ;;   (vl-genelement-case x
+    ;;     :vl-genbase (vl-modelement-loc x.item)
+    ;;     :vl-genloop   x.loc
+    ;;     :vl-genif     x.loc
+    ;;     :vl-gencase   x.loc
+    ;;     :vl-genblock  x.loc
+    ;;     :vl-genarray  x.loc))
 
     (local (defun my-default-hint (fnname id clause world)
              (declare (xargs :mode :program))
@@ -165,7 +167,8 @@ sorting the elements by type; see @(see vl-sort-genelements).</p>
                           ,@(project-over-modelement-types
                              'vl-modelementlist-p-when-vl-__type__list-p)
                           (:rules-of-class :type-prescription :here)
-                          (:ruleset tag-reasoning))))))
+                          ;(:ruleset tag-reasoning)
+                          )))))
 
     (define vl-sort-genelements
       :parents (genblob)
@@ -776,7 +779,7 @@ etc., are overwritten with whatever is in the genblob.</p>"
                                    ,@(and new-x '((make-vl-genblock
                                                    :elems new-elems
                                                    :loc (vl-modelement->loc x.item))))))))
-       
+
        (define ,apply-to-gencaselist ((x vl-gencaselist-p) . ,raw-formals)
          :returns ,(maybe-mv-fn `(,@returns
                                   ,@(and new-x '((new-x vl-gencaselist-p)))))
