@@ -42,7 +42,9 @@
 (local (defthm vl-modelement-p-when-vl-blockitem-p
          (implies (vl-blockitem-p x)
                   (vl-modelement-p x))
-         :hints(("Goal" :in-theory (enable vl-blockitem-p vl-modelement-p)))))
+         :hints(("Goal" :in-theory (enable tag-reasoning
+                                           vl-blockitem-p
+                                           vl-modelement-p)))))
 
 (local (defthm vl-modelementlist-p-when-vl-blockitemlist-p
          (implies (vl-blockitemlist-p x)
@@ -1143,7 +1145,7 @@ all of its identifiers.</p>"
     (b* ((ifports (vl-interfaceportlist-fix ifports))
          (st (make-vl-implicitst :decls     (make-fast-alist
                                              (pairlis$ (vl-portlist->names ifports)
-                                                       (redundant-list-fix ifports)))
+                                                       (list-fix ifports)))
                                  :portdecls nil
                                  :imports   nil
                                  :ss        ss))
