@@ -40,6 +40,7 @@
 (include-book "../osets/conversions")
 (include-book "../osets/extras")
 (include-book "../osets/map")
+(include-book "../osets/listsets-base")
 (include-book "../bags/basic") ;bzo is this okay - can records depend on bags?
 
 (local (include-book "../util/iff"))
@@ -53,6 +54,22 @@
 
 (local (in-theory (disable mod floor))) ;bzo make non-local?
 
+
+;; [Jared] Previously this book did its own (set::quantify-predice (true-listp
+;; set)).  But that was incompatible with coi/osets/listsets.lisp because, it
+;; turns out, set::quantify-predicate forms can't be redundant due to the use
+;; of deftheory forms.  What a pain.  Anyway, moved the quantify-predicate form
+;; to coi/osets/listsets-base, leaving only these (unfortunately non-local)
+;; disables...
+
+(in-theory (disable SET::ALL<NOT-TRUE-LISTP>))
+(in-theory (disable SET::ALL<TRUE-LISTP>))
+(in-theory (disable SET::ALL-IN-2-NOT<TRUE-LISTP>))
+(in-theory (disable SET::ALL-IN-2<TRUE-LISTP>))
+
+
+
+
 ;bzo make a bunch of the stuff in this file local?
 
 ;; ((200 . 2) 1 . 3)
@@ -409,7 +426,7 @@
        (set::all<natp-less-than-size> (mem::mem-tree-domain (caar mem) (caddr mem)) (cadr mem))
        (set::all<not-natp-less-than-size> (set::rkeys (cdddr mem)) (cadr mem))
 
-       (equal (signed-byte-p 30 (caddr mem))
+       (equal (signed-byte-p 29 (caddr mem))
               (cdar mem))
 
 ;bzo drop?
@@ -669,12 +686,6 @@
 
 ;(in-theory (disable SET::FILTER-IN-NOT<NATP-LESS-THAN-SIZE>))
 
-(set::quantify-predicate (true-listp set))
-
-(in-theory (disable SET::ALL<NOT-TRUE-LISTP>))
-(in-theory (disable SET::ALL<TRUE-LISTP>))
-(in-theory (disable SET::ALL-IN-2-NOT<TRUE-LISTP>))
-(in-theory (disable SET::ALL-IN-2<TRUE-LISTP>))
 
 (defthm len-of-CONVERT-INTEGER-TO-REVERSED-BIT-LIST
   (equal (LEN (CONVERT-INTEGER-TO-REVERSED-BIT-LIST A len))
@@ -1269,4 +1280,4 @@
 (defthm good-memoryp-of-clear
   (implies (acl2::good-memoryp mem)
            (acl2::good-memoryp (mem::clear addr mem)))
-  :hints (("Goal" :in-theory (enable mem::clear))))
\ No newline at end of file
+  :hints (("Goal" :in-theory (enable mem::clear))))
