@@ -1,4 +1,14 @@
-(in-package "ACL2")
+;; David M. Russinoff
+;; david@russinoff.com
+;; http://www.russinoff.com
+
+(in-package "RTL")
+
+(include-book "support/euclid")
+
+(set-enforce-redundancy t)
+(set-inhibit-warnings "theory") ; avoid warning in the next event
+(local (in-theory nil))
 
 ;; This book contains proofs of two theorems of Euclid:
 
@@ -6,16 +16,9 @@
 
 ;;   (2) If p is a prime divisor of a*b, then p divides either a or b.
 
-; (Matt K., 10/2013: Changed rel8 to rel9.)
-(include-book "rtl/rel9/support/lib3/arith" :dir :system) ; basic arithmetic
-(include-book "rtl/rel9/support/lib3/basic" :dir :system)   ; properties of mod
-
 ;; We first list some basic properties of divisibility.
 
 (defun divides (x y)
-  (declare (xargs :guard (and (integerp x)
-                              (integerp y)
-                              (not (= x 0)))))
   (and (not (= x 0))
        (integerp (/ y x))))
 
@@ -48,9 +51,7 @@
 		  (integerp z)
 		  (divides x y))
 	     (divides x (* y z)))
-  :rule-classes ()
-  :hints (("Goal" :use (:instance associativity-of-* (x (/ x)))
-		  :in-theory (disable a9 associativity-of-*))))
+  :rule-classes ())
 
 (defthm divides-transitive
     (implies (and (integerp x)
@@ -59,8 +60,7 @@
 		  (divides x y)
 		  (divides y z))
 	     (divides x z))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance divides-product (z (/ z y)))))))
+  :rule-classes ())
 
 (defthm divides-self
     (implies (and (integerp x)
@@ -78,9 +78,7 @@
 		  (not (zp n)))
 	     (iff (divides n (- a b))
 		  (= (mod a n) (mod b n))))
-  :rule-classes ()
-  :hints (("Goal" :in-theory (enable divides)
-		  :use (mod-equal-int mod-equal-int-reverse))))
+  :rule-classes ())
 
 (defthm divides-mod-0
     (implies (and (integerp a)
@@ -92,15 +90,13 @@
 
 (in-theory (disable divides))
 
-;; Our definition of primality is based on the following function, which computes 
-;; the least divisor of a natural number n that is greater than or equal to k:
+;; Our definition of primality is based on the following function, which computes
+;; the least divisor of a natural number n that is greater than or equal to k.
+;; (In the book "mersenne", in which we are concerned with efficiency, we shall
+;; introduce an equivalent version that checks for divisors only up to sqrt(n).)
 
 (defun least-divisor (k n)
-  (declare (xargs :guard (and (integerp n)
-                              (integerp k)
-                              (< 1 k)
-                              (<= k n))
-                  :measure (nfix (- n k))))
+  (declare (xargs :measure (nfix (- n k))))
   (if (and (integerp n)
 	   (integerp k)
 	   (< 1 k)
@@ -108,7 +104,7 @@
       (if (divides k n)
 	  k
 	(least-divisor (1+ k) n))
-    nil))
+    ()))
 
 (defthm least-divisor-divides
     (implies (and (integerp n)
@@ -133,9 +129,6 @@
   :rule-classes ())
 
 (defun primep (n)
-  (declare (xargs :guard t
-                  :guard-hints (("goal" :use (:instance least-divisor-divides
-                                               (k 2))))))
   (and (integerp n)
        (>= n 2)
        (= (least-divisor 2 n) n)))
@@ -152,54 +145,37 @@
 		  (divides d p)
 		  (> d 1))
 	     (= d p))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance least-divisor-is-least (n p) (k 2))
-			(:instance divides-leq (x d) (y p))))))
+  :rule-classes ())
 
 (defthm primep-least-divisor
     (implies (and (integerp n)
 		  (>= n 2))
 	     (primep (least-divisor 2 n)))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance least-divisor-divides (k 2))
-			(:instance least-divisor-divides (k 2) (n (least-divisor 2 n)))
-			(:instance divides-transitive
-				   (x (least-divisor 2 (least-divisor 2 n)))
-				   (y (least-divisor 2 n))
-				   (z n))
-			(:instance least-divisor-is-least
-				   (d (least-divisor 2 (least-divisor 2 n)))
-				   (k 2))))))
+  :rule-classes ())
 
-(in-theory (disable primep))		   
+(in-theory (disable primep))
 
-;; In order to prove the infinitude of the set of primes, we define a function "greater-prime" 
+;; Our formulation of the infinitude of the set of primes is based on a function that
 ;; returns a prime that is greater than its argument:
 
 (defun fact (n)
-  (declare (xargs :guard (natp n)))
   (if (zp n)
       1
     (* n (fact (1- n)))))
 
 (defun greater-prime (n)
-  (declare (xargs :guard (natp n)))
   (least-divisor 2 (1+ (fact n))))
 
 (defthm greater-prime-divides
     (divides (greater-prime n) (1+ (fact n)))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance primep-least-divisor (n (1+ (fact n))))
-			(:instance least-divisor-divides (k 2) (n (1+ (fact n))))))))
+  :rule-classes ())
 
 (defthm divides-fact
     (implies (and (integerp n)
 		  (integerp k)
 		  (<= 1 k)
 		  (<= k n))
-	     (divides k (fact n)))
-  :hints (("Subgoal *1/4" :use ((:instance divides-product (x k) (y (fact (1- n))) (z n))))
-	  ("Subgoal *1/3" :use ((:instance divides-product (x k) (y k) (z (fact (1- k))))))))
+	     (divides k (fact n))))
 
 (defthm not-divides-fact-plus1
     (implies (and (integerp n)
@@ -207,27 +183,19 @@
 		  (< 1 k)
 		  (<= k n))
 	     (not (divides k (1+ (fact n)))))
-  :rule-classes ()
-  :hints (("Goal" :use (divides-fact
-			(:instance divides-leq (x k) (y 1))
-			(:instance divides-sum (x k) (y (- (fact n))) (z (1+ (fact n))))
-			(:instance divides-product (x k) (y (fact n)) (z -1))))))
+  :rule-classes ())
 
 (defthm infinitude-of-primes
     (implies (integerp n)
 	     (and (primep (greater-prime n))
 		  (> (greater-prime n) n)))
-  :rule-classes ()
-  :hints (("Goal" :use (greater-prime-divides
-			(:instance primep-least-divisor (n (1+ (fact n))))
-			(:instance not-divides-fact-plus1 (k (greater-prime n)))))))
+  :rule-classes ())
 
-;; Our main theorem depends on the properties of the greatest common divisor,
-;; which We define according to Euclid's algorithm.
+;; Our main theorem of Euclid depends on the properties of the greatest common divisor,
+;; which we define according to Euclid's algorithm.
 
 (defun g-c-d-nat (x y)
-  (declare (xargs :guard (and (natp x) (natp y))
-                  :measure (nfix (+ x y))))
+  (declare (xargs :measure (nfix (+ x y))))
   (if (zp x)
       y
     (if (zp y)
@@ -237,7 +205,6 @@
 	(g-c-d-nat (- x y) y)))))
 
 (defun g-c-d (x y)
-  (declare (xargs :guard (and (integerp x) (integerp y))))
   (g-c-d-nat (abs x) (abs y)))
 
 (defthm g-c-d-nat-pos
@@ -253,28 +220,21 @@
 		  (not (and (= x 0) (= y 0))))
 	     (and (integerp (g-c-d x y))
 		  (> (g-c-d x y) 0)))
-  :rule-classes ()
-  :hints (("Goal" :use (:instance g-c-d-nat-pos (x (abs x)) (y (abs y))))))
+  :rule-classes ())
 
 (defthm divides-g-c-d-nat
     (implies (and (natp x)
 		  (natp y))
 	     (and (or (= x 0) (divides (g-c-d-nat x y) x))
 		  (or (= y 0) (divides (g-c-d-nat x y) y))))
-  :rule-classes ()
-  :hints (("Goal" :induct (g-c-d-nat x y))
-	  ("Subgoal *1/4" :use (:instance divides-sum (x (g-c-d-nat (- x y) y)) (z (- x y))))
-	  ("Subgoal *1/3" :use (:instance divides-sum (x (g-c-d-nat x (- y x) )) (y x) (z (- y x))))))
+  :rule-classes ())
 
 (defthm g-c-d-divides
     (implies (and (integerp x)
 		  (integerp y))
 	     (and (or (= x 0) (divides (g-c-d x y) x))
 		  (or (= y 0) (divides (g-c-d x y) y))))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance divides-g-c-d-nat (x (abs x)) (y (abs y)))
-			(:instance divides-product (x (g-c-d-nat (abs x) (abs y))) (y (abs x)) (z -1))
-			(:instance divides-product (x (g-c-d-nat (abs x) (abs y))) (y (abs y)) (z -1))))))
+  :rule-classes ())
 
 ;; It remains to be shown that the gcd of x and y is divisible by any common
 ;; divisor of x and y.  This depends on the observation that the gcd may be
@@ -284,8 +244,7 @@
 (mutual-recursion
 
  (defun r-nat (x y)
-   (declare (xargs :guard (and (natp x) (natp y))
-                   :measure (nfix (+ x y))))
+   (declare (xargs :measure (nfix (+ x y))))
   (if (zp x)
       0
     (if (zp y)
@@ -295,8 +254,7 @@
 	(r-nat (- x y) y)))))
 
 (defun s-nat (x y)
-   (declare (xargs :guard (and (natp x) (natp y))
-                   :measure (nfix (+ x y))))
+   (declare (xargs :measure (nfix (+ x y))))
   (if (zp x)
       1
     (if (zp y)
@@ -304,7 +262,7 @@
       (if (<= x y)
 	  (s-nat x (- y x))
 	(- (s-nat (- x y) y) (r-nat (- x y) y))))))
-	
+
 )
 
 (defthm r-s-nat
@@ -316,13 +274,11 @@
   :rule-classes ())
 
 (defun r (x y)
-  (declare (xargs :guard (and (integerp x) (integerp y))))
   (if (< x 0)
       (- (r-nat (abs x) (abs y)))
     (r-nat (abs x) (abs y))))
 
 (defun s (x y)
-  (declare (xargs :guard (and (integerp x) (integerp y))))
   (if (< y 0)
       (- (s-nat (abs x) (abs y)))
     (s-nat (abs x) (abs y))))
@@ -341,8 +297,7 @@
 	     (= (+ (* (r x y) x)
 		   (* (s x y) y))
 		(g-c-d x y)))
-  :rule-classes ()
-  :hints (("Goal" :use (:instance r-s-nat (x (abs x)) (y (abs y))))))
+  :rule-classes ())
 
 (in-theory (disable g-c-d r s))
 
@@ -353,23 +308,16 @@
 		  (not (= d 0))
 		  (divides d x)
 		  (divides d y))
-	     (divides d (g-c-d x y)))
-  :hints (("Goal" :use (g-c-d-linear-combination
-			(:instance divides-sum (x d) (y (* (r x y) x)) (z (* (s x y) y)))
-			(:instance divides-product (x d) (y x) (z (r x y)))
-			(:instance divides-product (x d) (z (s x y)))))))
+	     (divides d (g-c-d x y))))
 
 (defthm g-c-d-prime
     (implies (and (primep p)
 		  (integerp a)
 		  (not (divides p a)))
 	     (= (g-c-d p a) 1))
-  :rule-classes ()
-  :hints (("Goal" :use ((:instance g-c-d-divides (x p) (y a))
-			(:instance g-c-d-pos (x p) (y a))
-			(:instance primep-no-divisor (d (g-c-d p a)))))))
+  :rule-classes ())
 
-;; The main theorem:
+;; the main theorem:
 
 (defthm euclid
     (implies (and (primep p)
@@ -378,12 +326,5 @@
 		  (not (divides p a))
 		  (not (divides p b)))
 	     (not (divides p (* a b))))
-  :rule-classes ()
-  :hints (("Goal" :use (g-c-d-prime
-			(:instance cancel-equal-* (a b) (r (+ (* (r p a) p) (* (s p a) a))) (s 1))
-			(:instance g-c-d-linear-combination (x p) (y a))
-			(:instance divides-sum (x p) (y (* (r p a) p b)) (z (* (s p a) a b)))
-			(:instance divides-product (x p) (y (* a b)) (z (s p a)))
-			(:instance divides-product (x p) (y p) (z (* b (r p a))))))))
-
+  :rule-classes ())
 
