@@ -1,24 +1,12 @@
-; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic 
-; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc. 
+; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic
+; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc.
 ;
 ; Contact:
 ;   David Russinoff
 ;   1106 W 9th St., Austin, TX 78703
 ;   http://www.russsinoff.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.
-;
-; This program is distributed in the hope that it will be useful but WITHOUT ANY
-; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-; PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-;
-; You should have received a copy of the GNU General Public License along with
-; this program; see the file "gpl.txt" in this directory.  If not, write to the
-; Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA
-; 02110-1335, USA.
+; See license file books/rtl/rel9/license.txt.
 ;
 ; Author: David M. Russinoff (david@russinoff.com)
 
@@ -28,7 +16,6 @@
 
 
 (include-book "../lib1/rtl")
-(include-book "../lib1/rtlarr")
 
 (local (include-book "../lib1/bvecp-raw-helpers"))
 (local (include-book "../lib1.delta1/bvecp-raw-helpers-extra"))
@@ -257,7 +244,7 @@
   :rule-classes ((:type-prescription :typed-term (lnot x n))))
 
 ;lnot-nonnegative-integer-type is strictly better, and we don't need both
-(in-theory (disable (:type-prescription lnot))) 
+(in-theory (disable (:type-prescription lnot)))
 
 (defthm lnot-bvecp
   (implies (and (<= n k)
@@ -487,15 +474,6 @@
   :RULE-CLASSES
   (:REWRITE (:linear :trigger-terms ((UNKNOWN KEY SIZE N)))))
 
-;BOZO dup?
-(defthm bv-arrp-implies-nonnegative-integerp
-  (implies (bv-arrp obj size)
-           (and (INTEGERP (ag index obj))
-                (<= 0 (ag index obj))))
-  :rule-classes (:rewrite :type-prescription)
-  )
-
-
 ; land
 
 (defthm land-nonnegative-integer-type
@@ -610,12 +588,6 @@
   :rule-classes
   (:rewrite (:linear :trigger-terms ((unknown key size n)))))
 
-(defthm bv-arrp-implies-nonnegative-integerp
-  (implies (bv-arrp obj size)
-           (and (INTEGERP (ag index obj))
-                (<= 0 (ag index obj))))
-  :rule-classes (:rewrite :type-prescription))
-
 ;(local (in-theory (enable floor-fl)))
 
 ;These next two are for the bus unit bvecp lemmas:
@@ -623,7 +595,7 @@
 ;could use (local (in-theory (enable expt-compare-with-double)))
 ;remove?
 (defthm bits-does-nothing-hack
-  (implies (and (< x (expt 2 i)) 
+  (implies (and (< x (expt 2 i))
                 (integerp x)
                 (<= 0 x)
                 (integerp i)
@@ -633,7 +605,7 @@
 
 ;remove?
 (defthm bits-does-nothing-hack-2
-  (implies (and (< x (expt 2 i)) 
+  (implies (and (< x (expt 2 i))
                 (integerp x)
                 (<= 0 x)
                 (integerp i)
@@ -773,7 +745,7 @@
 
 #|
 ;can remove these two?
-(defthm natp-* 
+(defthm natp-*
   (implies (and (integerp x)
                 (>= x 0)
                 (integerp y)
@@ -781,7 +753,7 @@
            (and (integerp (* x y))
                 (>= (* x y) 0))))
 
-(defthm natp-+ 
+(defthm natp-+
   (implies (and (integerp x)
                 (>= x 0)
                 (integerp y)
@@ -804,16 +776,16 @@
 ;;;
 ;;; Tue Jul  3 14:55:18 2007
 ;;;
-;;; this following are added to allow ACL2 to reason about 
-;;; simple inequalities by concluding from the differences 
+;;; this following are added to allow ACL2 to reason about
+;;; simple inequalities by concluding from the differences
 ;;; in bit positions quickly
-;;; 
-;;; They are proved with a general result 
+;;;
+;;; They are proved with a general result
 ;;;
 ;;; in lib1.delta1/bvecp-raw-helpers-extra.lisp
 ;;;
 ;;; enable the general rule in the bvecp-raw.lisp proof will
-;;; be too inefficient  
+;;; be too inefficient
 
 (defthm cat-fact
   (implies (equal (cat x 1 3 2) 3)
@@ -847,7 +819,7 @@
 
 
 
- 
+
 
 (DEFTHM CAT-FACT-7
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1) 1)
@@ -856,29 +828,29 @@
 (DEFTHM CAT-FACT-8
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1) 3)
            (NOT (EQUAL (CAT X 1 Y1 1 Z1 1) 5))))
- 
+
 (DEFTHM CAT-FACT-9
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1) 1)
            (NOT (EQUAL (CAT X 1 Y 1 Z1 1) 3))))
- 
+
 
 (DEFTHM CAT-FACT-10
   (IMPLIES (EQUAL (CAT 1 1 X 1 Y 1) 4)
            (NOT (EQUAL (CAT Z 1 X 1 Y 1) 1))))
- 
+
 
 
 (DEFTHM CAT-FACT-11
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                   125)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
-                       127)))) 
+                       127))))
 
 (DEFTHM CAT-FACT-12
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                   123)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
-                       125)))) 
+                       125))))
 
 (DEFTHM CAT-FACT-13
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3) 119)
@@ -886,159 +858,159 @@
                        123))))
 
 
- 
+
 (DEFTHM CAT-FACT-14
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3)
                        119))))
- 
+
 
 (DEFTHM CAT-FACT-15
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111))))
- 
+
 (DEFTHM CAT-FACT-16
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 31 5) 95))))
- 
+
 
 (DEFTHM CAT-FACT-17
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111))))
- 
+
 (DEFTHM CAT-FACT-18
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3)
                        119))))
- 
+
 
 (DEFTHM CAT-FACT-19
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                        123))))
- 
+
 
 (DEFTHM CAT-FACT-20
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                        125))))
- 
+
 
 (DEFTHM CAT-FACT-21
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 (DEFTHM CAT-FACT-22
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3)
                        119))))
- 
+
 
 (DEFTHM CAT-FACT-23
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                        123))))
- 
+
 
 (DEFTHM CAT-FACT-24
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                        125))))
- 
+
 
 (DEFTHM CAT-FACT-25
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 
 (DEFTHM CAT-FACT-26
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 
 (DEFTHM CAT-FACT-27
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                        125))))
- 
+
 
 (DEFTHM CAT-FACT-28
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                        123))))
- 
+
 (DEFTHM CAT-FACT-29
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3)
                        119))))
- 
+
 
 (DEFTHM CAT-FACT-30
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3) 119)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                        125))))
- 
+
 
 (DEFTHM CAT-FACT-31
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3) 119)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 
 (DEFTHM CAT-FACT-32
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                   123)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 
 (DEFTHM CAT-FACT-33
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                   125)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        127))))
- 
+
 
 (DEFTHM CAT-FACT-34
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 1 1)
                   125)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 (DEFTHM CAT-FACT-35
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 3 2)
                   123)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 (DEFTHM CAT-FACT-36
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 U 1 7 3) 119)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 (DEFTHM CAT-FACT-37
   (IMPLIES (EQUAL (CAT X 1 Y 1 Z 1 15 4) 111)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 (DEFTHM CAT-FACT-38
   (IMPLIES (EQUAL (CAT X 1 Y 1 31 5) 95)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 (DEFTHM CAT-FACT-39
   (IMPLIES (EQUAL (CAT X 1 63 6) 63)
            (NOT (EQUAL (CAT X 1 Y 1 Z 1 U 1 V 1 W 1 P 1)
                        126))))
- 
+
 
 
 
