@@ -1,24 +1,12 @@
-; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic 
-; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc. 
+; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic
+; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc.
 ;
 ; Contact:
 ;   David Russinoff
 ;   1106 W 9th St., Austin, TX 78703
 ;   http://www.russsinoff.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.
-;
-; This program is distributed in the hope that it will be useful but WITHOUT ANY
-; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-; PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-;
-; You should have received a copy of the GNU General Public License along with
-; this program; see the file "gpl.txt" in this directory.  If not, write to the
-; Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA
-; 02110-1335, USA.
+; See license file books/rtl/rel9/license.txt.
 ;
 ; Author: David M. Russinoff (david@russinoff.com)
 
@@ -30,7 +18,6 @@
 ;Also contains type lemmas (non-negative integer, natp, etc.)
 
 (include-book "rtl")
-(include-book "rtlarr")
 
 (include-book "bits")
 (include-book "float")
@@ -272,7 +259,7 @@
   :hints (("Goal" :in-theory (e/d (lnot) ()))))
 
 ;lnot-nonnegative-integer-type is strictly better, and we don't need both
-(in-theory (disable (:type-prescription lnot))) 
+(in-theory (disable (:type-prescription lnot)))
 
 (defthm lnot-bvecp
   (implies (and (<= n k)
@@ -514,20 +501,6 @@
   (:REWRITE (:linear :trigger-terms ((UNKNOWN KEY SIZE N)))))
 
 
-;BOZO dup?
-(defthm bv-arrp-implies-nonnegative-integerp
-  (implies (bv-arrp obj size)
-           (and (INTEGERP (ag index obj))
-                (<= 0 (ag index obj))))
-  :rule-classes (:rewrite :type-prescription)
-  :hints (("Goal" :use (:instance
-                        ag-maps-bv-arr-to-bvecp (a index) (r obj) (k size))
-           :in-theory (set-difference-theories
-                       (enable bvecp)
-                       '(ag-maps-bv-arr-to-bvecp))))
-  )
-
-
 ; land
 
 (defthm land-nonnegative-integer-type
@@ -643,12 +616,6 @@
   :rule-classes
   (:rewrite (:linear :trigger-terms ((unknown key size n)))))
 
-(defthm bv-arrp-implies-nonnegative-integerp
-  (implies (bv-arrp obj size)
-           (and (INTEGERP (ag index obj))
-                (<= 0 (ag index obj))))
-  :rule-classes (:rewrite :type-prescription))
-
 ;(local (in-theory (enable floor-fl)))
 
 ;These next two are for the bus unit bvecp lemmas:
@@ -657,7 +624,7 @@
 ;remove?
 
 (defthm bits-does-nothing-hack
-  (implies (and (< x (expt 2 i)) 
+  (implies (and (< x (expt 2 i))
                 (integerp x)
                 (<= 0 x)
                 (integerp i)
@@ -669,7 +636,7 @@
 
 ;remove?
 (defthm bits-does-nothing-hack-2
-  (implies (and (< x (expt 2 i)) 
+  (implies (and (< x (expt 2 i))
                 (integerp x)
                 (<= 0 x)
                 (integerp i)
@@ -811,7 +778,7 @@
 
 #|
 ;can remove these two?
-(defthm natp-* 
+(defthm natp-*
   (implies (and (integerp x)
                 (>= x 0)
                 (integerp y)
@@ -819,7 +786,7 @@
            (and (integerp (* x y))
                 (>= (* x y) 0))))
 
-(defthm natp-+ 
+(defthm natp-+
   (implies (and (integerp x)
                 (>= x 0)
                 (integerp y)
