@@ -1,24 +1,12 @@
-; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic 
-; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc. 
+; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic
+; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc.
 ;
 ; Contact:
 ;   David Russinoff
 ;   1106 W 9th St., Austin, TX 78703
 ;   http://www.russsinoff.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.
-;
-; This program is distributed in the hope that it will be useful but WITHOUT ANY
-; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-; PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-;
-; You should have received a copy of the GNU General Public License along with
-; this program; see the file "gpl.txt" in this directory.  If not, write to the
-; Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA
-; 02110-1335, USA.
+; See license file books/rtl/rel9/license.txt.
 ;
 ; Author: David M. Russinoff (david@russinoff.com)
 
@@ -39,7 +27,7 @@
 ;;;**********************************************************************
 
 
-(defund sgn (x) 
+(defund sgn (x)
   (declare (xargs :guard t))
   (if (or (not (rationalp x)) (equal x 0))
       0
@@ -160,7 +148,7 @@
 
 
 (defthm sig-sig
-    (equal (sig (sig x)) 
+    (equal (sig (sig x))
 	   (sig x)))
 
 
@@ -200,12 +188,12 @@
   (implies (and (rationalp x)
                 (not (equal x 0))
                 (integerp n))
-           (equal (expo (* (expt 2 n) x)) 
+           (equal (expo (* (expt 2 n) x))
                   (+ n (expo x)))))
 
 
 (defthmd sig-shift
-  (equal (sig (* (expt 2 n) x)) 
+  (equal (sig (* (expt 2 n) x))
          (sig x)))
 
 
@@ -533,7 +521,7 @@
 
 
 (defund esgnf_alt  (x p q) (bitn_alt x (+ p q)))
-(defund eexpof_alt (x p q) (bits_alt x (1- (+ p q)) p)) 
+(defund eexpof_alt (x p q) (bits_alt x (1- (+ p q)) p))
 (defund esigf_alt  (x p)   (bits_alt x (1- p) 0))
 
 (defund bias (q) (- (expt 2 (- q 1)) 1) )
@@ -591,7 +579,7 @@
 		  (integerp p)
 		  (> p 1)
 		  (integerp q)
-		  (> q 0))  
+		  (> q 0))
 	     (equal (expo (ndecode_alt x p q))
 		    (- (iexpof_alt x p q) (bias q))))
     :hints (("Goal" :in-theory (enable expo-ndecode))))
@@ -836,32 +824,3 @@
                 (> q 0)
                 (drepp r p q))
            (>= (abs r) (spd p q))))
-
-
-;; (defthmd spd-mult
-;;   (implies (and (integerp p)
-;;                 (> p 1)
-;;                 (integerp q)
-;;                 (> q 0)
-;; 		(rationalp r)
-;; 		(= m (/ r (spd p q))))
-;; 	   (iff (drepp r p q)
-;; 		(and (natp m)
-;; 		     (<= 1 m)
-;; 		     (< m (expt 2 (1- p)))))))
-;; not true!! 
-
-(defthmd spd-mult
-  (implies (and (integerp p)
-                (> p 1)
-                (integerp q)
-                (> q 0)
-                (> r 0)
-		(rationalp r)
-		(= m (/ r (spd p q))))
-	   (iff (drepp r p q)
-		(and (natp m)
-		     (<= 1 m)
-		     (< m (expt 2 (1- p)))))))
-
-
