@@ -1,31 +1,19 @@
-; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic 
-; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc. 
+; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic
+; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc.
 ;
 ; Contact:
 ;   David Russinoff
 ;   1106 W 9th St., Austin, TX 78703
 ;   http://www.russsinoff.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.
-;
-; This program is distributed in the hope that it will be useful but WITHOUT ANY
-; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-; PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-;
-; You should have received a copy of the GNU General Public License along with
-; this program; see the file "gpl.txt" in this directory.  If not, write to the
-; Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA
-; 02110-1335, USA.
+; See license file books/rtl/rel9/license.txt.
 ;
 ; Author: David M. Russinoff (david@russinoff.com)
 
 (in-package "RTL")
 
 (include-book "arithmetic-5/top" :dir :system)
-(include-book "reps")
+(include-book "float")
 (local (include-book "../lib3/top"))
 
 (defund fl (x)
@@ -89,9 +77,9 @@
         ((endp (cddddr x))
          `(binary-cat ,@x))
         (t
-         `(binary-cat ,(car x) 
-                      ,(cadr x) 
-                      (cat ,@(cddr x)) 
+         `(binary-cat ,(car x)
+                      ,(cadr x)
+                      (cat ,@(cddr x))
                       ,(cat-size (cddr x))))))
 
 (defund mulcat (l n x)
@@ -111,6 +99,21 @@
                           l))
                     (t 0))))
 
+(defund bias (q) (- (expt 2 (- q 1)) 1) )
+
+(defund spn (q)
+  (expt 2 (- 1 (bias q))))
+
+(defund spd (p q)
+     (expt 2 (+ 2 (- (bias q)) (- p))))
+
+(defund drepp (x p q)
+  (and (rationalp x)
+       (not (= x 0))
+       (<= (- 2 p) (+ (expo x) (bias q)))
+       (<= (+ (expo x) (bias q)) 0)
+       ;number of bits available in the sig field = p - 1 - ( - bias - expo(r))
+       (exactp x (+ -2 p (expt 2 (- q 1)) (expo x)))))
 
 ;;;**********************************************************************
 ;;;                         Truncation
@@ -118,8 +121,8 @@
 
 (defund trunc (x n)
   (declare (xargs :guard (integerp n)))
-  (* (sgn x) 
-     (fl (* (expt 2 (1- n)) (sig x))) 
+  (* (sgn x)
+     (fl (* (expt 2 (1- n)) (sig x)))
      (expt 2 (- (1+ (expo x)) n))))
 
 (defthmd trunc-integer-type-prescription
@@ -134,8 +137,8 @@
 		  (integerp n)
 		  (> n 0))
 	     (equal (trunc x n)
-		    (* (sgn x) 
-		       (fl (* (expt 2 (- (1- n) (expo x))) (abs x))) 
+		    (* (sgn x)
+		       (fl (* (expt 2 (- (1- n) (expo x))) (abs x)))
 		       (expt 2 (- (1+ (expo x)) n))))))
 
 (defthmd abs-trunc
@@ -254,7 +257,7 @@
 
 (defthm trunc-exactp-b
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 0))
 	     (iff (= x (trunc x n))
 		  (exactp x n)))
@@ -359,8 +362,8 @@
 ;;;**********************************************************************
 
 (defund away (x n)
-  (* (sgn x) 
-     (cg (* (expt 2 (1- n)) (sig x))) 
+  (* (sgn x)
+     (cg (* (expt 2 (1- n)) (sig x)))
      (expt 2 (- (1+ (expo x)) n))))
 
 (defthmd away-integer-type-prescription
@@ -375,14 +378,14 @@
 		  (integerp n)
 		  (> n 0))
 	     (equal (away x n)
-		    (* (sgn x) 
-		       (cg (* (expt 2 (- (1- n) (expo x))) (abs x))) 
+		    (* (sgn x)
+		       (cg (* (expt 2 (- (1- n) (expo x))) (abs x)))
 		       (expt 2 (- (1+ (expo x)) n))))))
 
 (defthmd abs-away
     (implies (and (rationalp x)
 		  (integerp n))
-	     (equal (abs (away x n)) 
+	     (equal (abs (away x n))
 		    (* (cg (* (expt 2 (1- n)) (sig x))) (expt 2 (- (1+ (expo x)) n))))))
 
 (defthm away-to-0
@@ -508,7 +511,7 @@
 
 (defthm away-exactp-b
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 0))
 	     (iff (= x (away x n))
 		  (exactp x n)))
@@ -537,7 +540,7 @@
 		  (not (exactp x n)))
 	     (= (away x n)
 		(+ (trunc x n)
-		   (expt 2 (+ (expo x) 1 (- n))))))		
+		   (expt 2 (+ (expo x) 1 (- n))))))
   :rule-classes ())
 
 
@@ -672,7 +675,7 @@
   :rule-classes (:type-prescription :linear))
 
 (defthm near-0
-  (equal (near 0 n) 
+  (equal (near 0 n)
          0))
 
 (defthm near-exactp-a
@@ -681,7 +684,7 @@
 
 (defthm near-exactp-b
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 0))
 	     (iff (= x (near x n))
 		  (exactp x n)))
@@ -770,7 +773,7 @@
   :rule-classes ())
 
 (defthm near-est
-    (implies (and (integerp n) 
+    (implies (and (integerp n)
 		  (> n 0)
 		  (rationalp x))
 	     (<= (abs (- x (near x n)))
@@ -811,7 +814,7 @@
 		  (integerp n)
 		  (integerp k)
 		  (> k 0)
-		  (>= n k)		  
+		  (>= n k)
 		  (< 0 a)
 		  (< a x)
 		  (< 0 y)
@@ -836,7 +839,7 @@
 
 (defthm near-exact
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 1)
 		  (exactp x (1+ n))
 		  (not (exactp x n)))
@@ -949,7 +952,7 @@
 
 (defthm near+-exactp-b
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 0))
 	     (iff (= x (near+ x n))
 		  (exactp x n)))
@@ -1010,7 +1013,7 @@
   :rule-classes ())
 
 (defthm near+-est
-    (implies (and (integerp n) 
+    (implies (and (integerp n)
 		  (> n 0)
 		  (rationalp x))
 	     (<= (abs (- x (near+ x n)))
@@ -1078,7 +1081,7 @@
 		  (integerp n)
 		  (> n 0))
 	     (= (near+ x n)
-		(trunc (+ x (expt 2 (- (expo x) n))) n)))		
+		(trunc (+ x (expt 2 (- (expo x) n))) n)))
   :rule-classes ())
 
 (defthm near+-trunc-cor
@@ -1109,7 +1112,7 @@
 
 (defthmd sticky-positive
     (implies (and (< 0 x)
-                  (rationalp x) 
+                  (rationalp x)
 		  (integerp n)
                   (> n 0))
 	     (> (sticky x n) 0))
@@ -1117,7 +1120,7 @@
 
 (defthmd sticky-negative
     (implies (and (< x 0)
-                  (rationalp x) 
+                  (rationalp x)
 		  (integerp n)
                   (> n 0))
 	     (< (sticky x n) 0))
@@ -1134,7 +1137,7 @@
 		  (integerp n) (> n 0)
 		  (integerp k))
 	     (= (sticky (* (expt 2 k) x) n)
-		(* (expt 2 k) (sticky x n))))		
+		(* (expt 2 k) (sticky x n))))
   :rule-classes ())
 
 (defthm expo-sticky
@@ -1152,7 +1155,7 @@
 
 (defthm sticky-exactp-b
     (implies (and (rationalp x)
-		  (integerp n) 
+		  (integerp n)
 		  (> n 0))
 	     (iff (= x (sticky x n))
 		  (exactp x n)))
@@ -1169,7 +1172,7 @@
 (defthm sticky-exactp-m
     (implies (and (rationalp x)
 		  (integerp m)
-		  (integerp n) 
+		  (integerp n)
 		  (> n m)
 		  (> m 0))
 	     (iff (exactp (sticky x n) m)
@@ -1360,7 +1363,7 @@
 (defthm rnd-exactp-b
   (implies (and (rationalp x)
                 (common-rounding-mode-p mode)
-                (integerp n) 
+                (integerp n)
                 (> n 0))
            (equal (equal x (rnd x mode n))
 		  (exactp x n))))
@@ -1395,7 +1398,7 @@
 
 (defthm rnd>=trunc
     (implies (and (rationalp x)
-		  (> x 0) ;; 
+		  (> x 0) ;;
 		  (common-rounding-mode-p mode)
                   (INTEGERP N)
                   (> N 0))
@@ -1455,9 +1458,9 @@
 (defthmd rnd-sticky
   (implies (and (common-rounding-mode-p mode)
                 (rationalp x)
-                (integerp m) 
+                (integerp m)
 		(> m 0)
-                (integerp n) 
+                (integerp n)
 		(>= n (+ m 2)))
            (equal (rnd (sticky x n) mode m)
                   (rnd x mode m))))
@@ -1505,10 +1508,10 @@
                   (trunc x n))))
   :rule-classes ()
   :hints (("Goal" :use roundup_alt-thm)))
-  
+
 
 ;;;**********************************************************************
-;;;                         Denormal Rounding 
+;;;                         Denormal Rounding
 ;;;**********************************************************************
 
 (defund drnd (x mode p q)
