@@ -31,7 +31,9 @@
 (in-package "STR")
 (include-book "ieqv")
 (include-book "std/basic/defs" :dir :system)
-(include-book "std/util/deflist" :dir :system)
+(include-book "std/util/deflist-base" :dir :system)
+(include-book "std/lists/rev" :dir :system)
+(include-book "std/lists/append" :dir :system)
 (local (include-book "arithmetic"))
 (local (include-book "misc/assert" :dir :system))
 (local (include-book "ihs/quotient-remainder-lemmas" :dir :system))
