@@ -202,6 +202,9 @@ makes things more concise and better looking, e.g., compare:</p>
    ...)
 })
 
+<p>See @(see define-guards) for discussion of the various ways guards can be
+given, additionally.</p>
+
 <p>Some additional minor options include:</p>
 
 <dl>
@@ -261,6 +264,12 @@ difference can sometimes cause problems in later theorems.  Setting
 @(':no-function t') will avoid binding @('__function__') for better backwards
 compatibility.</dd>
 
+<dt>@(':hooks hooks')</dt>
+
+<dd>(Advanced feature).  Override which @(see post-define-hook)s are to be
+executed.  For instance, @(':hooks nil') can be used to disable all such
+hooks.</dd>
+
 </dl>
 
 <h4>@('Returns') Specifications</h4>
@@ -298,11 +307,173 @@ some kind of separator!</p>
 
 ")
 
+(defxdoc define-guards
+  :parents (define)
+  :short "Discussion of how guards are given in @(see define)."
+  :long "<p>@(csee define) allows several different ways to specify the guards
+of a function.  The ordering of these in the final guards provided to the
+generated defun is sometimes significant (when the guards themselves have
+guards) and define's behavior in trying to address this can be quirky.</p>
+
+<p>The following example shows all (?) the ways in which guards can be specified:</p>
+
+@({
+  (define foo ((a natp)                   ;; formal guard
+               (b :type unsigned-byte)    ;; formal type
+               c d$ e f$ g
+               state)                     ;; implicit stobj
+     :guard (consp c)                     ;; guard keyword
+     :stobjs (d$)                         ;; stobjs keyword
+     (declare (xargs :guard (stringp e))) ;; guard in declare form
+     (declare (xargs :stobjs (f$)))       ;; stobjs in declare form
+     (declare (type symbol g))            ;; type in declare form
+   ...)
+ })
+
+<p>This define generates the following declarations:</p>
+
+@({
+  (DECLARE (XARGS :STOBJS (D$ F$ STATE)))         ;; implicit stobjs
+  (DECLARE (TYPE UNSIGNED-BYTE B))                ;; formal types
+  (DECLARE (XARGS :GUARD (NATP A)))               ;; formal guards
+  (DECLARE (XARGS :GUARD (STRINGP E)))            ;; declare forms
+  (DECLARE (XARGS :STOBJS (F$)))
+  (DECLARE (TYPE SYMBOL G))
+  (DECLARE (XARGS :STOBJS (D$) :GUARD (CONSP C))) ;; guard and stobj keywords
+ })
+
+<p>The reasons for this ordering are somewhat heuristic: stobj declarations
+shouldn't have any dependencies and type declarations are also unlikely to.
+Formal guards usually occur first in the define form and are also usually used
+for simple unary type constraints, so we put them next.  Declare forms may also
+include stobj and type declarations, which the guard keywords might depend on.
+Finally, the guards specified by the guard keyword come last.</p>
+
+<p>One further quirk is that we reorder the formal guards, putting those that
+only refer to one variable first.  This is because we have encountered
+situations where we want to put main formals first and auxiliary parameters
+later, but the main formals' guards depend on the auxiliary parameters.  For
+example,</p>
+
+@({
+ (define fp-add ((x1 (fp-vec-p x1 size))
+                 (x2 (fp-vec-p x2 size))
+                 (size fp-size-p))
+   ...)
+ })
+
+<p>Since size has a unary guard @('(fp-size-p size)'), we put that first, which
+is good if @('fp-vec-p') has that as a guard on its size argument.</p>
+
+<p>It is possible to construct define forms that look like they should succeed
+but actually fail due to the heuristic reordering of guards.  If you encounter
+one of these in the wild and have a suggestion to improve the heuristic, please
+mention it.  In any case, explicitly stating your guards in order in a declare
+form is usually an adequate work-around.</p>")
+
+
+; -------------- Xargs Extraction ---------------------------------------------
+
+(defun extract-xargs-from-xguts
+  (ctx         ; Context for error messages
+   xargs-guts  ; Guts from an (xargs ...) form, i.e., (:verify-guards ... :hints ... :guard ...)
+   )
+  (b* (((when (atom xargs-guts))
+        nil)
+       ((unless (member (car xargs-guts) acl2::*xargs-keywords*))
+        (er hard? ctx "Malformed xargs: expected a valid xargs keyword but found ~x0.~%" (car xargs-guts)))
+       ((when (atom (cdr xargs-guts)))
+        (er hard? ctx "Malformed xargs: no value for ~x0?~%" (car xargs-guts))))
+    (cons (cons (first xargs-guts)
+                (second xargs-guts))
+          (extract-xargs-from-xguts ctx (cddr xargs-guts)))))
+
+#||
+ (extract-xargs-from-xguts 'foo '(:verify-guards t :guard-debug t))
+||#
+
+(defun extract-xargs-from-declguts
+  (ctx        ; Context for error messages
+   declguts   ; Guts from a single (declare ...) form, i.e., ((xargs :verify-guards ...) (ignore ...) ...)
+   )
+  (b* (((when (atom declguts))
+        nil)
+       (decl1 (car declguts))
+       ((unless (and (consp decl1)
+                     (eq (car decl1) 'xargs)))
+        (extract-xargs-from-declguts ctx (cdr declguts)))
+       (xguts (cdr decl1)))
+    (append (extract-xargs-from-xguts ctx xguts)
+            (extract-xargs-from-declguts ctx (cdr declguts)))))
+
+#||
+ (extract-xargs-from-declguts 'foo
+                              '((xargs :verify-guards t)
+                                (ignore foo)
+                                (type integer foo)
+                                (xargs :guard-debug t :measure-debug t)
+                                (ignorable bar)))
+||#
+
+(defun extract-xargs-from-traditional-decls/docs
+  (ctx   ; Context for error messages
+   decls ; List of traditional (declare ...) forms or doc strings
+   )
+  ;; Returns extracted xargs-alist
+  (b* (((when (atom decls))
+        nil)
+       (decl1 (car decls))
+       ((unless (and (consp decl1)
+                     (eq (car decl1) 'declare)))
+        (extract-xargs-from-traditional-decls/docs ctx (cdr decls)))
+       (declguts (cdr decl1)))
+    (append (extract-xargs-from-declguts ctx declguts)
+            (extract-xargs-from-traditional-decls/docs ctx (cdr decls)))))
+
+#||
+ (extract-xargs-from-traditional-decls/docs 'foo
+                                            '("traditional doc string"
+                                              (declare (xargs :guard t :hints 5))
+                                              (declare)
+                                              (declare (ignore x))
+                                              (declare (xargs :verify-guards t)
+                                                       (type integer x)
+                                                       (xargs :measure-debug t))))
+||#
+
+(defun append-hints-from-xargs-alist (xargs-alist)
+  ;; This is an unusual function that is to support using either top-level
+  ;; :hints or (declare (xargs :hints ...)) forms when using the t-proof
+  ;; option.
+  ;;
+  ;; To understand what we are doing, note that ACL2 permits you to write
+  ;; crazy looking things like this:
+  ;;
+  ;; (defun f1 (x)
+  ;;   (declare (xargs :measure (consp x))
+  ;;            (xargs :hints(("Goal" :in-theory (cw "HINT 1~%"))))
+  ;;            (xargs :hints(("Goal" :in-theory (cw "HINT 2~%")))))
+  ;;   (if (zp x)
+  ;;       0
+  ;;     (+ 1 (f (cdr x)))))
+  ;;
+  ;; Which seem to have the same effect as just appending all the different
+  ;; hints together.  The idea here is to try to support this madness as
+  ;; transparently as possible by collecting the xargs and then appending them
+  ;; together.
+  (b* (((when (atom xargs-alist))
+        nil)
+       ((cons key value) (car xargs-alist))
+       ((when (eq key :hints))
+        (append value (append-hints-from-xargs-alist (cdr xargs-alist)))))
+    (append-hints-from-xargs-alist (cdr xargs-alist))))
+
 
 ; -------------- Main Stuff Parsing -------------------------------------------
 
+
 (defun get-xargs-from-kwd-alist (kwd-alist)
-  ;; Munges the xargs stuff together into a form suitable for a declare.
+  ;; Munges the top-level xargs stuff together into a form suitable for a declare.
   (declare (xargs :guard (alistp kwd-alist)))
   (b* (((when (atom kwd-alist))
         nil)
@@ -394,8 +565,9 @@ some kind of separator!</p>
                    ;; many values it returns.
                    (not (getprop fnname 'acl2::non-executablep nil 'acl2::current-acl2-world world))))
         (er hard? 'return-value-names
-            "Error in ~x0: ACL2 thinks this function has ~x0 return ~
-             values, but :returns has ~x1 entries!"
+            "Error in ~x0: ACL2 thinks this function has ~x1 return ~
+             values, but :returns has ~x2 entries!"
+            fnname
             (len stobjs-out)
             (len returnspecs))))
     ;; Else the user documented things, so everything has a name and we should
@@ -658,6 +830,7 @@ some kind of separator!</p>
    formals     ;; already parsed, macro parts removed
 
    rest-events ;; events in the /// part
+   pe-entry    ;; raw form to use in the PE-table
    ))
 
 (table define)
@@ -696,10 +869,50 @@ some kind of separator!</p>
 
 ; ----------------- Hooks -----------------------------------------------------
 
-; WARNING: Undocumented, experimental feature; all details may change.
+(defxdoc post-define-hook
+  :parents (define)
+  :short "A way to extend @(see define) to carry out additional behaviors."
+  :long "<p>The @(see define) macro can be configured with ``hooks'' to
+automatically generate and submit certain additional events.  For instance, the
+@(see fty::fixequiv-hook) extends define to automatically prove certain
+congruence rules.</p>
 
-; Hook function signature:
-;    my-hook-name : defguts * user-args * state -> (mv er val state)
+<p>This is an advanced and somewhat experimental feature.  To introduce a new
+post-define hook, first define a function of the signature:</p>
+
+@({
+     (my-hook-fn defguts user-args state) -> (mv er val state)
+})
+
+<p>Here:</p>
+
+<ul>
+
+<li>the @('defguts') argument is a structure which will have information from
+the @('define') itself, see the comments in @('std/util/define.lisp') for
+details</li>
+
+<li>the @('user-args') are any arguments that can be passed to your hook
+function at @('define') time via the @(':hooks') argument, or via the default
+post-define hook mechanism.</li>
+
+<li>the return value is an ordinary ACL2 error triple, where the @('val')
+should be some additional events to submit.</li>
+
+</ul>
+
+<p>Once your function is defined, you can register it as a post-define hook as
+follows:</p>
+
+@({
+     (add-post-define-hook :myhook my-hook-fn)
+})
+
+<p>And subsequently it will be legal to use @(':hooks ((:myhook ...))') or
+similar.  Define can also be configured with default post-define hooks, see
+@('add-default-post-define-hook') in the @('std/util/define.lisp') source code;
+also see the @('std/util/tests/define.lisp') file for some working
+examples.</p>")
 
 (defun remove-from-alist (key alist)
   (cond ((atom alist)
@@ -793,12 +1006,97 @@ some kind of separator!</p>
         (er hard? 'post-hook-make-events "Invalid post-define hook specifier: ~x0" spec1))
        (look (assoc hook-kwd hooks-alist))
        ((unless look)
-        (er hard? 'post-hook-make-events "Post-define hook not found: ~x0." hook-kwd))
+        (er hard? 'post-hook-make-events "Post-define hook not found: ~x0.  Known hooks: ~&1."
+            hook-kwd (strip-cars hooks-alist)))
        (hook-fn (cdr look))
        (event1 `(make-event (,hook-fn ',guts ',user-args state))))
     (cons event1
           (post-hook-make-events (cdr hook-specs) hooks-alist guts))))
 
+(defun sort-formal-guards-aux (guards wrld state-vars)
+  ;; Returns two lists: simple and complex.  Simple is any guards that we can
+  ;; translate and that have at most 1 variable.  Complex is everything else.
+  ;; Both simple and complex are in the order they were encountered.
+  (b* (((when (atom guards))
+        (mv nil nil))
+       (guard1 (car guards))
+       ((mv err transguard)
+        (acl2::translate-cmp guard1
+                             '(nil) ;; returns single non-stobj
+                             nil    ;; execution, not logic-modep
+                             nil    ;; known-stobjs -- probably don't need them?
+                             'sort-formal-guards
+                             wrld state-vars))
+       ((mv rest-simple rest-complex)
+        (sort-formal-guards-aux (cdr guards) wrld state-vars))
+       ((when (or err
+                  (consp (cdr (all-vars transguard)))))
+        (mv rest-simple
+            (cons guard1 rest-complex))))
+    (mv (cons guard1 rest-simple)
+        rest-complex)))
+
+(defun sort-formal-guards (guards wrld)
+  (b* (((mv simple complex)
+        (sort-formal-guards-aux guards wrld (acl2::default-state-vars nil))))
+    (append simple complex)))
+
+(defun keep-assocs (keys-to-keep alist)
+  (cond ((atom alist)
+         nil)
+        ((member (caar alist) keys-to-keep)
+         (cons (car alist)
+               (keep-assocs keys-to-keep (cdr alist))))
+        (t
+         (keep-assocs keys-to-keep (cdr alist)))))
+
+
+(defun prettify-returnspec-for-pe (returnspec)
+  ;; For the :pe display, we don't want to include things like :hints for the
+  ;; returns specifiers, but it's definitely nice to include a simplified
+  ;; :returns information.
+  (b* (((returnspec x) returnspec)
+       ((when (and (eq x.return-type t)
+                   (equal x.doc "")))
+        x.name))
+    (append (list x.name)
+            (cond ((eq x.return-type t)
+                   nil)
+                  ((and (tuplep 2 x.return-type)
+                        (equal (second x.return-type) x.name))
+                   ;; Simple return type, use concise syntax
+                   (list (first x.return-type)))
+                  (t
+                   ;; Complex return type, use full syntax
+                   (list x.return-type)))
+            (if (eq x.hyp t)
+                nil
+              `(:hyp ,x.hyp))
+            (if (eq x.rule-classes :rewrite)
+                nil
+              `(:rule-classes ,x.rule-classes))
+            (if (equal x.doc "") nil (list x.doc)))))
+
+(defun prettify-returnspecs-for-pe-aux (returnspecs)
+  (if (atom returnspecs)
+      nil
+    (cons (prettify-returnspec-for-pe (car returnspecs))
+          (prettify-returnspecs-for-pe-aux (cdr returnspecs)))))
+
+(defun prettify-returnspecs-for-pe (returnspecs)
+  (cond ((atom returnspecs)
+         nil)
+        ((atom (cdr returnspecs))
+         (prettify-returnspec-for-pe (car returnspecs)))
+        (t
+         (cons 'mv (prettify-returnspecs-for-pe-aux returnspecs)))))
+
+(defun convert-kwd-alist-back-into-plist (kwd-alist)
+  (if (atom kwd-alist)
+      nil
+    (list* (caar kwd-alist)
+           (cdar kwd-alist)
+           (convert-kwd-alist-back-into-plist (cdr kwd-alist)))))
 
 (defun parse-define
   (name            ; User-level name, e.g., FOO
@@ -822,7 +1120,6 @@ some kind of separator!</p>
 
        (non-exec   (getarg :non-executable nil      kwd-alist))
        (returns    (getarg :returns        nil      kwd-alist))
-       (enabled-p  (getarg :enabled        nil      kwd-alist))
        (inline     (getarg :inline         :default kwd-alist))
        (prepwork   (getarg :prepwork       nil      kwd-alist))
 
@@ -830,11 +1127,7 @@ some kind of separator!</p>
         (raise "Error in ~x0: expected :prepwork to be a true-listp, but found ~x1."
                name prepwork))
 
-       (t-proof    (getarg :t-proof        nil kwd-alist))
-       ; If you can think of a good extension for t-proof, you may relax the
-       ; requirement of booleanp while preserving the old behavior for t and
-       ; nil. It would be nice if you would invent a syntactically generalisable
-       ; extension (s.t. some syntax remains invalid)
+       (t-proof (getarg :t-proof nil kwd-alist))
        ((unless (booleanp t-proof))
         (raise "Error in ~x0: expected :t-proof to be a booleanp, but found ~x1."
                name prepwork))
@@ -842,6 +1135,20 @@ some kind of separator!</p>
         (raise "Error in ~x0: expected :inline to be T, NIL, or :DEFAULT, but found ~x1."
                name inline))
 
+       (embedded-xargs-alist (extract-xargs-from-traditional-decls/docs (list 'define name)
+                                                                        traditional-decls/docs))
+       (t-hints (and t-proof
+                     ;; This order of this append is meant to match the order
+                     ;; of the hints given to define in the case of no t-proof,
+                     ;; but where both :hints and (xargs :hints ...) are given
+                     ;; simultaneously.  The top-level :hints get processed
+                     ;; LAST in that case, as described below.
+                     (append
+                      ;; All :hints found anywhere in (declare (xargs ...)) forms
+                      (append-hints-from-xargs-alist embedded-xargs-alist)
+                      ;; Any top-level :hints
+                      (getarg :hints nil kwd-alist))))
+
        (need-macrop (or (not (eq inline :default))
                         (has-macro-args raw-formals)))
        (name-fn     (cond ((eq inline t)
@@ -886,22 +1193,73 @@ some kind of separator!</p>
                                    ,extended-body)
                         extended-body))
 
-       (xargs         (get-xargs-from-kwd-alist (remove-from-alist ':hints kwd-alist)))
-       (t-hints       (getarg :hints nil kwd-alist))
+       (xargs         (get-xargs-from-kwd-alist kwd-alist))
+
+       ;; BOZO packn??  Probably should use the function's package instead.
        (t-proof-name  (if t-proof (ACL2::packn (LIST name-fn '|-| t-proof)) nil))
-       (new-hint      (if t-proof `('(:by ,t-proof-name)) t-hints))
-       (xargs         (if new-hint (LIST* ':hints new-hint xargs) xargs))
 
        (returnspecs   (parse-returnspecs name returns world))
-       (defun-sym     (if enabled-p 'defun 'defund))
+
+       (pe-entry
+        `(define ,name ,raw-formals
+           ;; Documentation string, if simple enough...
+           ,@(b* ((short (cdr (assoc :short kwd-alist)))
+                  ((when (stringp short))
+                   `(:short ,short))
+                  ((when (or short (cdr (assoc :long kwd-alist))))
+                   ;; There's documentation but it's too complex to print nicely
+                   ;; right now, let's just tell the user there's docs available.
+                   `("Documentation is available via :doc.")))
+               ;; No documentation, skip this part.
+               nil)
+           ;; Returnspecs, cleaned up to remove hint stuff
+           ,@(and (consp returnspecs)
+                  (list :returns (prettify-returnspecs-for-pe returnspecs)))
+           ;; We also don't want to see other ugly proof-related things like
+           ;; :hints and :guard-debug and stuff like that, so keep only the
+           ;; things that seem most valuable/short.
+           ,@(convert-kwd-alist-back-into-plist
+              (keep-assocs '(:inline :non-executable :well-founded-relation :measure)
+                           kwd-alist))
+           ,@traditional-decls/docs
+           ,body))
+
+       (guard-verification-will-happen-anyway-p
+        ;; Design decision: define will ignore set-verify-guards-eagerness 1
+        ;; and always try to verify guards.  (I think I much more frequently
+        ;; have guards of T than want to not verify guards.)  We could do this
+        ;; by always adding an explicit (declare (xargs :guard t)).  But that's
+        ;; a bit ugly, so work harder and only add it if there are no guards
+        ;; coming from other places like extended formals, type declarations,
+        ;; etc.
+        (or (consp formal-types)
+            (consp formal-guards)
+            (assoc :guard kwd-alist)
+            (assoc :guard embedded-xargs-alist)
+            ;; BOZO eventually could also look for type declarations in
+            ;; the traditional decls/docs
+            ))
+
        (main-def
-        `(,defun-sym ,name-fn ,formal-names
+        `(;; Historically we used defund unless the function was enabled-p.
+          ;; But this ran afoul of Issue 464 for DEFINE forms that were
+          ;; embedded in DEFINES.  It seems simpler and easier to just handle
+          ;; the enabling or disabling later, since DEFUND isn't really
+          ;; anything special.  So, we now just always use DEFUN instead.
+          defun
+          ,name-fn ,formal-names
 
 ; Subtle: this order isn't what we always used, but Sol ran into some problems
 ; where, e.g., traditional type declarations weren't coming before the guards
 ; from formals, and therefore the guards wouldn't verify.  We now try to use an
 ; order that seems like it is most probably the one you want.
 
+; 0. If using t-proof, put its :by hint here to try to make sure it happens
+; first.
+
+           ,@(and t-proof
+                  `((declare (xargs :hints ('(:by ,t-proof-name))))))
+
 ; 1. Stobj names, since they give us stobj-p guards, which may be useful and
 ; probably can't depend on anything else
            ,@(and stobj-names
@@ -917,16 +1275,14 @@ some kind of separator!</p>
 ; don't have further dependencies, e.g., don't rely on the top-level :guard
 
            ,@(cond ((atom formal-guards)
-                    ;; Design decision: I prefer to put in a declaration here
-                    ;; instead of leaving it out.  This makes define trigger
-                    ;; guard verification even with eagerness 1.  I think I
-                    ;; much more frequently have guards of T than want to not
-                    ;; verify guards.
-                    `((declare (xargs :guard t))))
+                    nil)
                    ((atom (cdr formal-guards))
                     `((declare (xargs :guard ,(car formal-guards)))))
                    (t
-                    `((declare (xargs :guard (and . ,formal-guards))))))
+                    ;; Sort the guards by putting those that we can determine
+                    ;; to be dependent on only 1 variable first.
+                    (b* ((sorted-formal-guards (sort-formal-guards formal-guards world)))
+                      `((declare (xargs :guard (and . ,sorted-formal-guards)))))))
 
 ; 4. This is kind of arbitrary.  We put the traditional decls before the top-level
 ; xargs because it seems rather unlikely that someone would write
@@ -948,6 +1304,10 @@ some kind of separator!</p>
            ,@(and xargs
                   `((declare (xargs . ,xargs))))
 
+           ;; Just in case there is nothing else to provoke guard verification:
+           ,@(and (not guard-verification-will-happen-anyway-p)
+                  `((declare (xargs :guard t))))
+
            ,final-body
            )))
 
@@ -961,6 +1321,7 @@ some kind of separator!</p>
                   :formals     formals
                   :rest-events (xdoc::make-xdoc-fragments rest-events)
                   :t-proof     (if t-proof (cons t-proof-name t-hints) nil)
+                  :pe-entry    pe-entry
                   )))
 
 (defun add-signature-from-guts (guts)
@@ -990,7 +1351,7 @@ some kind of separator!</p>
 
 (defun events-from-guts (guts world)
   (b* (((defguts guts) guts)
-
+       (enabled-p  (getarg :enabled        nil guts.kwd-alist))
        (prepwork   (getarg :prepwork       nil guts.kwd-alist))
        (short      (getarg :short          nil guts.kwd-alist))
        (long       (getarg :long           nil guts.kwd-alist))
@@ -1044,6 +1405,22 @@ some kind of separator!</p>
                    (make-define-ret-patbinder ',guts (w state)))))
 
          (local
+          ;; [Jared] Previously, when we sometimes used DEFUND and DEFUN, this
+          ;; was a sensible way to ensure that the function was always enabled
+          ;; during the processing of the /// section.  You might think (and I
+          ;; originally DID think) that, now that we always use DEFUN, that
+          ;; this would be unnecessary.  However, in rare cases, for instance,
+          ;; if we are redundantly DEFINEing a function that was originally
+          ;; DEFINEd before now, the function may NOT be enabled even though we
+          ;; have introduced it with DEFUN, because you can do something like
+          ;;
+          ;;    (defun foo ...)
+          ;;    (in-theory (disable foo))
+          ;;    (defun foo ...) ;; redundantly
+          ;;
+          ;; And at this point FOO is still disabled.  So, this is a bummer
+          ;; because enabling/disabling is expensive, but at least we can keep
+          ;; this local...
           (make-event
            (if (logic-mode-p ',guts.name-fn (w state))
                '(in-theory (enable ,guts.name))
@@ -1056,6 +1433,13 @@ some kind of separator!</p>
                        `(with-output :stack :pop (progn . ,events))
                      '(value-triple :invisible)))))
 
+         ;; BOZO using name-fn here is kind of weird, but otherwise we see ugly
+         ;; output when there are macro arguments involved because ACL2 shows us
+         ;; our nice output and then says, "oh but this is really a macro" and
+         ;; then shows us the ugly macro.  But targeting name-fn instead seems
+         ;; to do the right thing.
+         (acl2::extend-pe-table ,guts.name-fn ,guts.pe-entry)
+
          ,@(and guts.rest-events
                 `((with-output :stack :pop
                     (progn
@@ -1064,16 +1448,24 @@ some kind of separator!</p>
          ,@(and hook-specs
                 `((value-triple (cw "; Running post-define hooks.~%"))
                   .
-                  ,(post-hook-make-events hook-specs hooks-alist guts))))
+                  ,(post-hook-make-events hook-specs hooks-alist guts)))
+
+         ,@(if prognp
+               `((set-define-current-function nil))
+             nil)
+
+         ,@(if enabled-p
+               nil
+             `((make-event
+                (if (logic-mode-p ',guts.name-fn (w state))
+                    '(in-theory (disable ,guts.name))
+                  '(value-triple :invisible))))))
+
 
        ;; Now that the section has been submitted, its xdoc exists, so we can
        ;; do the doc generation and prepend it to the xdoc.
        ,(add-signature-from-guts guts)
 
-       ,@(if prognp
-             `((set-define-current-function nil))
-           nil)
-
        (make-event (list 'value-triple
                          (if (eql ,start-max-absolute-event-number
                                   (acl2::max-absolute-event-number
@@ -1391,10 +1783,11 @@ the @(':pre-bind') argument accepts a list of @(see b*) bindings that occur
 before the binding of the return values.  You may also just want to not share
 names between your formals and returns.</p>")
 
-(defun defret-fn (name args world)
+(defun defret-fn (name args disablep world)
   (b* ((__function__ 'defret)
        ((mv kwd-alist args)
-        (extract-keywords `(defret ,name) '(:hyp :fn :hints :rule-classes :pre-bind)
+        (extract-keywords `(defret ,name) '(:hyp :fn :hints :rule-classes :pre-bind
+                                            :otf-flg)
                           args nil))
        ((unless (consp args))
         (raise "No body"))
@@ -1422,19 +1815,24 @@ names between your formals and returns.</p>")
                   (t (cdr hyp?))))
        (rule-classes? (assoc :rule-classes kwd-alist))
        (hints? (assoc :hints kwd-alist))
+       (otf-flg? (assoc :otf-flg kwd-alist))
        (pre-bind (cdr (assoc :pre-bind kwd-alist)))
        (concl `(b* (,@pre-bind ,@binding) ,concl-term))
        (thm (if hyp?
                 `(implies ,hyp ,concl)
               concl)))
-    `(defthm ,name
+    `(,(if disablep 'defthmd 'defthm) ,name
        ,thm
        ,@(and hints?        `(:hints ,(cdr hints?)))
+       ,@(and otf-flg?      `(:otf-flg ,(cdr otf-flg?)))
        ,@(and rule-classes? `(:rule-classes ,(cdr rule-classes?))))))
 
 
 (defmacro defret (name &rest args)
-  `(make-event (defret-fn ',name ',args (w state))))
+  `(make-event (defret-fn ',name ',args nil (w state))))
+
+(defmacro defretd (name &rest args)
+  `(make-event (defret-fn ',name ',args t (w state))))
 
 
 
