@@ -106,7 +106,7 @@ At each atom in the tree:
   @({
     (:@proj <subtemplates-name> subforms)
    })
-  or 
+  or
   @({
     (:@append <subtemplates-name> subforms)
    })
@@ -231,7 +231,7 @@ defthm would disappear.</p>
       (intern-in-package-of-symbol
        str (if (keywordp sym)
                sym
-             (or pkg? pkg-sym))))))
+             (or pkg? pkg-sym sym))))))
 
 (defun tmpl-sym-tree-sublis (alist tree pkg-sym)
   (declare (xargs :mode :program))
