@@ -1,5 +1,5 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2011 Centaur Technology
+; Copyright (C) 2009-2015 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -879,10 +879,10 @@
   (str::rchars-to-string (transform-code-segments x 0 (length x) nil)))
 
 #||
- (transform-code 
-"<p>This is 
+ (transform-code
+"<p>This is
 some regular text</p>
-<code>     
+<code>
     blah1
     <blah>blah2</blah>
     blah3
@@ -1081,7 +1081,14 @@ baz
           (mv nil sexpr)))
        ((mv err vals state) (acl2::unsound-eval sexpr))
        ((when err)
-        (mv (str::cat "Error: failed to evaluate @(`...`): " err)
+        (mv (str::cat "Error: failed to evaluate @(`...`): "
+                      ;; BOZO this isn't right, we really want something like
+                      ;; ~@, but we don't have that unless we use ACL2's
+                      ;; built-in fmt-to-string stuff, which I don't want to
+                      ;; use due to the problems described in
+                      ;; fmt-to-str-orig.lisp...  For now, we can at least just
+                      ;; stringify the error in a dumb way.
+                      (str::pretty err :config (str::make-printconfig :home-package base-pkg)))
             acc
             state))
        (ret (cond ((atom vals)
@@ -1121,7 +1128,7 @@ baz
              kind (str::pretty sexpr))
         acc state)))
 
-(defun preprocess-eval (str topics-fal context base-pkg kpa state acc)
+(defun preprocess-eval (str context topics-fal base-pkg kpa state acc)
   "Returns (MV ACC STATE)"
   (b* (((mv errmsg sexpr state) (preprocess-eval-parse str base-pkg state))
        ((when errmsg)
