@@ -1,5 +1,5 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2014 Centaur Technology
+; Copyright (C) 2009-2015 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -36,7 +36,7 @@
 ; time of top.lisp.
 
 (in-package "XDOC")
-(include-book "import-acl2doc")  ;; For base acl2 documentation
+(include-book "top")
 
 (defxdoc xdoc
   :parents (documentation)
@@ -59,8 +59,8 @@ href='http://dx.doi.org/10.4204/EPTCS.152.2'>Industrial Strength Documentation
 for ACL2</a>.  ACL2 Workshop 2014.  EPTCS 152.  Pages 9-25.
 </blockquote>
 
-<p>See @(see acl2::|Building the ACL2+Books Manual|) for information on
-building the ACL2+Books manual.</p>
+<p>See <i>Building the Manual</i> under @(see acl2::books-certification) for
+information on building the ACL2+Books manual.</p>
 
 
 <p>To use XDOC to document your own books, the first step is:</p>
@@ -102,6 +102,11 @@ developing your book.  Afterward, each @(see defxdoc) form you submit will be
 immediately shown at the terminal, giving you a quick, text-mode preview that
 may help you to diagnose any markup problems.</p>")
 
+(defxdoc missing-parents-test
+  :short "A topic with no @(':parents')."
+  :long "<p>This topic has no @(':parents') and is just meant to ensure that
+@(see missing-parents) is working correctly.</p>")
+
 (local (set-default-parents xdoc))
 
 (defxdoc defxdoc
@@ -541,7 +546,14 @@ name, it may or may not include the package portion of @('name').</li>
 </ul>
 
 <p>You can use these to write your own @('<see>') tags.  You should probably
-<i>never</i> write a @('<see>') tag yourself without using @('@(url ...)').</p>
+<i>never</i> write a @('<see>') tag yourself without using @('@(url ...)').
+Some examples:</p>
+
+<ul>
+<li>@('@(url defxdoc)') expands to @(url defxdoc)</li>
+<li>@('@(sym defxdoc)') expands to @(sym defxdoc)</li>
+<li>@('@(csym defxdoc)') expands to @(csym defxdoc)</li>
+</ul>
 
 <h3>Escaping of @</h3>
 
@@ -611,7 +623,6 @@ manual with others, you should read about @(see deploying-manuals).</li>
 
 @({
     (save <target-dir>
-          [:import    import]    ;; default is t
           [:redef-okp bool]      ;; default is nil
           [:zip-p     bool]      ;; default is t
           )
@@ -631,24 +642,16 @@ overwritten</color>.</li>
 
 <h3>Avoiding Unwanted Documentation</h3>
 
-<p>By default, the @('save') command will automatically import:</p>
-<ul>
-<li>the documentation for the ACL2 theorem prover</li>
-<li>any @(see defdoc) style documentation from books you have loaded</li>
-<li>the documentation for @('xdoc') itself.</li>
-</ul>
+<p>By default, the @('save') command will generate a manual that covers the
+documentation for all books that you have loaded.  This usually works well
+as long as you know all of the books that you need to include.</p>
 
-<p>You can turn off <b>most</b> of this by setting @(':import nil') in your
-@('save') command.</p>
-
-<p>However, you may find that even after setting @(':import nil'), some
-extraneous documentation is still being included!  For instance, you may find
+<p>One caveat is that @('xdoc/save') includes some supporting books that are,
+themselves, documented.  Accordingly, you may find that your manual includes
 documentation from libraries like @(see acl2::std/strings) and @(see
-oslib::oslib) in your output.</p>
-
-<p>This is because @('xdoc/save') includes some supporting books that are,
-themselves, documented.  If you really want precise control over what goes into
-your manual, then, you may want to do something like this:</p>
+oslib::oslib) in your output even if you haven't loaded these libraries
+yourself.  If you really want precise control over what goes into your manual,
+then, you may want to do something like this:</p>
 
 @({
  ;; nothing-extra-manual.lisp - manual with nothing extra
@@ -670,7 +673,7 @@ your manual, then, you may want to do something like this:</p>
    :short \"My Manual\"
    :long \"<p>This manual explains how to use my books...</p>\")
 
- (xdoc::save \"./mylib-manual\" :import nil)
+ (xdoc::save \"./mylib-manual\")
 })")
 
 
@@ -1174,14 +1177,34 @@ not introduce a new local scope, but a @('defsection') does.</p>")
 
 
 (defxdoc undocumented
-  :short "Placeholder for undocumented topics.")
+  :short "Placeholder for documentation topics that lack good @(':parents')."
 
-(defxdoc unselected-parents
-  :short "Placeholder for topics with bad parents."
-  :long "<p>These are topics that we'd like to place somewhere other than, for
-  example, in `acl2-built-ins`.  See <a
-  href=\"https://github.com/acl2/acl2/issues/316\">Github Issue #316</a> for
-  more info.</p>")
+  :long "<p>Many @(see acl2::macros) such as @(see std::deflist) and other
+@(see acl2::std/util) macros can automatically generate @(see xdoc)
+documentation topics.</p>
+
+<p>When someone uses these macros but doesn't give any @(':parents') for the
+resulting documentation, we put the resulting documentation here.  This seems
+better than just dropping the documentation completely, since at least you can
+at least see the boilerplate documentation and any embedded documentation that
+the programmer did provide.</p>
+
+<p>As ongoing documentation improvement work, most topics here can benefit from
+being given more appropriate @(':parents').</p>")
+
+(defxdoc missing-parents
+  :short "Placeholder for documentation topics that lack @(':parents')."
+
+  :long "<p>If a @(see defxdoc) form ends up having no parents, it ends up
+being put here.  See also @(see undocumented).</p>
+
+<p>Historic note.  We used to put these topics directly underneath @(see top)
+instead.  But we found that during development, this sometimes led to a
+strange-looking hierarchy where ``random'' topics were presented as top-level
+topics just because they were new or being moved around or because @(see
+set-default-parents) forms weren't quite in the right places.  To avoid this,
+we now move these topics to @('missing-parents') and print notes about them
+when a manual is saved with @(see xdoc::save).</p>")
 
 (defxdoc set-default-parents
   :short "Set up default parents to use for @(see xdoc)."
