@@ -1,5 +1,5 @@
-; ACL2 Version 7.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2015, Regents of the University of Texas
+; ACL2 Version 7.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2016, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -83,6 +83,30 @@
 (verify-termination-boot-strap packn-pos) ; and guards
 )
 
+(verify-termination-boot-strap read-file-into-string1) ; and guards
+
+(encapsulate
+ ()
+
+; At one the following local lemma seemed to be helpful, but it is not
+; currently necessary.  If we simplify read-file-into-string2, for example by
+; removing ec-call, then perhaps we will need this lemma once again.
+
+;(local
+; (defthm stringp-read-file-into-string1
+;   (implies (car (read-file-into-string1 channel state ans bound))
+;            (stringp (car (read-file-into-string1 channel state ans
+;                                                  bound))))))
+
+ (verify-termination-boot-strap read-file-into-string2) ; and guards
+ )
+
+(verify-termination-boot-strap read-file-into-string) ; and guards
+
+; miscellaneous
+
+(verify-termination-boot-strap guard-or-termination-theorem-msg) ; and guards
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Attachment: too-many-ifs-post-rewrite and too-many-ifs-pre-rewrite
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1061,7 +1085,7 @@
   (declare (xargs :stobjs state
                   :guard (symbolp name)))
   (let ((wrld (w state)))
-    (or (getprop name 'theorem nil 'current-acl2-world wrld)
+    (or (getpropc name 'theorem nil wrld)
         (mv-let (flg prop)
                 (constraint-info name wrld)
                 (cond ((eq prop *unknown-constraints*)
@@ -1217,7 +1241,7 @@
        ((':formula name)
         (meta-extract-formula name st))
        ((':lemma fn n)
-        (let* ((lemmas (getprop fn 'lemmas nil 'current-acl2-world (w st)))
+        (let* ((lemmas (getpropc fn 'lemmas nil (w st)))
                (rule (nth n lemmas)))
 
 ; The use of rewrite-rule-term below relies on the fact that the 'LEMMAS
@@ -1246,6 +1270,20 @@
 ; feature, and will simply trust that functions marked in
 ; *system-verify-guards-alist* can be guard-verified.
 
+; The following commands will check that things are as they should be, after
+; adjusting *system-verify-guards-alist* (see comments there).  Altogether they
+; took only about two minutes on a fast machine in May 2015.
+
+;   (time nice make ACL2_DEVEL=d)
+;   cd books
+;   make clean ACL2=`pwd`/../saved_acl2d
+;   time ./build/cert.pl -j 8 --acl2 `pwd`/../saved_acl2d system/top.cert
+;   cd ..
+;   (time nice make -j 8 devel-check ACL2=`pwd`/saved_acl2d)
+
+; For details, see the comment just above the call of system-verify-guards near
+; the end of this section.
+
 ; A flaw in our approach is that user-supplied guard verifications may depend
 ; on package axioms.  Thus, we view such verifications as strong hints, rather
 ; than as ironclad guarantees that the functions can be guard-verified in
@@ -1265,8 +1303,7 @@
             (cdr fns)
             wrld
             (cons (let* ((fn (car fns))
-                         (justification (getprop fn 'justification nil
-                                                 'current-acl2-world wrld))
+                         (justification (getpropc fn 'justification nil wrld))
                          (ms (and (consp justification) ; for guard
                                   (consp (cdr justification)) ; for guard
                                   (access justification justification :subset))))
@@ -1288,8 +1325,7 @@
           nil))
         ((and (eq (cadar wrld) 'symbol-class)
               (eq (cddar wrld) :COMMON-LISP-COMPLIANT)
-              (getprop (caar wrld) 'predefined nil 'current-acl2-world
-                       installed-wrld))
+              (getpropc (caar wrld) 'predefined nil installed-wrld))
          (new-verify-guards-fns1 (cdr wrld)
                                  installed-wrld
                                  (cons (caar wrld) acc)))
@@ -1326,8 +1362,14 @@
 ; executable; only the books in the keys of this alist need to be certified.
 
   '(("system/top"
+     (>=-LEN X)
+     (ALIST-TO-DOUBLETS ALIST)
+     (ALL->=-LEN LST)
      (ARGLISTP)
      (ARGLISTP1 LST)
+     (ARITIES-OKP USER-TABLE)
+     (ARITY)
+     (COLLECT-BY-POSITION FULL-DOMAIN)
      (CONS-TERM1-MV2)
      (DUMB-NEGATE-LIT)
      (FETCH-DCL-FIELD)
@@ -1340,13 +1382,16 @@
      (LEGAL-CONSTANTP1)
      (LEGAL-VARIABLE-OR-CONSTANT-NAMEP)
      (LEGAL-VARIABLEP)
+     (MAKE-LAMBDA-APPLICATION)
+     (MERGE-SORT-TERM-ORDER L)
+     (MERGE-TERM-ORDER L2 L1)
      (META-EXTRACT-CONTEXTUAL-FACT)
      (META-EXTRACT-GLOBAL-FACT+)
      (META-EXTRACT-RW+-TERM)
-     #+acl2-legacy-doc (MISSING-FMT-ALIST-CHARS)
-     #+acl2-legacy-doc (MISSING-FMT-ALIST-CHARS1 CHAR-TO-TILDE-S-STRING-ALIST)
      (PLAUSIBLE-DCLSP LST)
      (PLAUSIBLE-DCLSP1 LST)
+     (PLIST-WORLDP-WITH-FORMALS ALIST)
+     (STRIP-CADRS X)
      (STRIP-DCLS LST)
      (STRIP-DCLS1 LST)
      (STRIP-KEYWORD-LIST LST)
@@ -1362,7 +1407,12 @@
      (SUBST-EXPR1 TERM)
      (SUBST-EXPR1-LST ARGS)
      (SUBST-VAR FORM)
-     (SUBST-VAR-LST L))))
+     (SUBST-VAR-LST L)
+     (TERM-LIST-LISTP L)
+     (TERM-LISTP X)
+     (TERM-ORDER)
+     (TERM-ORDER1)
+     (TERMP X))))
 
 (defconst *len-system-verify-guards-alist*
   (length *system-verify-guards-alist*))
@@ -1422,8 +1472,8 @@
         (t (cons-absolute-event-numbers
             (cdr fns-alist)
             wrld
-            (acons (or (getprop (caar fns-alist) 'absolute-event-number nil
-                                'current-acl2-world wrld)
+            (acons (or (getpropc (caar fns-alist) 'absolute-event-number nil
+                                 wrld)
                        (er hard? 'cons-absolute-event-numbers
                            "The 'absolute-event-number property is missing ~
                             for ~x0."
@@ -1462,14 +1512,93 @@
 ; (chk-new-verified-guards i) for each i less than the length of
 ; *system-verify-guards-alist*, in order to check that the effect of
 ; system-verify-guards is sound.  This check is performed by using `make' with
-; target devel-check, for example as follows, where <acl2d> denotes a full
-; pathname for a build of ACL2 using feature :acl2-devel (see comments above
-; for how to make such a build):
-;   (time nice make -j 8 regression-fresh devel-check ACL2=<acl2d>)
+; target devel-check, as shown near the top of this section.
+
 #+(and acl2-loop-only ; Note that make-event can't be called here in raw Lisp.
        (not acl2-devel))
 (system-verify-guards)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Support for system-events
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defmacro system-event (event &optional (book-name '"system/top"))
+
+; (System-event E) expands to (skip-proofs E) during normal builds.  However,
+; for acl2-devel builds (see discussion under the section "Support for
+; system-verify-guards" in boot-strap-pass-2.lisp), we merely add the event to
+; a table, to be checked by Make target devel-check, which invokes function
+; system-verify-skip-proofs for that purpose.
+
+  #+acl2-devel `(table system-event-table ',event ',book-name)
+
+; It is tempting to generate a progn, where the skip-proofs is preceded by:
+
+; (value-triple ,(concatenate 'string "Verified in community book " book-name))
+
+; However, that value-triple event doesn't show up with any of :pe, :pcb, or
+; :pcb!, so we won't bother.
+
+  #-acl2-devel (declare (ignore book-name))
+  #-acl2-devel `(skip-proofs ,event))
+
+(defun system-events-fn (events book-name)
+  (declare (xargs :guard (true-listp events)))
+  (cond ((endp events) nil)
+        (t (cons `(system-event ,(car events) ,book-name)
+                 (system-events-fn (cdr events) book-name)))))
+
+(defmacro system-events (book-name &rest events)
+  (declare (xargs :guard (stringp book-name)))
+  (cons 'progn (system-events-fn events book-name)))
+
+(defun system-include-book-forms (book-names)
+  (declare (xargs :guard (true-listp book-names)))
+  (cond ((endp book-names) nil)
+        (t (cons `(include-book ,(car book-names) :dir :system)
+                 (system-include-book-forms (cdr book-names))))))
+
+(defmacro check-system-events ()
+
+; Executed by "make devel-check".
+
+  `(make-event
+    (let ((event-book-alist (table-alist 'system-event-table (w state))))
+      (cons 'progn
+            (append (system-include-book-forms
+                     (remove-duplicates (strip-cdrs event-book-alist)
+                                        :test 'equal))
+                    '((set-enforce-redundancy t))
+                    (strip-cars event-book-alist)
+                    '((value-triple :CHECK-SYSTEM-EVENTS-SUCCESS)))))))
+
+(system-events "system/termp"
+
+(defthm legal-variable-or-constant-namep-implies-symbolp
+  (implies (not (symbolp x))
+           (not (legal-variable-or-constant-namep x))))
+
+(in-theory (disable legal-variable-or-constant-namep))
+
+(defthm termp-implies-pseudo-termp
+  (implies (termp x w)
+           (pseudo-termp x))
+  :rule-classes (:rewrite :forward-chaining))
+
+(defthm term-listp-implies-pseudo-term-listp
+  (implies (term-listp x w)
+           (pseudo-term-listp x))
+  :rule-classes (:rewrite :forward-chaining))
+
+(defthm arities-okp-implies-arity
+  (implies (and (arities-okp user-table w)
+                (assoc fn user-table))
+           (equal (arity fn w) (cdr (assoc fn user-table)))))
+
+(in-theory (disable arity arities-okp))
+
+)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Memoization
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1617,20 +1746,23 @@
 ; 13018 kaufmann  20   0  512g 664m  36m S   99  2.1   2:15.91 lx86cl64
 ; 13018 kaufmann  20   0  512g 671m  36m S   42  2.1   2:17.16 lx86cl64
 
-; With CCL executable built without start-sol-gc and with EGC on,
-; after the above sequence of unmemoize and memoize:
+; With CCL executable built without start-sol-gc (now called
+; set-gc-strategy-builtin-delay) and with EGC on, after the above sequence of
+; unmemoize and memoize:
 ; Time:  136.47 seconds (prove: 136.45, print: 0.02, other: 0.00)
 ; 13049 kaufmann  20   0  512g  59m  36m S   99  0.2   2:14.93 lx86cl64
 ; 13049 kaufmann  20   0  512g  59m  36m S   96  0.2   2:17.81 lx86cl64
 
-; With CCL executable built without start-sol-gc and with EGC on,
-; after the above unmemoize form:
+; With CCL executable built without start-sol-gc (now called
+; set-gc-strategy-builtin-delay) and with EGC on, after the above unmemoize
+; form:
 ; Time:  86.33 seconds (prove: 86.33, print: 0.01, other: 0.00)
 ; 13178 kaufmann  20   0  512g  58m  35m S  100  0.2   1:27.18 lx86cl64
 ; 13178 kaufmann  20   0  512g  58m  35m S   17  0.2   1:27.70 lx86cl64
 
-; With CCL executable built without start-sol-gc and with EGC on, out of the
-; box except for redefining waterfall1 in raw Lisp so that its body is (prog2$
+; With CCL executable built without start-sol-gc (now called
+; set-gc-strategy-builtin-delay) and with EGC on, out of the box except for
+; redefining waterfall1 in raw Lisp so that its body is (prog2$
 ; (clear-memoize-table 'worse-than-builtin) <old-body>); notice that :forget
 ; remains nil.
 ; Time:  182.61 seconds (prove: 182.58, print: 0.02, other: 0.00)
