@@ -1,5 +1,5 @@
-; ACL2 Version 7.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2015, Regents of the University of Texas
+; ACL2 Version 7.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2016, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -85,7 +85,8 @@
         32-BIT-INTEGERP
         32-BIT-INTEGERP-FORWARD-TO-INTEGERP
         < <-ON-OTHERS <= = > >= ?-FN @ A! ABORT!
-        ABS ACCUMULATED-PERSISTENCE ACL2-COUNT
+        ABS ACCESS ACCUMULATED-PERSISTENCE ACCUMULATED-PERSISTENCE-OOPS
+        ACL2-COUNT
         ACL2-INPUT-CHANNEL-PACKAGE ACL2-NUMBER-LISTP ACL2-NUMBERP
         ACL2-ORACLE ACL2-OUTPUT-CHANNEL-PACKAGE
         ACL2-PACKAGE ACONS ACTIVE-RUNEP
@@ -106,12 +107,12 @@
         ALPHA-CHAR-P-FORWARD-TO-CHARACTERP
         ALPHORDER AND AND-MACRO APPEND
         AREF-32-BIT-INTEGER-STACK AREF-T-STACK
-        AREF1 AREF2 ARGS ARRAY1P ARRAY1P-CONS
+        AREF1 AREF2 ARGS ARITIES-OKP ARITY ARRAY1P ARRAY1P-CONS
         ARRAY1P-FORWARD ARRAY1P-LINEAR
         ARRAY2P ARRAY2P-CONS ARRAY2P-FORWARD
         ARRAY2P-LINEAR ASET-32-BIT-INTEGER-STACK
         ASET-T-STACK ASET1 ASET2
-        ASH ASSERT$ ASSERT-EVENT ASSIGN ASSOC
+        ASH ASSERT$ ASSERT* ASSERT-EVENT ASSIGN ASSOC
         ASSOC-ADD-PAIR ASSOC-EQ ASSOC-EQ-EQUAL
         ASSOC-EQ-EQUAL-ALISTP ASSOC-EQUAL
         ASSOC-KEYWORD ASSOC-STRING-EQUAL ASSOC2
@@ -141,7 +142,7 @@
         CDAAR CDADAR CDADDR CDADR CDAR CDDAAR
         CDDADR CDDAR CDDDAR CDDDDR CDDDR CDDR
         CDR CDR-CONS CDRN CEILING CERTIFY-BOOK
-        CERTIFY-BOOK! CHAR CHAR-CODE
+        CERTIFY-BOOK! CHANGE CHAR CHAR-CODE
         CHAR-CODE-CODE-CHAR-IS-IDENTITY
         CHAR-CODE-LINEAR CHAR-DOWNCASE
         CHAR-EQUAL CHAR-UPCASE CHAR< CHAR<=
@@ -156,7 +157,7 @@
         CHARACTERP CHARACTERP-CHAR-DOWNCASE
         CHARACTERP-CHAR-UPCASE CHARACTERP-NTH
         CHARACTERP-PAGE CHARACTERP-RUBOUT
-        CHARACTERP-TAB CHECK-VARS-NOT-FREE
+        CHARACTERP-TAB CHECK-INVARIANT-RISK CHECK-VARS-NOT-FREE
         CHECKPOINT-FORCED-GOALS CLAUSE
         CLOSE-INPUT-CHANNEL CLOSE-OUTPUT-CHANNEL
         CLOSE-TRACE-FILE CLOSURE CODE-CHAR
@@ -207,20 +208,21 @@
         DEFAULT-DENOMINATOR
         DEFAULT-HINTS DEFAULT-IMAGPART
         DEFAULT-MEASURE-FUNCTION
-        DEFAULT-NUMERATOR DEFAULT-TOTAL-PARALLELISM-WORK-LIMIT
+        DEFAULT-NUMERATOR
         DEFAULT-PRINT-PROMPT DEFAULT-REALPART DEFAULT-RULER-EXTENDERS
         DEFAULT-SYMBOL-NAME
-        DEFAULT-SYMBOL-PACKAGE-NAME
+        DEFAULT-SYMBOL-PACKAGE-NAME DEFAULT-TOTAL-PARALLELISM-WORK-LIMIT
         DEFAULT-UNARY-/ DEFAULT-UNARY-MINUS
         DEFAULT-VERIFY-GUARDS-EAGERNESS
         DEFAULT-WELL-FOUNDED-RELATION
         DEFAXIOM DEFCHOOSE DEFCONG DEFCONST
-        DEFDOC DEFEQUIV DEFEVALUATOR DEFEXEC
+        DEFEQUIV DEFEVALUATOR DEFEXEC
         DEFINE-PC-ATOMIC-MACRO DEFINE-PC-HELP
         DEFINE-PC-MACRO DEFINE-PC-META
         DEFINE-TRUSTED-CLAUSE-PROCESSOR
         DEFLABEL DEFLOCK DEFMACRO DEFMACRO-LAST DEFN DEFND DEFPKG
-        DEFPROXY DEFREFINEMENT DEFSTOBJ DEFSTUB DEFTHEORY DEFTHEORY-STATIC
+        DEFPROXY DEFREC DEFREFINEMENT DEFSTOBJ DEFSTUB
+        DEFTHEORY DEFTHEORY-STATIC
         DEFTHM DEFTHMD DEFTTAG DEFUN DEFUN-INLINE DEFUN-NOTINLINE DEFUN-NX
         DEFUN-SK DEFUND DEFUND-INLINE DEFUND-NOTINLINE DEFUND-NX DEFUNS
         DELETE-ASSOC DELETE-ASSOC-EQ DELETE-ASSOC-EQUAL
@@ -241,32 +243,37 @@
         EQLABLE-LISTP-FORWARD-TO-ATOM-LISTP
         EQLABLEP EQLABLEP-RECOG EQUAL
         EQUAL-CHAR-CODE ER ER-PROGN ER-PROGN-FN
-        ER-PROGN@PAR ER-PROGN-FN@PAR
+        ER-PROGN-FN@PAR ER-PROGN@PAR
         EVENP EVENS EVENT EVISC-TUPLE
         EXECUTABLE-COUNTERPART-THEORY
-        EXIT EXPLODE-ATOM
+        EXISTS EXIT EXPLODE-ATOM
         EXPLODE-NONNEGATIVE-INTEGER EXPT
         EXPT-TYPE-PRESCRIPTION-NON-ZERO-BASE
         EXTEND-32-BIT-INTEGER-STACK
-        EXTEND-T-STACK
+        EXTEND-PE-TABLE EXTEND-T-STACK
         EXTEND-WORLD EXTRA-INFO F-GET-GLOBAL F-PUT-GLOBAL
         FAST-ALIST-CLEAN FAST-ALIST-CLEAN! FAST-ALIST-FORK FAST-ALIST-FORK!
-        FAST-ALIST-FREE-ON-EXIT FERTILIZE FC-REPORT
+        FAST-ALIST-FREE-ON-EXIT FC-REPORT FERTILIZE
         FGETPROP FIFTH FILE-CLOCK FILE-CLOCK-P
         FILE-CLOCK-P-FORWARD-TO-INTEGERP FINALIZE-EVENT-USER
         FIRST FIRST-N-AC
         FIX FIX-TRUE-LIST FLET FLOOR FLUSH-COMPRESS
-        FMS FMS! FMT FMT! FMT-TO-COMMENT-WINDOW FMT1 FMT1!
-        FMS-TO-STRING FMS!-TO-STRING FMT-TO-STRING FMT!-TO-STRING
-        FMT1-TO-STRING FMT1!-TO-STRING
-        FNCALL-TERM FORCE FOURTH FUNCTION-SYMBOLP
-        FUNCTION-THEORY GAG-MODE GC$ GC-VERBOSE GENERALIZE GET-COMMAND-SEQUENCE
-        GET-GLOBAL GET-OUTPUT-STREAM-STRING$ GET-TIMER GET-WORMHOLE-STATUS
-        GCS GETENV$ GETPROP GETPROP-DEFAULT GETPROPS
+        FMS FMS!
+        FMS!-TO-STRING FMS-TO-STRING
+        FMT FMT! FMT!-TO-STRING
+        FMT-TO-COMMENT-WINDOW FMT-TO-STRING FMT1 FMT1!
+        FMT1!-TO-STRING FMT1-TO-STRING
+        FNCALL-TERM FORALL FORCE FORMULA FOURTH FUNCTION-SYMBOLP
+        FUNCTION-THEORY GAG-MODE GC$
+        GC-STRATEGY GC-VERBOSE GCS GENERALIZE
+        GET-CHECK-INVARIANT-RISK GET-COMMAND-SEQUENCE
+        GET-EVENT-DATA GET-GLOBAL GET-OUTPUT-STREAM-STRING$
+        GET-TIMER GET-WORMHOLE-STATUS
+        GETENV$ GETPROP GETPROP-DEFAULT GETPROPC GETPROPS
         GETPROPS1 GLOBAL-TABLE GLOBAL-TABLE-CARS
         GLOBAL-TABLE-CARS1 GLOBAL-VAL
-        GOOD-ATOM-LISTP GOOD-BYE GROUND-ZERO
-        GUARD GUARD-OBLIGATION HARD-ERROR
+        GOOD-ATOM-LISTP GOOD-BYE GROUND-ZERO GTHM
+        GUARD GUARD-OBLIGATION GUARD-THEOREM HARD-ERROR
         HAS-PROPSP HAS-PROPSP1 HEADER HELP HIDE
         HIST ; for computed hints
         HONS-ASSOC-EQUAL HONS-COPY-PERSISTENT
@@ -285,7 +292,7 @@
         INTEGER-ABS INTEGER-IMPLIES-RATIONAL
         INTEGER-LENGTH INTEGER-LISTP
         INTEGER-LISTP-FORWARD-TO-RATIONAL-LISTP
-        INTEGER-STEP INTEGERP INTERN
+        INTEGER-RANGE-P INTEGER-STEP INTEGERP INTERN
         INTERN$ INTERN-IN-PACKAGE-OF-SYMBOL
         INTERN-IN-PACKAGE-OF-SYMBOL-SYMBOL-NAME
         INTERSECTION$ INTERSECTION-EQ
@@ -318,7 +325,7 @@
         LOWER-CASE-P-FORWARD-TO-ALPHA-CHAR-P
         LOWEST-TERMS LP MACRO-ALIASES MACRO-ARGS
         MAIN-TIMER MAIN-TIMER-TYPE-PRESCRIPTION
-        MAKE-CHARACTER-LIST
+        MAKE MAKE-CHARACTER-LIST
         MAKE-CHARACTER-LIST-MAKE-CHARACTER-LIST
         MAKE-EVENT MAKE-FAST-ALIST
         MAKE-FMT-BINDINGS MAKE-INPUT-CHANNEL
@@ -328,7 +335,7 @@
         MAKE-VAR-LST MAKE-VAR-LST1
         MAKE-WORMHOLE-STATUS MAKUNBOUND-GLOBAL
         MAX MAXIMUM-LENGTH MAY-NEED-SLASHES
-        MBE MBT
+        MBE MBT MBT*
         MEMBER MEMBER-EQ MEMBER-EQUAL MEMBER-SYMBOL-NAME
         META-EXTRACT-CONTEXTUAL-FACT META-EXTRACT-FORMULA
         META-EXTRACT-GLOBAL-FACT META-EXTRACT-GLOBAL-FACT+ META-EXTRACT-RW+-TERM
@@ -336,8 +343,8 @@
         MOD MOD-EXPT MONITOR MONITORED-RUNES
         MORE MORE! MORE-DOC MSG MUST-BE-EQUAL
         MUTUAL-RECURSION MUTUAL-RECURSION-GUARDP
-        MV MV? MV-LET MV?-LET MV-LIST
-        MV-NTH NAT-LISTP NATP NEEDS-SLASHES NEVER-MEMOIZE NEWLINE
+        MV MV-LET MV-LIST MV-NTH MV? MV?-LET
+        NAT-LISTP NATP NEEDS-SLASHES NEVER-MEMOIZE NEWLINE
         NFIX NIL NIL-IS-NOT-CIRCULAR NINTH
         NO-DUPLICATESP NO-DUPLICATESP-EQ NO-DUPLICATESP-EQUAL
         NON-EXEC NONNEGATIVE-INTEGER-QUOTIENT
@@ -365,13 +372,13 @@
         OPEN-OUTPUT-CHANNEL-P1
         OPEN-OUTPUT-CHANNELS OPEN-TRACE-FILE
         OR OR-MACRO
-        ORACLE-FUNCALL ORACLE-APPLY ORACLE-APPLY-RAW
+        ORACLE-APPLY ORACLE-APPLY-RAW ORACLE-FUNCALL
         ORDERED-SYMBOL-ALISTP
         ORDERED-SYMBOL-ALISTP-ADD-PAIR
         ORDERED-SYMBOL-ALISTP-ADD-PAIR-FORWARD
+        ORDERED-SYMBOL-ALISTP-DELETE-ASSOC-EQ
         ORDERED-SYMBOL-ALISTP-FORWARD-TO-SYMBOL-ALISTP
         ORDERED-SYMBOL-ALISTP-GETPROPS
-        ORDERED-SYMBOL-ALISTP-DELETE-ASSOC-EQ
         OTHERWISE
         OUR-DIGIT-CHAR-P OVERRIDE-HINTS
         P! PAIRLIS$ PAIRLIS2 PAND PARGS
@@ -385,7 +392,7 @@
         POSITION-EQUAL POSITION-EQUAL-AC
         POSITIVE POSP POWER-EVAL PPROGN PR
         PR! PREPROCESS PRIN1$ PRIN1-WITH-SLASHES
-        PRIN1-WITH-SLASHES1 PRINC$ PRINT-GV
+        PRIN1-WITH-SLASHES1 PRINC$ PRINT-BASE-P PRINT-GV
         PRINT-OBJECT$ PRINT-OBJECT$-SER PRINT-RATIONAL-AS-DECIMAL
         PRINT-TIMER PROFILE
         PROG2$ PROGN PROGN! PROGN$ PROGRAM
@@ -398,7 +405,7 @@
         PUT-ASSOC PUT-ASSOC-EQ PUT-ASSOC-EQL
         PUT-ASSOC-EQUAL PUT-GLOBAL PUTPROP
         QUICK-AND-DIRTY-SUBSUMPTION-REPLACEMENT-STEP
-        QUIT QUOTE QUOTEP R-SYMBOL-ALISTP R-EQLABLE-ALISTP
+        QUIT QUOTE QUOTEP R-EQLABLE-ALISTP R-SYMBOL-ALISTP
         RANDOM$ RASSOC RASSOC-EQ RASSOC-EQUAL
         RATIO RATIONAL RATIONAL-IMPLIES1
         RATIONAL-IMPLIES2 RATIONAL-LISTP
@@ -409,7 +416,7 @@
         RATIONALP-UNARY--
         RATIONALP-UNARY-/ READ-ACL2-ORACLE
         READ-ACL2-ORACLE-PRESERVES-STATE-P1
-        READ-BYTE$ READ-CHAR$ READ-FILE-LISTP
+        READ-BYTE$ READ-CHAR$ READ-FILE-INTO-STRING READ-FILE-LISTP
         READ-FILE-LISTP-FORWARD-TO-TRUE-LIST-LISTP
         READ-FILE-LISTP1
         READ-FILE-LISTP1-FORWARD-TO-TRUE-LISTP-AND-CONSP
@@ -426,8 +433,9 @@
         REAL/RATIONALP REALFIX REALPART
         REALPART-COMPLEX REALPART-IMAGPART-ELIM
         REBUILD REDEF REDEF!
-        REDEF+ REDEF- REDO-FLAT REM REMOVE
+        REDEF+ REDEF- REDO-FLAT
         REGENERATE-TAU-DATABASE
+        REM REMOVE
         REMOVE-BINOP REMOVE-CUSTOM-KEYWORD-HINT
         REMOVE-DEFAULT-HINTS
         REMOVE-DEFAULT-HINTS!
@@ -444,7 +452,7 @@
         RESIZE-LIST REST RESTORE-MEMOIZATION-SETTINGS RETRACT-WORLD
         RETRIEVE RETURN-LAST RETURN-LAST-TABLE
         REVAPPEND REVERSE REWRITE-STACK-LIMIT
-        RFIX ROUND SATISFIES
+        RFIX ROUND RW-CACHE SATISFIES
         SAVE-AND-CLEAR-MEMOIZATION-SETTINGS SAVE-EXEC
         SEARCH SECOND
         SERIALIZE-READ SERIALIZE-WRITE SET-ABSSTOBJ-DEBUG
@@ -453,21 +461,24 @@
         SET-BODY SET-BOGUS-DEFUN-HINTS-OK
         SET-BOGUS-MUTUAL-RECURSION-OK
         SET-CASE-SPLIT-LIMITATIONS
-        SET-CBD SET-CHECKPOINT-SUMMARY-LIMIT
+        SET-CBD
+        SET-CHECK-INVARIANT-RISK
+        SET-CHECKPOINT-SUMMARY-LIMIT
         SET-COMPILE-FNS
         SET-COMPILER-ENABLED SET-DEBUGGER-ENABLE
         SET-DEFAULT-BACKCHAIN-LIMIT
         SET-DEFAULT-HINTS SET-DEFAULT-HINTS!
         SET-DEFERRED-TTAG-NOTES
-        SET-DIFFERENCE-EQ SET-DIFFERENCE-EQUAL SET-DIFFERENCE$
+        SET-DIFFERENCE$ SET-DIFFERENCE-EQ SET-DIFFERENCE-EQUAL
         SET-DIFFERENCE-THEORIES
+        SET-DUPLICATE-KEYS-ACTION
+        SET-DUPLICATE-KEYS-ACTION!
         SET-ENFORCE-REDUNDANCY
         SET-EQUALP-EQUAL SET-EVISC-TUPLE
         SET-FC-CRITERIA SET-FC-REPORT-ON-THE-FLY
         SET-FMT-HARD-RIGHT-MARGIN
         SET-FMT-SOFT-RIGHT-MARGIN
-        SET-GAG-MODE SET-GUARD-CHECKING
-        #+acl2-legacy-doc SET-IGNORE-DOC-STRING-ERROR
+        SET-GAG-MODE SET-GC-STRATEGY SET-GUARD-CHECKING SET-GUARD-MSG
         SET-IGNORE-OK SET-INHIBIT-OUTPUT-LST
         SET-INHIBIT-WARNINGS SET-INHIBIT-WARNINGS!
         SET-INHIBITED-SUMMARY-TYPES
@@ -484,27 +495,29 @@
         SET-NON-LINEARP
         SET-OVERRIDE-HINTS SET-OVERRIDE-HINTS!
         SET-PARALLEL-EXECUTION
-        SET-PRINT-BASE SET-PRINT-CASE
+        SET-PRINT-BASE SET-PRINT-BASE-RADIX SET-PRINT-CASE
         SET-PRINT-CIRCLE SET-PRINT-CLAUSE-IDS
-        SET-PRINT-ESCAPE SET-PRINT-LENGTH
+        SET-PRINT-ESCAPE SET-PRINT-GV-DEFAULTS SET-PRINT-LENGTH
         SET-PRINT-LEVEL SET-PRINT-LINES
         SET-PRINT-RADIX SET-PRINT-READABLY
         SET-PRINT-RIGHT-MARGIN SET-PROVER-STEP-LIMIT SET-RAW-MODE
-        SET-RAW-MODE-ON! SET-RAW-PROOF-FORMAT
+        SET-RAW-MODE-ON! SET-RAW-PROOF-FORMAT SET-RAW-WARNING-FORMAT
         SET-REWRITE-STACK-LIMIT
         SET-RULER-EXTENDERS
         SET-RW-CACHE-STATE SET-RW-CACHE-STATE!
         SET-SAVED-OUTPUT SET-SERIALIZE-CHARACTER
+        SET-SKIP-META-TERMP-CHECKS SET-SKIP-META-TERMP-CHECKS!
         SET-SPLITTER-OUTPUT SET-STATE-OK
         SET-TAU-AUTO-MODE SET-TIMER
         SET-TOTAL-PARALLELISM-WORK-LIMIT
         SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR
         SET-TRACE-EVISC-TUPLE SET-VERIFY-GUARDS-EAGERNESS
-        SET-W SET-WELL-FOUNDED-RELATION
+        SET-W
         SET-WATERFALL-PARALLELISM
         SET-WATERFALL-PARALLELISM-HACKS-ENABLED
         SET-WATERFALL-PARALLELISM-HACKS-ENABLED!
         SET-WATERFALL-PRINTING
+        SET-WELL-FOUNDED-RELATION
         SET-WORMHOLE-DATA
         SET-WORMHOLE-ENTRY-CODE SET-WRITE-ACL2X SETENV$ SEVENTH
         SGETPROP SHOW-ACCUMULATED-PERSISTENCE
@@ -512,7 +525,7 @@
         SHOW-FC-CRITERIA SHRINK-32-BIT-INTEGER-STACK
         SHRINK-T-STACK
         SIGNED-BYTE SIGNED-BYTE-P SIGNUM SIMPLIFY
-        SIXTH SKIP-PROOFS SOME-SLASHABLE SPEC-MV-LET
+        SIXTH SKIP-PROOFS SLEEP SOME-SLASHABLE SPEC-MV-LET
         SPLITTER-OUTPUT STABLE-UNDER-SIMPLIFICATIONP
         STANDARD-CHAR STANDARD-CHAR-LISTP
         STANDARD-CHAR-LISTP-APPEND
@@ -560,10 +573,13 @@
         T T-STACK T-STACK-LENGTH T-STACK-LENGTH1
         TABLE TABLE-ALIST TAKE
         TAU-DATA TAU-DATABASE
-        TAU-INTERVAL-DOM TAU-INTERVAL-LO TAU-INTERVAL-LO-REL
-        TAU-INTERVAL-HI TAU-INTERVAL-HI-REL TAU-INTERVALP
+        TAU-INTERVAL-DOM
+        TAU-INTERVAL-HI TAU-INTERVAL-HI-REL
+        TAU-INTERVAL-LO TAU-INTERVAL-LO-REL
+        TAU-INTERVALP
         TAU-STATUS TAU-SYSTEM
-        TENTH TERM-ORDER THE THE-CHECK THE-FIXNUM
+        TENTH TERM-LIST-LISTP TERM-LISTP TERM-ORDER TERMINATION-THEOREM TERMP
+        THE THE-CHECK THE-FIXNUM
         THE-FIXNUM! THEORY THEORY-INVARIANT
         THIRD THM TIME$ TIME-TRACKER TIME-TRACKER-TAU TIMER-ALISTP
         TIMER-ALISTP-FORWARD-TO-TRUE-LIST-LISTP-AND-SYMBOL-ALISTP
@@ -575,11 +591,11 @@
         TRUE-LISTP
         TRUE-LISTP-CADR-ASSOC-EQ-FOR-OPEN-CHANNELS-P
         TRUE-LISTP-UPDATE-NTH
-        TRUNCATE TTAGS-SEEN TYPE
+        TRUNCATE TTAG TTAGS-SEEN TTHM TYPE
         TYPED-IO-LISTP TYPED-IO-LISTP-FORWARD-TO-TRUE-LISTP
         TYPESPEC-CHECK
         U UBT UBT!
-        UBT-PREHISTORY UBU UBU! UNARY-- UNARY-/
+        UBT-PREHISTORY UBT? UBU UBU! UBU? UNARY-- UNARY-/
         UNARY-FUNCTION-SYMBOL-LISTP UNICITY-OF-0
         UNICITY-OF-1 UNION$ UNION-EQ UNION-EQUAL
         UNION-THEORIES UNIVERSAL-THEORY
@@ -591,8 +607,8 @@
         UPDATE-BIG-CLOCK-ENTRY UPDATE-FILE-CLOCK
         UPDATE-GLOBAL-TABLE UPDATE-IDATES
         UPDATE-LIST-ALL-PACKAGE-NAMES-LST
-        UPDATE-NTH UPDATE-OPEN-INPUT-CHANNELS
-        UPDATE-OPEN-OUTPUT-CHANNELS
+        UPDATE-NTH UPDATE-NTH-ARRAY
+        UPDATE-OPEN-INPUT-CHANNELS UPDATE-OPEN-OUTPUT-CHANNELS
         UPDATE-READ-FILES
         UPDATE-T-STACK UPDATE-USER-STOBJ-ALIST
         UPDATE-USER-STOBJ-ALIST1
@@ -604,7 +620,8 @@
         VERIFY-GUARDS VERIFY-GUARDS+ VERIFY-GUARDS-FORMULA
         VERIFY-TERMINATION W WALKABOUT
         WARNING! WATERFALL-PARALLELISM WATERFALL-PRINTING
-        WET WITH-FAST-ALIST WITH-GUARD-CHECKING
+        WET WITH-FAST-ALIST
+        WITH-GUARD-CHECKING WITH-GUARD-CHECKING-ERROR-TRIPLE
         WITH-LIVE-STATE WITH-LOCAL-STATE WITH-LOCAL-STOBJ WITH-OUTPUT
         WITH-OUTPUT-LOCK
         WITH-PROVER-STEP-LIMIT WITH-PROVER-STEP-LIMIT!
