@@ -1,5 +1,5 @@
-; ACL2 Version 7.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2015, Regents of the University of Texas
+; ACL2 Version 7.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2016, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -814,9 +814,10 @@
     (if timeout received-signal t))
   #+(and sb-thread sbcl-sno-patch)
   (if timeout
-      (sb-thread:wait-on-semaphore semaphore
-                                   :timeout timeout
-                                   :notification notification)
+      (and (sb-thread:wait-on-semaphore semaphore
+                                        :timeout timeout
+                                        :notification notification)
+           t)
     (progn (sb-thread:wait-on-semaphore semaphore
                                         :timeout timeout
                                         :notification notification)
