@@ -1,5 +1,5 @@
-; ACL2 Version 7.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2015, Regents of the University of Texas
+; ACL2 Version 7.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2016, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -373,31 +373,6 @@
                       (change-pc-state pc-state :instruction (make-pretty-pc-instr ,instr))))
                state)))
 
-#+acl2-legacy-doc
-(defun add-pc-doc-header (command-type str)
-  (declare (xargs :guard (and (stringp command-type)
-                              (stringp str))))
-  (string-append
-   ":Doc-Section ACL2::Proof-checker-commands
-
-"
-   (string-append (string-append command-type "
-")
-                  str)))
-
-#+acl2-legacy-doc
-(defun remove-doc (command-type body)
-  ;; puts in doc if there isn't any, and puts the appropriate header on
-  (declare (xargs :guard (stringp command-type)))
-  (if (and (consp body) (consp (cdr body)) (stringp (car body)))
-      (mv (add-pc-doc-header command-type (car body)) (cdr body))
-    (mv nil body)))
-
-#-acl2-legacy-doc ; Delete this and its uses when finally excising legacy doc:
-(defun remove-doc (command-type body)
-  (declare (ignore command-type))
-  (mv nil body))
-
 (defun pc-primitive-defun-form (raw-name name formals doc body)
   `(defun ,name (args state)
      ;; notice that args aren't ignored, since even if they're nil, they're
@@ -1057,8 +1032,7 @@
                     (pc-single-step-primitive instr state)))
            (meta
             (cond ((and (not (f-get-global 'in-verify-flg state))
-                        (not (getprop (car instr) 'predefined nil
-                                      'current-acl2-world (w state))))
+                        (not (getpropc (car instr) 'predefined nil (w state))))
                    (er soft 'proof-checker
                        "You may only invoke a user-defined proof-checker meta ~
                         command, such as ~x0, when you are inside the ~
